@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -306,6 +306,7 @@ int main(int argc, char* argv[]) {
 #endif
 
 #ifdef HAVE_PYTHON
+  PythonScript::initPythonHome( argv[0] );
   PythonScript::setargv( argc, argv );
 #endif
   // Settings and command line arguments ---
@@ -336,15 +337,22 @@ int main(int argc, char* argv[]) {
   help_message += " -M --no-mirror         Suppress flipping screen\n";
   help_message += " -f --fullscreen        Open in fullscreen mode\n";
   help_message += " -F --no-fullscreen     Suppress fullscreen mode\n";
-  help_message += "    --screen=<W>x<H>    Set width and height for window\n";
+  help_message += "    --screen=<W>x<H>    Set width and height for the opengl rendering area.\n";
+  help_message += "                        Note that the operating system might limit the size\n";
+  help_message += "                        range due to captions and borders.\n";
   help_message += "\n";
   help_message += "    --deviceinfo=<file> Use <file> as device info\n";
   help_message += "    --stylus=<file>     Use <file> as stylus model\n";
   help_message += "    --viewpoint=<file>  Use <file> as viewpoint\n";
-  help_message += "    --rendermode=<mode> Use render mode <mode>\n";
+  help_message += "    --rendermode=<mode> Use stereo render mode <mode>\n";
   help_message += " -s --spacemouse        Use 3DConnexion space mouse to\n";
   help_message += "                        navigate scene.\n";
   help_message += "\n";
+#ifdef HAVE_NVAPI
+  help_message += "    --nvsettings=<file> Path to a file that contains a NvidiaSettings\n";
+  help_message += "                        node as it top most node.\n";
+  help_message += "\n";
+#endif
   help_message += " -h --help              This help message\n";
   help_message += "\n";
   help_message += " When the scene is running the following keyBindings can ";
@@ -385,7 +393,7 @@ int main(int argc, char* argv[]) {
   char *r = getenv( "H3D_ROOT" );
   string h3d_root = r ? r : ""; 
 
-#ifdef WIN32
+#ifdef H3D_WINDOWS
   char *home = getenv( "HOMEPATH" );
   string ini_file_path = home ? string(home) + string( "/h3dload.ini" ) :
                                 string("h3dload.ini");
@@ -393,15 +401,17 @@ int main(int argc, char* argv[]) {
   char *home = getenv( "HOME" );
   string ini_file_path = home ? string(home) + string( "/.h3dload.ini" ) :
                                 string(".h3dload.ini");
-#endif
+#endif // H3D_WINDOWS
 
   bool ini_file_exists = false;
-  ifstream check_ini_file( ini_file_path.c_str() );
-  if( check_ini_file.is_open() ) {
-    ini_file_exists = true;
+  {
+    ifstream check_ini_file( ini_file_path.c_str() );
+    if( check_ini_file.is_open() ) {
+      ini_file_exists = true;
+    }
+    check_ini_file.close();
   }
-  check_ini_file.close();
-  
+
   if( !ini_file_exists ){
     ini_file_path = h3d_root + "/settings/h3dload.ini";
     
@@ -427,7 +437,7 @@ int main(int argc, char* argv[]) {
   else if( ostream_str == "cout" )
     console_ostream = &cout;
   else
-    Console(4) << "Invalid value \"" <<  ostream_str
+    Console(LogLevel::Error) << "Invalid value \"" <<  ostream_str
     << "\" on ostream variable. Must be cerr or cout."<< endl;
 
   Console.setShowTime(console_show_time);
@@ -473,7 +483,7 @@ int main(int argc, char* argv[]) {
     else if (strcmp( buffer, "FALSE" ) == 0 ){
       manualCursorControl = false; }
     else
-      Console(4) << "Invalid value \"" << buffer 
+      Console(LogLevel::Error) << "Invalid value \"" << buffer 
       << "\" on environment "
       << "variable H3D_MANUALCURSORCONTROL. Must be TRUE or FALSE. "
       << endl;
@@ -492,7 +502,7 @@ int main(int argc, char* argv[]) {
     else if (strcmp( buffer, "FALSE" ) == 0 ){
       fullscreen = false; }
     else
-      Console(4) << "Invalid value \"" << buffer 
+      Console(LogLevel::Error) << "Invalid value \"" << buffer 
       << "\" on environment "
       << "variable H3D_FULLSCREEN. Must be TRUE or FALSE. "
       << endl;
@@ -505,7 +515,7 @@ int main(int argc, char* argv[]) {
     else if (strcmp( buffer, "FALSE" ) == 0 ){
       mirrored = false; }
     else
-      Console(4) << "Invalid value \"" << buffer 
+      Console(LogLevel::Error) << "Invalid value \"" << buffer 
       << "\" on environment "
       << "variable H3D_MIRRORED. Must be TRUE or FALSE. "<< endl;
   }
@@ -580,8 +590,13 @@ int main(int argc, char* argv[]) {
 
       else if( !strcmp(argv[i]+2,"spacemouse") ){
         use_space_mouse = true; }
+#ifdef HAVE_NVAPI
+      else if( !strncmp(argv[i]+2,"nvsettings=", strlen("nvsettings=")) ) {
+        Scene::nvidia_graphics_options_file_url = strstr(argv[i],"=")+1;
+      }
+#endif
       else {
-        Console(4) << "Unknown argument "
+        Console(LogLevel::Error) << "Unknown argument "
           << "'" << argv[i] << "'" << endl; }
       break;
 
@@ -616,13 +631,13 @@ int main(int argc, char* argv[]) {
       break;
 
     default:
-      Console(4) << "Unknown argument "
+      Console(LogLevel::Error) << "Unknown argument "
         << "'" << argv[i] << "'" << endl;
     }
   }
 
   if (!xml_files.size()){
-    Console(4) << help_message << endl;
+    Console(LogLevel::Error) << help_message << endl;
     return 1;
   }
 
@@ -647,7 +662,7 @@ int main(int argc, char* argv[]) {
       try {
         device_info = X3D::createX3DNodeFromURL( deviceinfo_file );
       } catch( const Exception::H3DException &e ) {
-        Console(3) << "Warning: Could not create default DeviceInfo node "
+        Console(LogLevel::Warning) << "Warning: Could not create default DeviceInfo node "
           << "from file \"" << deviceinfo_file << "\": "
           << e << endl;
       }
@@ -660,7 +675,7 @@ int main(int argc, char* argv[]) {
       try {
         default_stylus = X3D::createX3DNodeFromURL( stylus_file );
       } catch( const Exception::H3DException &e ) {
-        Console( 4 ) << "Warning: Could not create default stylus "
+        Console(LogLevel::Error) << "Warning: Could not create default stylus "
           << "from file \"" << stylus_file << "\": "
           << e << endl;
       }
@@ -691,7 +706,7 @@ int main(int argc, char* argv[]) {
     AutoRef< Group > g( new Group );
     for( vector<string>::iterator file = xml_files.begin() ;
       file != xml_files.end() ; ++file ){
-      Console(3) << "Loading " << *file << endl;
+      Console(LogLevel::Info) << "Loading " << *file << endl;
       scene->loadSceneRoot( *file );
     }
 
@@ -723,7 +738,7 @@ int main(int argc, char* argv[]) {
       try {
         viewpoint = X3D::createX3DNodeFromURL( viewpoint_file );
       } catch( const Exception::H3DException &e ) {
-        Console(3) << "Warning: Could not create default Viewpoint node "
+        Console(LogLevel::Warning) << "Warning: Could not create default Viewpoint node "
           << "from file \"" << viewpoint_file << "\": "
           << e << endl;
       }
@@ -741,7 +756,7 @@ int main(int argc, char* argv[]) {
   }
 
   catch (const Exception::H3DException &e) {
-    Console(4) << e << endl;
+    Console(LogLevel::Error) << e << endl;
 
     // This is required to ensure the scene is deleted
     // in all cases probably due to circular references
