@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2009-2014, SenseGraphics AB
+//    Copyright 2009-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -41,6 +41,9 @@ H3DViewerFieldValuesPanel::H3DViewerFieldValuesPanel( wxWindow* parent,
   wxPanel( parent, id, pos, size, style ),
   displayed_node( NULL )
 {
+
+  AUTOREF_DEBUG_NAME( displayed_node, "H3DViewerFieldValuesDialog::displayed_node")
+
   wxBoxSizer* bSizer2;
   bSizer2 = new wxBoxSizer( wxVERTICAL );
   
@@ -78,11 +81,19 @@ H3DViewerFieldValuesPanel::H3DViewerFieldValuesPanel( wxWindow* parent,
   this->Layout();
   
   // Connect Events
+#if (wxMAJOR_VERSION >= 3)
+  FieldValuesGrid->Connect( wxEVT_GRID_CELL_CHANGED, wxGridEventHandler( H3DViewerFieldValuesPanel::OnCellEdit ), NULL, this );
+#else
   FieldValuesGrid->Connect( wxEVT_GRID_CELL_CHANGE, wxGridEventHandler( H3DViewerFieldValuesPanel::OnCellEdit ), NULL, this );
+#endif
 }
 
 H3DViewerFieldValuesPanel::~H3DViewerFieldValuesPanel() {
+#if (wxMAJOR_VERSION >= 3)
+  FieldValuesGrid->Disconnect( wxEVT_GRID_CELL_CHANGED, wxGridEventHandler( H3DViewerFieldValuesPanel::OnCellEdit ), NULL, this );
+#else
   FieldValuesGrid->Disconnect( wxEVT_GRID_CELL_CHANGE, wxGridEventHandler( H3DViewerFieldValuesPanel::OnCellEdit ), NULL, this );
+#endif
 }
 
 
@@ -154,8 +165,9 @@ void H3DViewerFieldValuesPanel::updateGridFromNode( wxGrid *FieldValuesGrid,
 
   // initialize only fields.
   if( init_only_fields.size() > 0 ) {
-    if( rows >= FieldValuesGrid->GetNumberRows() )
+    if (rows >= FieldValuesGrid->GetNumberRows()){
       FieldValuesGrid->AppendRows(1);
+    }
     FieldValuesGrid->SetCellSize( rows, 0, 1, 2 );
     FieldValuesGrid->SetCellAlignment(rows, 0, wxALIGN_CENTRE, wxALIGN_CENTRE);
     FieldValuesGrid->SetCellValue(rows, 0, _T("Initialize only"));
@@ -320,9 +332,9 @@ void H3DViewerFieldValuesPanel::updateRowFromField( wxGrid *FieldValuesGrid,
                                                      const string &custom_field_name ) {
   if(!default_field) default_field = f;
   
-  if( SFNode *sfnode = dynamic_cast< SFNode * >( f ) ) {
+  if( dynamic_cast< SFNode * >( f ) ) {
     
-  } else if( MFNode *mfnode = dynamic_cast< MFNode * >( f ) ) {
+  } else if( dynamic_cast< MFNode * >( f ) ) {
     
   } else {
     wxGridCellEditor *current_editor = FieldValuesGrid->GetCellEditor( row, 1 );
