@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2010-2014, SenseGraphics AB
+//    Copyright 2010-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -156,7 +156,23 @@ void H3DViewerFieldValuesDialogPropGrid::displayFieldsFromNode( Node *n ) {
   if( !n ) {
     SetTitle(wxT(""));
   } else {
-    SetTitle( wxString(n->getTypeName().c_str(),wxConvUTF8) );
+    string tree_string = n->getTypeName();
+    if (n->hasName()) {
+      tree_string = tree_string + " (" + n->getName() + ")";
+    }
+
+    stringstream s;
+    s << n->getTypeName();
+
+    if (n->hasName()) {
+      s << " (" << n->getName() << ")";
+    }
+
+#ifdef H3D_REFERENCE_COUNT_DEBUG
+    s << " (refs: " << n->getRefCount() << ")";
+#endif
+
+    SetTitle(wxString(s.str().c_str(), wxConvUTF8));
   }
   prop_grid_panel->displayFieldsFromNode( n );
 }
@@ -174,6 +190,8 @@ H3DViewerFieldValuesPanelPropGrid::H3DViewerFieldValuesPanelPropGrid( wxWindow*
   wxPanel( parent, id, pos, size, style ),
   displayed_node( NULL ),
   mode( 0 ) {
+  
+  AUTOREF_DEBUG_NAME( displayed_node, "H3DViewerFieldValuesDialogPropGrid::displayed_node")
 
   this->SetSizeHints( wxDefaultSize, wxDefaultSize );
   
@@ -239,7 +257,17 @@ void H3DViewerFieldValuesPanelPropGrid::displayFieldsFromNode( Node *n ) {
 void H3DViewerFieldValuesPanelPropGrid::OnIdle( wxIdleEvent& event ) {
   TimeStamp now;
   if( now - last_fields_update > 0.25 ) {
-    if( mode == 0 ) displayFieldsFromNode( displayed_node.get() );
+    
+    if( mode == 0 ) {
+#ifdef H3D_REFERENCE_COUNT_DEBUG
+    H3DViewerFieldValuesDialogPropGrid *parent = dynamic_cast<H3DViewerFieldValuesDialogPropGrid *>( GetParent() );
+    if (parent) {
+      parent->displayFieldsFromNode( displayed_node.get() );
+    }
+#else
+      displayFieldsFromNode( displayed_node.get() );
+#endif      
+    }
     last_fields_update = now;
     for( unsigned int i = 0; i < property_update_fields.size(); ++i ) {
       property_update_fields[i]->upToDate();
@@ -366,8 +394,6 @@ void H3DViewerFieldValuesPanelPropGrid::populateGridFromNode( wxPropertyGrid *Fi
     }
   }
 
-  int rows = 0;
-
   // initialize only fields.
   if( init_only_fields.size() > 0 ) {
     FieldValuesGrid->Append( new wxPropertyCategory(wxT("Initialize only"),wxPG_LABEL) );
@@ -499,7 +525,7 @@ void H3DViewerFieldValuesPanelPropGrid::PropertyUpdater::update() {
     wxVectorProperty *p = static_cast< wxVectorProperty * >( property );
     const Vec4d &v = sfvec4d->getValue();
     H3DwxVector wx_v(4);
-    wx_v.setValue(0, v.x); wx_v.setValue(1, v.y); wx_v.setValue(2, v.z); wx_v.setValue(3, v.z);
+    wx_v.setValue(0, v.x); wx_v.setValue(1, v.y); wx_v.setValue(2, v.z); wx_v.setValue(3, v.w);
     p->SetValue( WXVARIANT(wx_v) );
   } else if ( x3d_type == X3DTypes::SFROTATION ) {
     SFRotation *sfrotation = static_cast< SFRotation * >( f );
@@ -612,10 +638,6 @@ wxPGProperty *H3DViewerFieldValuesPanelPropGrid::getPropertyFromField( Field *f,
     wxprop_name_s << field_name << f;
     string wxprop_name = wxprop_name_s.str();
 
-    bool allow_cell_update = true;
-
-   
-
     if( x3d_type == X3DTypes::SFSTRING ) {
       SFString *sfstring = static_cast< SFString * >( f );
 
@@ -671,7 +693,7 @@ wxPGProperty *H3DViewerFieldValuesPanelPropGrid::getPropertyFromField( Field *f,
 #endif //USE_VECTOR_PROPERTIES
     else if( x3d_type == X3DTypes::SFCOLOR ) {
       property = new wxColourProperty(field_name.c_str(), wxprop_name.c_str());
-    } else if( ParsableField *pfield = dynamic_cast< ParsableField * >( f ) ) {
+    } else if( dynamic_cast< ParsableField * >( f ) ) {
       property = new H3DLongStringProperty( false, field_name.c_str(), wxprop_name.c_str() );
     }
   }
@@ -697,13 +719,11 @@ void H3DViewerFieldValuesPanelPropGrid::setupProperty( wxPropertyGrid *FieldValu
                                                        Field *default_f,
                                                        H3DUtil::AutoPtrVector< H3D::Field > &property_update_fields,
                                                        const string &property_name ) {
-
   void *default_value = NULL;
   unsigned int default_value_size = 0;
   
 
   if( MFieldClass *mfield = dynamic_cast< MFieldClass * >( default_f ) ) {
-    // TODO: MFString does not work with getValueAsVoidPtr. Do special case or fix getValueAsVoidPtr.
     if( !dynamic_cast< MFString * >( default_f ) ) {
       unsigned int data_size = mfield->valueTypeSize() * mfield->size();
       unsigned int nr_elements;
@@ -724,10 +744,25 @@ void H3DViewerFieldValuesPanelPropGrid::setupProperty( wxPropertyGrid *FieldValu
       default_value_size = sfield->getValueAsVoidPtr( default_value, data_size );
     }
   }
-  
+
   wxPGProperty *p = getPropertyFromField( f, property_name );
 
+
+
   if(p) {
+    // Avoid adding a property if it already exists. This is because there are some fields which
+    // have alias names, and the same property name can not be used twice.
+    wxPGProperty *values_grid_root = FieldValuesGrid->GetRoot();
+    for( unsigned int i = 0; i < values_grid_root->GetChildCount(); ++i ) {
+      wxPGProperty *category = values_grid_root->Item( i );
+      for( unsigned int j = 0; j < category->GetChildCount(); ++j ) {
+        wxPGProperty *field_value_item = category->Item( j );
+        if( field_value_item->GetName().Cmp( p->GetName() ) == 0 ) {
+          delete p;
+          return;
+        }
+      }
+    }
     if( f->getAccessType() != Field::INPUT_ONLY ) {
       PropertyUpdater *updater = new PropertyUpdater(p, default_value, default_value_size );
       f->route( updater );
@@ -751,8 +786,12 @@ void H3DViewerFieldValuesPanelPropGrid::setupProperty( wxPropertyGrid *FieldValu
 
 WX_PG_IMPLEMENT_VARIANT_DATA_DUMMY_EQ(H3DwxVector)
 
-WX_PG_IMPLEMENT_PROPERTY_CLASS(wxVectorProperty,wxPGProperty,
-                               H3DwxVector,const H3DwxVector&,TextCtrl)
+#if ( defined( __WXMAC__ ) && ( wxMAJOR_VERSION >= 3 ) && ( wxMINOR_VERSION >= 1 ) && ( wxRELEASE_NUMBER >= 1 ) )
+  wxPG_IMPLEMENT_PROPERTY_CLASS(wxVectorProperty,wxPGProperty,TextCtrl)
+#else
+  WX_PG_IMPLEMENT_PROPERTY_CLASS(wxVectorProperty,wxPGProperty,
+                                 H3DwxVector,const H3DwxVector&,TextCtrl)
+#endif
 
 
 wxVectorProperty::wxVectorProperty( const wxString& label,
@@ -845,9 +884,9 @@ void H3DViewerFieldValuesPanelPropGrid::settingsCallback( Scene::ProgramSettings
   if( panel->mode == 1 && panel->IsShown() )
     panel->displayFieldsFromProgramSettings();
  /* if( event == Scene::ADD_SETTING ) {
-    Console(4) << "Added:" << setting.name << endl; 
+    Console(LogLevel::Error) << "Added:" << setting.name << endl; 
   } else {
-    Console(4) << "Removed: " << setting.name << endl; 
+    Console(LogLevel::Error) << "Removed: " << setting.name << endl; 
   }*/
 }
 
