@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2011-2014, SenseGraphics AB
+//    Copyright 2011-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -28,7 +28,7 @@
 //
 //////////////////////////////////////////////////////////////////////////////
 #include "H3DViewerPopupMenus.h"
-
+#include "WxReferenceInfoResultDialog.h"
 #include <fstream>
 #include <wx/wx.h>
 #include <H3D/Scene.h>
@@ -44,7 +44,6 @@
 
 #include <H3DUtil/LoadImageFunctions.h>
 
-/// Callback for collapse all menu choice.
 void H3DViewerPopupMenus::OnTreeViewCollapseAll( wxCommandEvent& event ) {
   wxTreeItemId id = treeview_dialog->TreeViewTree->GetSelection();
   if( id.IsOk() ) {
@@ -52,7 +51,6 @@ void H3DViewerPopupMenus::OnTreeViewCollapseAll( wxCommandEvent& event ) {
   }
 }
 
-/// Callback for expand all menu choice.
 void H3DViewerPopupMenus::OnTreeViewExpandAll( wxCommandEvent& event ) {
   wxTreeItemId id = treeview_dialog->TreeViewTree->GetSelection();
   if( id.IsOk() ) {
@@ -60,7 +58,7 @@ void H3DViewerPopupMenus::OnTreeViewExpandAll( wxCommandEvent& event ) {
   }
 }
 
-/// Callback for collapse children menu choice.
+//Callback for collapse children menu choice.
 void H3DViewerPopupMenus::OnTreeViewCollapseChildren( wxCommandEvent& event ) {
   wxTreeItemId id = treeview_dialog->TreeViewTree->GetSelection();
   wxTreeItemIdValue cookie;
@@ -71,7 +69,7 @@ void H3DViewerPopupMenus::OnTreeViewCollapseChildren( wxCommandEvent& event ) {
   }
 }
 
-/// Callback for node watch menu choice.
+//Callback for node watch menu choice.
 void H3DViewerPopupMenus::OnTreeViewNodeWatch( wxCommandEvent& event ) {
   wxTreeItemId id = treeview_dialog->TreeViewTree->GetSelection();
   
@@ -91,7 +89,48 @@ void H3DViewerPopupMenus::OnTreeViewNodeWatch( wxCommandEvent& event ) {
   }
 }
 
-/// Callback for node save x3d menu choice.
+//Callback for show references menu choice.
+void H3DViewerPopupMenus::OnTreeViewShowNodeReferences( wxCommandEvent& event ) {
+  wxTreeItemId id = treeview_dialog->TreeViewTree->GetSelection();
+  
+  H3DViewerTreeViewDialog::TreeIdMap::iterator ni = treeview_dialog->node_map.find( id.m_pItem );
+  if( ni == treeview_dialog->node_map.end() ) {
+    wxMessageBox( wxT("Selected tree item is not a node"),
+                  wxT("Error"),
+                  wxOK | wxICON_EXCLAMATION);
+  } else {
+
+#ifdef HAVE_PROFILER
+    wxString dialog_title( "test");
+    WxReferenceInfoResultDialog *dialog = new WxReferenceInfoResultDialog( this, wxID_ANY, dialog_title,
+    wxDefaultPosition, wxDefaultSize,
+    (wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER|wxMAXIMIZE_BOX|wxMINIMIZE_BOX|wxCLIP_CHILDREN));
+    dialog->setDisplayedNode( (*ni).second.get() );
+    //fv->displayFieldsFromNode( (*ni).second.get() );
+    dialog->Show();
+#endif
+  }
+}
+
+//Callback for show references menu choice.
+void H3DViewerPopupMenus::OnTreeViewDetectCircularReferences( wxCommandEvent& event ) {
+  wxTreeItemId id = treeview_dialog->TreeViewTree->GetSelection();
+  
+  H3DViewerTreeViewDialog::TreeIdMap::iterator ni = treeview_dialog->node_map.find( id.m_pItem );
+  if( ni == treeview_dialog->node_map.end() ) {
+    wxMessageBox( wxT("Selected tree item is not a node"),
+                  wxT("Error"),
+                  wxOK | wxICON_EXCLAMATION);
+  } else {
+  
+      wxMessageBox( wxT("No circular references detected"),
+                  wxT("Message"),
+                  wxOK | wxICON_EXCLAMATION);
+  }
+}
+
+
+//Callback for node save x3d menu choice.
 void H3DViewerPopupMenus::OnTreeViewSaveX3D( wxCommandEvent& event ) {
   wxTreeItemId id = treeview_dialog->TreeViewTree->GetSelection();
   
@@ -132,7 +171,57 @@ void H3DViewerPopupMenus::OnTreeViewSaveX3D( wxCommandEvent& event ) {
   }
 }
 
-/// Callback for node save VRML menu choice.
+void H3DViewerPopupMenus::onTreeViewLookAt( wxCommandEvent& event ) {
+  wxTreeItemId id = treeview_dialog->TreeViewTree->GetSelection();
+  H3DViewerTreeViewDialog::TreeIdMap::iterator ni = treeview_dialog->node_map.find( id.m_pItem );
+  if( ni == treeview_dialog->node_map.end() ) {
+    wxMessageBox( wxT("Selected tree item is not a node"),
+                  wxT("Error"),
+                  wxOK | wxICON_EXCLAMATION);
+    return;
+  }
+
+  Matrix4f local_to_global;
+  wxTreeItemId tmp_id = treeview_dialog->TreeViewTree->GetItemParent(id);
+  while( tmp_id.m_pItem != NULL ) {
+    H3DViewerTreeViewDialog::TreeIdMap::iterator ni_tmp = treeview_dialog->node_map.find( tmp_id.m_pItem );
+    if( ni_tmp == treeview_dialog->node_map.end() )
+      break;
+    if( MatrixTransform * mt = dynamic_cast< MatrixTransform * >(ni_tmp->second.get()) ) {
+      local_to_global = mt->matrix->getValue();
+    }
+    tmp_id = treeview_dialog->TreeViewTree->GetItemParent(tmp_id);
+  }
+
+  if( H3DBoundedObject * bound_object = dynamic_cast< H3DBoundedObject * >(ni->second.get()) ) {
+    Bound * the_bound = bound_object->bound->getValue();
+    if( the_bound ) {
+      if( BoxBound *bb = dynamic_cast< BoxBound * >(the_bound) ) {
+        Vec3f object_center = bb->center->getValue();
+        Vec3f object_size = bb->size->getValue();
+        X3DViewpointNode * vp = X3DViewpointNode::getActive();
+        Matrix4f vp_inv_m = vp->accInverseMatrix->getValue();
+        Vec3f desired_point = vp_inv_m * local_to_global * object_center;
+        Vec3f dir = vp->totalPosition->getValue() - desired_point;
+        H3DFloat dir_length = dir.length();
+        if( dir_length < Constants::f_epsilon )
+          dir = Vec3f( 0, 0, 1 );
+        else
+          dir /= dir_length;
+        Vec3f ltg_scaling = local_to_global.getScalePart();
+        Vec3f vp_scaling = vp_inv_m.getScalePart();
+        // 2 is simply the 
+        H3DFloat scaled_size = 2 * H3DMax( vp_scaling.x, H3DMax( vp_scaling.y, vp_scaling.z ) ) *
+          H3DMax( ltg_scaling.x, H3DMax( ltg_scaling.y, ltg_scaling.z ) ) *
+          object_size.length();
+        vp->moveTo( desired_point + dir * scaled_size );
+        vp->rotateAroundSelf( -vp->totalOrientation->getValue() * Rotation( Vec3f(0, 0, 1), dir ) );
+      }
+    }
+  }
+}
+
+//Callback for node save VRML menu choice.
 void H3DViewerPopupMenus::OnTreeViewSaveVRML( wxCommandEvent& event ) {
   wxTreeItemId id = treeview_dialog->TreeViewTree->GetSelection();
   
@@ -174,7 +263,7 @@ void H3DViewerPopupMenus::OnTreeViewSaveVRML( wxCommandEvent& event ) {
 }
 
 
-/// Callback for node save x3d menu choice.
+//Callback for node save x3d menu choice.
 void H3DViewerPopupMenus::OnTreeViewSaveSTL( wxCommandEvent& event ) {
   wxTreeItemId id = treeview_dialog->TreeViewTree->GetSelection();
   
@@ -225,7 +314,7 @@ void H3DViewerPopupMenus::OnTreeViewSaveSTL( wxCommandEvent& event ) {
   }
 }
 
-/// Callback for node save x3d menu choice.
+//Callback for node save x3d menu choice.
 void H3DViewerPopupMenus::OnTreeViewSaveTrianglesX3D( wxCommandEvent& event ) {
   wxTreeItemId id = treeview_dialog->TreeViewTree->GetSelection();
   
@@ -365,11 +454,16 @@ void H3DViewerPopupMenus::OnTreeViewAddChildNode( wxCommandEvent& event ) {
     wxMessageBox( wxT("Selected tree item is not a node"),
                   wxT("Error"),
                   wxOK | wxICON_EXCLAMATION);
+	return;
   }
 
   vector< wxString > node_fields;
 
   Node *selected_node = (*ni).second.get();
+  if (!selected_node) {
+    return;
+  }
+
   H3DNodeDatabase *db = H3DNodeDatabase::lookupNodeInstance( selected_node );
   for( H3DNodeDatabase::FieldDBConstIterator i = db->fieldDBBegin();
        db->fieldDBEnd() != i; ++i ) {
@@ -415,14 +509,12 @@ void H3DViewerPopupMenus::OnTreeViewAddChildNode( wxCommandEvent& event ) {
     }
   }
 
-  wxTextEntryDialog *node_name_dialog = 
-    new wxTextEntryDialog(this, 
-                          wxT("Enter the name of the node type you want to use" ),
-                          wxT("Add/replace node" ) );
+  H3DViewerAddChildDialog *node_name_dialog = new H3DViewerAddChildDialog(this);
+  node_name_dialog->SetTargetField(selected_node->getField( field_to_change ));
   if (node_name_dialog->ShowModal() == wxID_OK) {
-    Node *new_node = H3DNodeDatabase::createNode( std::string(node_name_dialog->GetValue().mb_str()) );
+    Node *new_node = H3DNodeDatabase::createNode( std::string(node_name_dialog->GetNodeName().mb_str()) );
     if( !new_node ) {
-      wxMessageBox( wxT("No such node type exists: " + node_name_dialog->GetValue()),
+      wxMessageBox( wxT("No such node type exists: " + node_name_dialog->GetNodeName()),
                     wxT("Error"),
                     wxOK | wxICON_EXCLAMATION);
     } else {
@@ -450,7 +542,8 @@ void H3DViewerPopupMenus::OnTreeViewAddChildNode( wxCommandEvent& event ) {
         }
       }
     }
-  } 
+  }
+  delete node_name_dialog;
 }
 
 // Callback for node save Nrrd menu choice.
@@ -479,13 +572,25 @@ void H3DViewerPopupMenus::OnTreeViewSaveNrrd( wxCommandEvent& event ) {
         if( X3DTexture2DNode * tex = 
             dynamic_cast< X3DTexture2DNode * >( n ) ) {
           image = tex->image->getValue();
+
+          // Some texture nodes don't write to their image field; 
+          // e.g. GeneratedTexture and RenderTargetTexture. 
+          // 
+          // For these cases, we extract the image from OpenGL.
+          if(!image) {
+            image = tex->renderToImage(-1, -1);
+          }
         } else if( X3DTexture3DNode * tex = 
                    dynamic_cast< X3DTexture3DNode * >( n ) ) {
           image = tex->image->getValue();
+
+          if(!image) {
+            image = tex->renderToImage(-1, -1);
+          }
         }
  
-        if( H3DUtil::saveImageAsNrrdFile( string(file_dialog->GetPath().mb_str()),
-                                          image ) != 0 ) {
+        if(!image || (H3DUtil::saveImageAsNrrdFile( 
+          std::string(file_dialog->GetPath().mb_str()), image ) != 0) ) {
           stringstream s;
           s << "Error saving nrrd file";
           wxMessageBox( wxString(s.str().c_str(),wxConvUTF8), wxT("Error"),
@@ -528,13 +633,25 @@ void H3DViewerPopupMenus::OnTreeViewSavePng( wxCommandEvent& event ) {
         if( X3DTexture2DNode * tex = 
             dynamic_cast< X3DTexture2DNode * >( n ) ) {
           image = tex->image->getValue();
+
+          // Some texture nodes don't write to their image field; 
+          // e.g. GeneratedTexture and RenderTargetTexture. 
+          // 
+          // For these cases, we extract the image from OpenGL.
+          if(!image) {
+            image = tex->renderToImage(-1, -1);
+          }
         } else if( X3DTexture3DNode * tex = 
                    dynamic_cast< X3DTexture3DNode * >( n ) ) {
           image = tex->image->getValue();
+
+          if(!image) {
+            image = tex->renderToImage(-1, -1);
+          }
         }
  
-        if( !H3DUtil::saveFreeImagePNG( string(file_dialog->GetPath().mb_str()),
-                                          *image ) ) {
+        if(!image || !H3DUtil::saveFreeImagePNG( 
+              std::string(file_dialog->GetPath().mb_str()), *image ) ) {
           stringstream s;
           s << "Error saving png file";
           wxMessageBox( wxString(s.str().c_str(),wxConvUTF8), wxT("Error"),
@@ -550,3 +667,25 @@ void H3DViewerPopupMenus::OnTreeViewSavePng( wxCommandEvent& event ) {
   }
 #endif
 }
+
+void H3DViewerPopupMenus::OnTreeViewShowImage ( wxCommandEvent& event ) {
+#ifdef HAVE_FREEIMAGE
+  wxTreeItemId id = treeview_dialog->TreeViewTree->GetSelection();
+  
+  H3DViewerTreeViewDialog::TreeIdMap::iterator ni = treeview_dialog->node_map.find( id.m_pItem );
+  if( ni == treeview_dialog->node_map.end() ) {
+    wxMessageBox( wxT("Selected tree item is not a node"),
+                  wxT("Error"),
+                  wxOK | wxICON_EXCLAMATION);
+  } else {
+    X3DTextureNode* image = dynamic_cast < X3DTextureNode* > ( (*ni).second.get() );
+    if ( image ) {
+      treeview_dialog->showImage ( *image );
+    } else {
+      wxMessageBox( wxT("Selected tree item is not an X3DTextureNode"),
+                    wxT("Error"),
+                    wxOK | wxICON_EXCLAMATION);
+    }
+  }
+#endif
+}
\ No newline at end of file
