@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2006-2014, SenseGraphics AB
+//    Copyright 2006-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -28,6 +28,9 @@
 //
 //////////////////////////////////////////////////////////////////////////////
 
+#ifndef H3D_H3DVIEWER_WXFRAME_H__
+#define H3D_H3DVIEWER_WXFRAME_H__
+
 #include <wx/wx.h>
 #include <wx/propdlg.h> 
 #include <wx/bookctrl.h>
@@ -63,6 +66,24 @@
 using namespace std;
 using namespace H3D;
 
+inline std::string toStr( const wxString &s ) {
+# if(wxUSE_UNICODE)
+  char *b = new char[s.size()+1];
+  const wchar_t *wb = s.c_str();
+  for( unsigned int i = 0; i < s.size(); ++i ) {
+    b[i] = (char)(wb[i]);
+  }
+
+  b[s.size()] = '\0';
+  std::string sb(b);
+  delete[] b;
+  return sb;
+#else
+  return std::string( s.c_str() );
+#endif
+}
+
+
 // ---------------------------------------------------------------------------
 //  Property Sheet Dialog
 // ---------------------------------------------------------------------------
@@ -156,6 +177,7 @@ public:
   // StereoInfoGUI
   wxTextCtrl *focal_distance_text;
   wxTextCtrl* interocular_distance_text;
+  wxCheckBox *swap_eyes_checkbox;
 
   // OpenHapticsOptionsGUI
   wxCheckBox *adaptive_viewport;
@@ -207,7 +229,8 @@ protected:
     ID_DRAW_BOUND_TREE,
     ID_DRAW_TREE_DEPTH,
     ID_FOCAL_DISTANCE,
-    ID_INTEROCULAR_DISTANCE
+    ID_INTEROCULAR_DISTANCE,
+    ID_SWAP_EYES
 
   };
 
@@ -226,7 +249,7 @@ class FrameRateDialog: public wxDialog {
   void updateFrameRates();
   void updateMenuItems();
 
-  void OnKeyDown(wxKeyEvent& event);
+  void OnCharHook(wxKeyEvent& event);
 
   wxBoxSizer *topsizer;
   wxStaticText *graphics_rate;
@@ -250,7 +273,9 @@ public:
               long _style = wxDEFAULT_FRAME_STYLE,
               const wxString& name = wxT("H3D Player"),
               bool cmd_line_filename = false,
-              bool disable_plugin_dialog = false );
+              bool disable_plugin_dialog = false,
+              bool _cmd_line_fullscreen = false,
+              bool _hide_menu = true );
 
   ~WxFrame() {
     if( recentFiles )
@@ -292,7 +317,6 @@ public:
   void OnExit( wxCommandEvent & event );
   void OnWindowExit( wxCloseEvent & event );
   void OnAbout( wxCommandEvent & event );
-  void OnHelp( wxCommandEvent & event );
   void ToggleFullscreen( wxCommandEvent & event );
   void MirrorScene( wxCommandEvent & event );
   void RenderMode( wxCommandEvent & event );
@@ -305,9 +329,16 @@ public:
   void ShowPluginsDialog( wxCommandEvent & event );
   void ShowFrameRate( wxCommandEvent & event );
   void ShowProgramSettings( wxCommandEvent & event );
+
+  void OculusRiftRecenter(wxCommandEvent & event);
+#if defined(HAVE_PYTHON) && defined(USE_PYTHON_CONSOLE)
+  void ShowPythonConsole( wxCommandEvent & event );
+#endif
   void OnKeepViewpointOnLoadCheck( wxCommandEvent & event );
   void OnRouteSendsEventsCheck( wxCommandEvent & event );
   void OnLoadTexturesInThreadCheck( wxCommandEvent & event );
+  void OnAlignConsoleAndTreeview( wxCommandEvent & event );
+  void OnShowWindowsInFullscreen(wxCommandEvent & event);
   void ChangeViewpoint( wxCommandEvent & event );
   void ResetViewpoint( wxCommandEvent & event );
   void ChangeNavigation( wxCommandEvent & event );
@@ -315,7 +346,6 @@ public:
   void ChangeCollision( wxCommandEvent & event );
   void OnSpeed( wxCommandEvent & event );
   void ChangeRenderer( wxCommandEvent & event );
-  //void ToggleHaptics( wxCommandEvent & event );
   void OnSettings( wxCommandEvent & event );
   void OnIdle( wxIdleEvent &event );
   void OnReload( wxCommandEvent &event);
@@ -341,10 +371,12 @@ public:
   void LoadPlugins();
   void buildNavMenu();
   void SetFullscreen( bool fullscreen );
+  void SetShowWindowsInFullscreen( bool show );
   void BuildViewpointsMenu( list< Node * > vp_list );
   void BuildViewpointsSubMenu( list< Node* > vp_list, wxMenu * menu, int &count, int &unnamed_vp, int &unnamed_vg );
   void DestroyViewpointsMenu();
   void DestroyViewpointsSubMenu( wxMenu * menu );
+  void SetOculusMenu(bool enabled);
   // returns a list of "top level" viewpoints and viewpoint groups
   list< Node * > GetTopLevelViews();
 
@@ -395,6 +427,7 @@ private:
     // StereoInfo
     float focal_distance;
     float interocular_distance;
+    bool swap_eyes;
 
     // Proxy radius used by ruspini
     float proxy_radius;
@@ -445,6 +478,8 @@ private:
 
   // Used in updating viewpoint menu
   TimeStamp last_viewmenu_update;
+  bool cmd_line_fullscreen;
+  bool hide_menu;
 
 public:
 
@@ -464,6 +499,7 @@ public:
   FrameRateDialog *  frameRates;
   SettingsDialog * settings;
   SpeedDialog *speed_slider;
+  wxStatusBar *the_status_bar;
 
   AutoRef< GlobalSettings > global_settings;
 
@@ -532,6 +568,14 @@ public:
   };
   auto_ptr< UpdateStereoModeMenu > updateStereoModeMenu;
 
+  // There is currently no way to just hide the menu bar using wxWidgets functions.
+  // However, menu works fine in fullscreen even though it is not shown.
+  // Sometimes there are weird cases in which it is useful to have a windowed mode
+  // with a size but no menu bar shown. The menu shortcuts should work however
+  // which is the reason for using this hack instead of setting the active menu bar
+  // to NULL.
+  void hideMenuBarHack();
+
 protected:
   DECLARE_EVENT_TABLE()
 };
@@ -556,9 +600,12 @@ enum
   FRAME_PLUGINS,
   FRAME_FRAMERATE,
   FRAME_PROGRAMSETTINGS,
+  FRAME_PYTHON_CONSOLE,
   FRAME_KEEPVIEWPOINTONLOAD,
   FRAME_ROUTESENDSEVENTS,
   FRAME_LOADTEXTURESINTHREAD,
+  FRAME_ALIGNCONSOLETREEVIEW,
+  FRAME_SHOWWINDOWSINFULLSCREEN,
   FRAME_SELECTION,
   FRAME_VIEWPOINT,
   FRAME_NAVIGATION = 6500,
@@ -590,7 +637,8 @@ enum
   FRAME_NVIDIA_3DVISION,
   FRAME_HDMI720P,
   FRAME_HDMI1080P,
-  //FRAME_DEVICECONTROL,
+  FRAME_VERTICAL_SPLIT_KEEP_ASPECT_ONE_PASS,
+  FRAME_OCULUS_RIFT,
   FRAME_HAPTICSCONTROL,
   BASIC_PREFRENDERER,
   BASIC_WIREFRAME,
@@ -604,13 +652,14 @@ enum
   BASIC_COLLISION,
   FRAME_SPEED,
   FRAME_SPEED_SLIDER,
-  //FRAME_HELP,
   FRAME_SETTINGS,
   FRAME_RESET_VIEWPOINT,
   FRAME_MOUSE_NAV,
   FRAME_KEYBOARD_NAV,
   FRAME_SWS_NAV,
   FRAME_HAPTICSDEVICE_NAV,
-  FRAME_RELOAD
+  FRAME_RELOAD,
+  FRAME_OCULUS_RECENTER
 };
 
+#endif
\ No newline at end of file
