@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2006-2014, SenseGraphics AB
+//    Copyright 2006-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -34,7 +34,7 @@
 #include <H3D/Viewpoint.h>
 #include <H3D/INIFile.h>
 
-#ifndef WIN32
+#ifndef H3D_WINDOWS
 #include <H3D/X3DKeyDeviceSensorNode.h>
 #include <H3D/MouseSensor.h>
 #endif
@@ -96,9 +96,13 @@ WxWidgetsWindow::WxWidgetsWindow( wxWindow *_theParent,
   theWindow->SetSizer( sizer );
 }
 
+WxWidgetsWindow::~WxWidgetsWindow() {
+  deinitWindow();
+}
+
 void WxWidgetsWindow::initWindow() {
   if( isInitialized() && !allow_new_pixel_format_creation ) {
-    Console(4) << "WxWidgetsWindow does not support changing pixel format from/to "
+    Console(LogLevel::Error) << "WxWidgetsWindow does not support changing pixel format from/to "
                << "quad buffered stereo support after initialization." << endl;
     if( last_render_mode == RenderMode::QUAD_BUFFERED_STEREO ) {
       renderMode->setValue( "QUAD_BUFFERED_STEREO" );
@@ -108,6 +112,7 @@ void WxWidgetsWindow::initWindow() {
     return;
   }
   RenderMode::Mode stereo_mode = renderMode->getRenderMode();
+  int num_samples = 0;
 
   int attribList[20];
   unsigned int i = 0;
@@ -121,16 +126,18 @@ void WxWidgetsWindow::initWindow() {
   // fullscreen anti-aliasing only supported on wxWidgets 2.9 or later
 #if( wxMAJOR_VERSION > 2 || wxMINOR_VERSION >= 9 ) 
   if( useFullscreenAntiAliasing->getValue() ) {
+    num_samples = 4;
+
     attribList[i++] = WX_GL_SAMPLE_BUFFERS;
     attribList[i++] = 1;
     attribList[i++] = WX_GL_SAMPLES;
-    attribList[i++] = 4;
+    attribList[i++] = num_samples;
   }
 #endif
 
-  // TODO: FIX stereo mode
+  numSamples->setValue(num_samples, id);
+
 #ifdef MACOSX
-  // TODO: stereo mode does not work with mac
   attribList[i++] = 0;
 #else
   // On Windows we can not check here if stereo is supported, simply because
@@ -229,11 +236,26 @@ void WxWidgetsWindow::initWindow() {
   is_initialized = true;
 }
 
+void WxWidgetsWindow::deinitWindow() {
+  H3DWindowNode::deinitWindow();
+  if( theWxGLCanvas )
+    theWxGLCanvas->myOwner = NULL;
+
+  if( !have_parent ) {
+    theWindow->Destroy();
+  }
+      
+  // Seems like wxWidgets 2.9 does not destruct explicit wxGLCanvas.
+  if( theWxGLContext ) {
+    delete theWxGLContext;
+    theWxGLContext = NULL;
+  }
+}
+
 void WxWidgetsWindow::setFullscreen( bool fullscreen ) {
   if ( last_fullscreen != fullscreen || !fullscreen_initialized ) {
     wxTopLevelWindow * tlw = dynamic_cast< wxTopLevelWindow * >(theWindow);
 #ifndef MACOSX
-    //TODO: fullscreen does not work well on macosx
     if( tlw ) {
       long style = wxFULLSCREEN_ALL;
       WxFrame * tmp_frame = dynamic_cast< WxFrame * >( theWindow );
@@ -256,10 +278,10 @@ void WxWidgetsWindow::setFullscreen( bool fullscreen ) {
 //#include<FreeImage.h>
 
 void WxWidgetsWindow::swapBuffers() {
-
-  RenderMode::Mode stereo_mode = renderMode->getRenderMode();
+  
 
 #ifdef HAVE_DX9
+  RenderMode::Mode stereo_mode = renderMode->getRenderMode();
   if( stereo_mode == RenderMode::NVIDIA_3DVISION ) {
     int current_w = generator->width->getValue();
     int current_h = generator->height->getValue();
@@ -271,7 +293,7 @@ void WxWidgetsWindow::swapBuffers() {
     glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, 0 );
     
     int row_length = 4 * current_w;
-    for( unsigned int i = 0; i < current_h; ++i ) {
+    for( int i = 0; i < current_h; ++i ) {
       memcpy( &gl_d3d_frame_buffer[0] + (current_h - i -1 )*row_length,
               &gl_frame_buffer[0] + i *row_length, 
               row_length );
@@ -309,7 +331,7 @@ BEGIN_EVENT_TABLE(WxWidgetsWindow::MyWxGLCanvas, wxGLCanvas)
   EVT_SIZE(WxWidgetsWindow::MyWxGLCanvas::OnSize)
   EVT_PAINT(WxWidgetsWindow::MyWxGLCanvas::OnPaint)
   EVT_ERASE_BACKGROUND(WxWidgetsWindow::MyWxGLCanvas::OnEraseBackground)
-#ifndef WIN32
+#ifndef H3D_WINDOWS
   EVT_KEY_DOWN( WxWidgetsWindow::MyWxGLCanvas::OnKeyDown )
   EVT_KEY_UP( WxWidgetsWindow::MyWxGLCanvas::OnKeyUp )
   EVT_CHAR( WxWidgetsWindow::MyWxGLCanvas::OnCharDown )
@@ -374,7 +396,7 @@ void WxWidgetsWindow::MyWxGLCanvas::OnSize( wxSizeEvent& event ) {
   // set GL viewport (not called by wxGLCanvas::OnSize on all platforms...)
   int w, h;
   GetClientSize(&w, &h);
-  if( myOwner ) {
+  if( myOwner && ( w != myOwner->width->getValue() || h != myOwner->height->getValue() ) ) {
     RenderMode::Mode stereo_mode = myOwner->renderMode->getRenderMode();
     if( stereo_mode != RenderMode::NVIDIA_3DVISION ) {
       myOwner->reshape( w, h );
@@ -549,12 +571,15 @@ void WxWidgetsWindow::MyWxGLCanvas::OnKeyDown(wxKeyEvent& event)
       break;
     case WXK_MENU: myOwner->onKeyDown( X3DKeyDeviceSensorNode::ALT, true );
       break;
+    case WXK_ALT: myOwner->onKeyDown( X3DKeyDeviceSensorNode::ALT, true );
+      break;
     case WXK_CONTROL: myOwner->onKeyDown( X3DKeyDeviceSensorNode::CONTROL,
                         true ); break;
     case WXK_SHIFT: myOwner->onKeyDown( X3DKeyDeviceSensorNode::SHIFT, true );
       break;
-    default: { event.Skip(); }
+    default: { myOwner->onKeyDown(event.GetKeyCode(), false ); }
   }
+  event.Skip();
 }
 
 void WxWidgetsWindow::MyWxGLCanvas::OnKeyUp(wxKeyEvent& event)
@@ -589,12 +614,15 @@ void WxWidgetsWindow::MyWxGLCanvas::OnKeyUp(wxKeyEvent& event)
       break;
     case WXK_MENU: myOwner->onKeyUp( X3DKeyDeviceSensorNode::ALT, true );
       break;
+    case WXK_ALT: myOwner->onKeyUp( X3DKeyDeviceSensorNode::ALT, true );
+      break;
     case WXK_CONTROL: myOwner->onKeyUp( X3DKeyDeviceSensorNode::CONTROL,
                         true ); break;
     case WXK_SHIFT: myOwner->onKeyUp( X3DKeyDeviceSensorNode::SHIFT, true );
       break;
     default: { myOwner->onKeyUp(event.GetKeyCode(), false ); }
   }
+  event.Skip();
 }
 
 void WxWidgetsWindow::MyWxGLCanvas::OnCharDown(wxKeyEvent& event)
@@ -772,7 +800,7 @@ void WxWidgetsWindow::D3D9Canvas::updateGLFrameBufferSize( int w, int h ) {
                                                   D3DPOOL_DEFAULT, 
                                                   &src_surface, 
                                                   NULL) != D3D_OK ) {
-      Console(4) << "Warning: Could not create DirectX offscreen "
+      Console(LogLevel::Error) << "Warning: Could not create DirectX offscreen "
                  << "surface for NVidia 3DVision rendering." << endl;
     } else {
       // Add NVidia stereo header in last row of offscreen surface.
@@ -806,6 +834,11 @@ void WxWidgetsWindow::D3D9Canvas::OnSize(wxSizeEvent& event)
   // get the size of the canvas.
   int w, h;
   GetClientSize(&w, &h);
+
+  // OnSize can sometimes be called even if the size has not changed
+  if( ( w == owner->width->getValue() ) && ( h == owner->height->getValue() ) ) {
+    return;
+  }
   
   // we need to reset the d3d device. In order to do that we are required
   // to first delete all surfaces that have been created previously and 
@@ -828,17 +861,17 @@ void WxWidgetsWindow::D3D9Canvas::OnSize(wxSizeEvent& event)
   // reset device
   HRESULT hr = g_pd3dDevice->Reset( &d3dpp );
   if(  hr != D3D_OK ) {
-    Console(4) << "DirectX device reset failed: ";
+    Console(LogLevel::Error) << "DirectX device reset failed: ";
     if( hr == D3DERR_DEVICELOST ) 
-      Console(4) << "Device lost" << endl;
+      Console(LogLevel::Error) << "Device lost" << endl;
     else if( hr == D3DERR_DEVICEREMOVED ) 
-      Console(4) << "Device removed" << endl;
+      Console(LogLevel::Error) << "Device removed" << endl;
     else if( hr == D3DERR_DRIVERINTERNALERROR )
-      Console(4) << "Driver internal error" << endl;
+      Console(LogLevel::Error) << "Driver internal error" << endl;
     else if( hr == D3DERR_OUTOFVIDEOMEMORY )
-      Console(4) << "Out of mem" << endl;
+      Console(LogLevel::Error) << "Out of mem" << endl;
     else
-      Console(4) << "Unknown error"<< endl;
+      Console(LogLevel::Error) << "Unknown error"<< endl;
   } 
 
   // restore d3d surfaces with the new size
@@ -903,7 +936,7 @@ void WxWidgetsWindow::D3D9Canvas::Render() {
                                  &dstRect,
                                  D3DX_DEFAULT,
                                  0)!=D3D_OK ) {
-      Console(4) <<"Warning: Error loading OpenGL data to DirectX offscreen surface" << endl;
+      Console(LogLevel::Error) <<"Warning: Error loading OpenGL data to DirectX offscreen surface" << endl;
     }
     
     // Debug: save surface to bitmap file
