@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -201,6 +201,10 @@ namespace H3D{
     struct H3DAPI_API ExecutionContext: public H3DUtil::RefCountedClass {
       ExecutionContext( bool _clean_up = false ) :
         clean_up( _clean_up ) {
+        DEFNODES_DEBUG_NAME( named_nodes, "SAI::ExecutionContext::named_nodes")
+        DEFNODES_DEBUG_NAME( exported_nodes, "SAI::ExecutionContext::exported_nodes")
+        AUTOREF_DEBUG_NAME( root_node, "SAI::ExecutionContext::root_node")
+
         instances.push_back( this );
       }
 
@@ -226,7 +230,6 @@ namespace H3D{
       //const string &getSpecificationVersion();
       //const string &getEncoding();
 
-      // TODO: 
       //      const ProfileInfo &getProfileInfo();
       // const ComponentInfo      
       // ...
@@ -543,7 +546,6 @@ namespace H3D{
     /// by the returned SAIScene value.
     SAIScene *createX3DFromURL( MFString *urls );
 
-    // TODO:
     //void updateControl( 
     //    registerBrowserInterest
 
@@ -652,7 +654,6 @@ namespace H3D{
     /// those events to the browser.
     void dispose();
 
-    /// TODO?:
     /// The setBrowserOption service allows setting options defined in 9.2.5 
     /// Browser options in ISO/IEC 19775-1. The name field shall be one of 
     /// the defined names in Table 9.2 in ISO/IEC 19775-1. This service shall
