@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -37,7 +37,6 @@
 #include <stack>
 #include <map>
 
-using namespace std;
 
 namespace H3D {
   // forward declarations
@@ -425,7 +424,7 @@ namespace H3D {
     void callPostTraverseCallbacks();
 
     /// Class grouping X3DLightNode and its transformation matrix.
-    class LightInfo {
+    class H3DAPI_API LightInfo {
     public:
       /// Constructor.
       LightInfo( X3DLightNode *, const Matrix4f &transform );
@@ -487,6 +486,6 @@ namespace H3D {
     std::map< string, void * > user_data;
   };
 
-};
+}
 
 #endif
