@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -17,7 +17,7 @@
 //    along with H3D API; if not, write to the Free Software
 //    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
-//    A commercial license is also available. Please contact us at 
+//    A commercial license is also available. Please contact us at
 //    www.sensegraphics.com for more information.
 //
 //
@@ -32,23 +32,23 @@
 
 using namespace H3D;
 
-H3DNodeDatabase Extrusion::database( 
-        "Extrusion", 
+H3DNodeDatabase Extrusion::database(
+        "Extrusion",
         &(newInstance< Extrusion > ),
         typeid( Extrusion ),
         &X3DGeometryNode::database );
 
 namespace ExtrusionInternals {
-  FIELDDB_ELEMENT( Extrusion, beginCap, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( Extrusion, ccw, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( Extrusion, convex, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( Extrusion, creaseAngle, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( Extrusion, crossSection, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( Extrusion, endCap, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( Extrusion, orientation, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( Extrusion, scale, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( Extrusion, solid, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( Extrusion, spine, INPUT_OUTPUT );
+  FIELDDB_ELEMENT( Extrusion, beginCap, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( Extrusion, ccw, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( Extrusion, convex, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( Extrusion, creaseAngle, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( Extrusion, crossSection, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( Extrusion, endCap, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( Extrusion, orientation, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( Extrusion, scale, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( Extrusion, solid, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( Extrusion, spine, INPUT_OUTPUT )
 
   // returns true if the two points are coincident
   template <typename T>
@@ -97,7 +97,7 @@ namespace ExtrusionInternals {
       j_minone = j - 1;
     else if( closed_cross_section )
       j_minone = cross_section_size_minone - 1;
-    
+
     vector< H3DInt32 > found_indices;
     if( if_caps_add > 0 && i == 0 ) {
       found_indices.push_back( 0 );
@@ -204,17 +204,15 @@ Extrusion::Extrusion(  Inst< SFNode      > _metadata,
 
 void Extrusion::VertexVector::update() {
   value.clear();
-  const vector< Vec2f > &cross_section = 
+  const vector< Vec2f > &cross_section =
     static_cast< MFVec2f * >( routes_in[0] )->getValue();
-  const vector< Rotation > &orn_vector = 
+  const vector< Rotation > &orn_vector =
     static_cast< MFRotation * >( routes_in[1] )->getValue();
-  const vector< Vec2f > &scale_vector = 
+  const vector< Vec2f > &scale_vector =
     static_cast< MFVec2f * >( routes_in[2] )->getValue();
-  const vector< Vec3f > &spine_vector = 
+  const vector< Vec3f > &spine_vector =
     static_cast< MFVec3f * >( routes_in[3] )->getValue();
 
-  Extrusion *extrusion = static_cast< Extrusion * >( getOwner() );
-
   H3DInt32 spine_size = (H3DInt32)spine_vector.size();
   H3DInt32 nr_of_cross_section_points = (H3DInt32)cross_section.size();
   H3DInt32 nr_of_scale_values = (H3DInt32)scale_vector.size();
@@ -236,14 +234,14 @@ void Extrusion::VertexVector::update() {
     // points. If there are more scale or orientation values than spine points
     // the excess values are ignored.
     if( nr_of_orn_values < spine_size && nr_of_orn_values > 1 ) {
-      Console(3)
+      Console(LogLevel::Warning)
         << "Warning: Not enough orientation values in Extrusion node( "
         << getName() << "). Node will not be rendered. " << endl;
       return;
     }
 
     if( nr_of_scale_values < spine_size && nr_of_scale_values > 1  ) {
-      Console(3) << "Warning: Not enough scale values in Extrusion node( "
+      Console(LogLevel::Warning) << "Warning: Not enough scale values in Extrusion node( "
         << getName() << "). Node will not be rendered. " << endl;
       return;
     }
@@ -260,7 +258,7 @@ void Extrusion::VertexVector::update() {
                                    spine_vector.back() ) )
       closed_spine = true;
 
-    // build vectors containing indicies to the first non-coincident spine. 
+    // build vectors containing indicies to the first non-coincident spine.
     // Backward and forward.
     // these vectors are used to calculate axis for the SCP around each spine.
     for( int i = 0; i < spine_size; ++i ) {
@@ -318,7 +316,7 @@ void Extrusion::VertexVector::update() {
       // point is undetermined at this stage add dummy.
       if( closed_spine ) {
         Vec3f temp = ( spine_vector[ spine_forward.front() ] -
-                       spine_vector[0] ).crossProduct( 
+                       spine_vector[0] ).crossProduct(
                        spine_vector[ spine_backward.front() ] -
                        spine_vector[0] );
         temp.normalizeSafe();
@@ -331,11 +329,11 @@ void Extrusion::VertexVector::update() {
       else
         z_axis.push_back( Vec3f( 0, 0, 0) );
 
-      // calculate z-axes for all spine points but the last and for the first 
+      // calculate z-axes for all spine points but the last and for the first
       // spine point if it is undetermined.
       for( int  i = 1; i < spine_size - 1; ++i ) {
         Vec3f temp =( spine_vector[ spine_forward[i] ] - spine_vector[i] )
-          .crossProduct( 
+          .crossProduct(
           spine_vector[ spine_backward[i] ] - spine_vector[i] );
         temp.normalizeSafe();
         z_axis.push_back( temp );
@@ -351,7 +349,7 @@ void Extrusion::VertexVector::update() {
           collinear = false;
           if( z_axis.front().length() < Constants::f_epsilon ) {
             z_axis.front() = z_axis.back();
-            for( int j = 1; 
+            for( int j = 1;
               j < i && z_axis[j].length() < Constants::f_epsilon;
               ++j )
               z_axis[j] = z_axis.front();
@@ -402,7 +400,7 @@ void Extrusion::VertexVector::update() {
         z_axis.front().normalizeSafe();
 
         Vec3f spine_dir = spine_vector.back() - spine_vector.front();
-        for ( int i = spine_size - 1; 
+        for ( int i = spine_size - 1;
           i > 0 && spine_dir.length() < Constants::f_epsilon;
           --i ) {
             spine_dir = ( spine_vector[i] - spine_vector.front() );
@@ -425,7 +423,7 @@ void Extrusion::VertexVector::update() {
       else {
         // axes for first spine point
         Vec3f temp;
-        if( closed_spine ) {      
+        if( closed_spine ) {
           temp = spine_vector[ spine_forward.front() ]
           - spine_vector[ spine_backward.front() ];
           temp.normalizeSafe();
@@ -484,7 +482,7 @@ void Extrusion::VertexVector::update() {
           * cross_section[j].x * x_axis[i];
         Vec3f point0_z = scale_vector[ i % nr_of_scale_values ].y
           * cross_section[j].y * z_axis[i];
-        Vec3f point0 = spine_vector[i] + 
+        Vec3f point0 = spine_vector[i] +
         ( point0_x + point0_z );
         value.push_back( point0 );
       }
@@ -501,10 +499,10 @@ void Extrusion::render() {
     H3DInt32 nr_of_cross_section_points = crossSection->size();
     H3DFloat crease_angle = creaseAngle->getValue();
 
-    // determine which side of polygons is front and back face. Since the 
+    // determine which side of polygons is front and back face. Since the
     // GLWindow renderCamera() function might scale the y-axis with -1 and
     // set the GL_FRONT_FACE to GL_CW if scaled and GL_CCW otherwise, we have
-    // to check what the previous value is to determine what the GL_FRONT_FACE 
+    // to check what the previous value is to determine what the GL_FRONT_FACE
     // will be. This can also happen if a Transform node has negative scaling.
     GLint front_face;
     glGetIntegerv( GL_FRONT_FACE, &front_face );
@@ -513,12 +511,12 @@ void Extrusion::render() {
       // to the opposite in order for it to be right when mirrored.
       if( ccw_value )
         glFrontFace( GL_CW );
-      else  
+      else
         glFrontFace( GL_CCW );
     } else {
       if( ccw->getValue() )
         glFrontFace( GL_CCW );
-      else 
+      else
         glFrontFace( GL_CW );
     }
 
@@ -548,7 +546,7 @@ void Extrusion::render() {
 
       for( int i = nr_of_cross_section_points - 1; i >= 0; --i )
       {
-        renderTexCoordForActiveTexture( Vec3f( caps_tex_coord[i].x, 
+        renderTexCoordForActiveTexture( Vec3f( caps_tex_coord[i].x,
                                                caps_tex_coord[i].y,
                                                0 ) );
         if( normal_per_vertex ) {
@@ -575,11 +573,10 @@ void Extrusion::render() {
           glNormal3f( n.x, n.y, n.z );
         }
 
-        renderTexCoordForActiveTexture( Vec3f( u_tex_coord[j], 
+        renderTexCoordForActiveTexture( Vec3f( u_tex_coord[j],
                                                v_tex_coord[i],
                                                0 ) );
         if( normal_per_vertex ) {
-          Vec3f n;
           if( crease_angle_below_pi )
             n = normals[ if_caps_add + quad_index * 6 ];
           else
@@ -589,7 +586,7 @@ void Extrusion::render() {
         v = vertexvec[ lower ];
         glVertex3f( v.x, v.y, v.z );
 
-        renderTexCoordForActiveTexture( Vec3f( u_tex_coord[ j + 1 ], 
+        renderTexCoordForActiveTexture( Vec3f( u_tex_coord[ j + 1 ],
                                                v_tex_coord[i],
                                                0 ) );
         if( normal_per_vertex ) {
@@ -602,7 +599,7 @@ void Extrusion::render() {
         v = vertexvec[ lower + 1 ];
         glVertex3f( v.x, v.y, v.z );
 
-        renderTexCoordForActiveTexture( Vec3f( u_tex_coord[ j + 1 ], 
+        renderTexCoordForActiveTexture( Vec3f( u_tex_coord[ j + 1 ],
                                                v_tex_coord[ i + 1 ],
                                                0 ) );
         if( normal_per_vertex ) {
@@ -621,11 +618,10 @@ void Extrusion::render() {
           glNormal3f( n.x, n.y, n.z );
         }
 
-        renderTexCoordForActiveTexture( Vec3f( u_tex_coord[j], 
+        renderTexCoordForActiveTexture( Vec3f( u_tex_coord[j],
                                                v_tex_coord[i],
                                                0 ) );
         if( normal_per_vertex ) {
-          Vec3f n;
           if( crease_angle_below_pi )
             n = normals[ if_caps_add + quad_index * 6 + 3 ];
           else
@@ -635,7 +631,7 @@ void Extrusion::render() {
         v = vertexvec[ lower ];
         glVertex3f( v.x, v.y, v.z );
 
-        renderTexCoordForActiveTexture( Vec3f( u_tex_coord[ j + 1 ], 
+        renderTexCoordForActiveTexture( Vec3f( u_tex_coord[ j + 1 ],
                                                v_tex_coord[ i + 1 ],
                                                0 ) );
         if( normal_per_vertex ) {
@@ -648,7 +644,7 @@ void Extrusion::render() {
         v = vertexvec[ upper + 1 ];
         glVertex3f( v.x, v.y, v.z );
 
-        renderTexCoordForActiveTexture( Vec3f( u_tex_coord[j], 
+        renderTexCoordForActiveTexture( Vec3f( u_tex_coord[j],
                                                v_tex_coord[ i + 1 ],
                                                0 ) );
         if( normal_per_vertex ) {
@@ -673,7 +669,6 @@ void Extrusion::render() {
 
       glBegin( GL_POLYGON );
 
-      Vec3f n, v;
       if( !normal_per_vertex ) {
         n = normals.back();
         glNormal3f( n.x, n.y, n.z );
@@ -681,7 +676,7 @@ void Extrusion::render() {
 
       for(int i = 0; i < nr_of_cross_section_points; ++i )
       {
-        renderTexCoordForActiveTexture( Vec3f( caps_tex_coord[i].x, 
+        renderTexCoordForActiveTexture( Vec3f( caps_tex_coord[i].x,
                                                caps_tex_coord[i].y,
                                                0 ) );
         if( normal_per_vertex ) {
@@ -720,7 +715,7 @@ void Extrusion::AutoNormal::update() {
                                  spine_vector.back() ) )
     closed_spine = true;
 
-  // build vectors containing indicies to the first non-coincident spine. 
+  // build vectors containing indicies to the first non-coincident spine.
   // Backward and forward.
   // these vectors are used to calculate axis for the SCP around each spine.
   for( int i = 0; i < spine_size; ++i ) {
@@ -769,7 +764,7 @@ void Extrusion::AutoNormal::update() {
     // check if the spine is collinear
     if( closed_spine ) {
       Vec3f temp = ( spine_vector[ spine_forward.front() ] - spine_vector[0] )
-        .crossProduct( 
+        .crossProduct(
         spine_vector[ spine_backward.front() ] - spine_vector[0] );
       // if one of the cross products calculated is not zero then the spine is
       // not collinear.
@@ -790,8 +785,8 @@ void Extrusion::AutoNormal::update() {
 
     if( collinear ) {
       // the y-axes is calculated like this since when the spine is collinear
-      // it might be that the stepping forward to the first non-coincidential 
-      // spine point might end up being the same spine point as if stepping 
+      // it might be that the stepping forward to the first non-coincidential
+      // spine point might end up being the same spine point as if stepping
       // backward to the first non-coincidential spine point
       // if the spine is closed.
       Vec3f temp = spine_vector[ spine_forward.front() ] - spine_vector[0];
@@ -841,7 +836,7 @@ void Extrusion::AutoNormal::update() {
   const vector< Vec2f > &cross_section = extrusion->crossSection->getValue();
   if( crease_angle > Constants::f_epsilon ) {
     if( crease_angle < Constants::pi )
-      value = generateNormalsPerVertex( vertex_vector, 
+      value = generateNormalsPerVertex( vertex_vector,
                                         extrusion->crossSection->getValue(),
                                         y_axis,
                                         extrusion->ccw->getValue(),
@@ -851,8 +846,8 @@ void Extrusion::AutoNormal::update() {
                                         crease_angle,
                            static_cast< SFBool * >(routes_in[2])->getValue(),
                            static_cast< SFBool * >(routes_in[3])->getValue() );
-    else 
-      value = generateNormalsPerVertex( vertex_vector, 
+    else
+      value = generateNormalsPerVertex( vertex_vector,
                                         extrusion->crossSection->getValue(),
                                         y_axis,
                                         extrusion->ccw->getValue(),
@@ -950,7 +945,7 @@ Normal * Extrusion::AutoNormal::generateNormalsPerFace(
                       bool closed_cross_section,
                       bool begin_cap,
                       bool end_cap ) {
-  
+
   vector < Vec3f > normal_vector;
   H3DInt32 closed_cross_section_int = 0;
   H3DFloat ccw_or_cw = 0;
@@ -963,7 +958,7 @@ Normal * Extrusion::AutoNormal::generateNormalsPerFace(
   for( int i = 0;
        i < nr_of_cross_section_points - closed_cross_section_int;
        ++i )
-    ccw_or_cw += cross_section[i].y * 
+    ccw_or_cw += cross_section[i].y *
                cross_section[ ( i + 1 ) % nr_of_cross_section_points ].x -
                cross_section[ ( i + 1 ) % nr_of_cross_section_points ].y *
                cross_section[i].x;
@@ -971,10 +966,10 @@ Normal * Extrusion::AutoNormal::generateNormalsPerFace(
   // calculate normal for the beginCap if it exists.
   if( begin_cap ) {
     if( ( ccw_value && ccw_or_cw >= 0 ) || ( !ccw_value && ccw_or_cw < 0 ) )
-      normal_vector.push_back( 
+      normal_vector.push_back(
         Vec3f( -y_axis.front().x, -y_axis.front().y, -y_axis.front().z ) );
     else
-      normal_vector.push_back( 
+      normal_vector.push_back(
         Vec3f( y_axis.front().x, y_axis.front().y, y_axis.front().z ) );
   }
 
@@ -1025,7 +1020,7 @@ Normal * Extrusion::AutoNormal::generateNormalsPerFace(
         first_normal.normalize();
         second_normal.normalize();
       }
-      
+
       if( ccw_value ) {
         normal_vector.push_back( first_normal );
         normal_vector.push_back( second_normal );
@@ -1035,7 +1030,7 @@ Normal * Extrusion::AutoNormal::generateNormalsPerFace(
       }
     }
   }
-  
+
   // calculate normal for the endCap if it exists
   if( end_cap ) {
     if( ( ccw_value && ccw_or_cw >= 0 ) || ( !ccw_value && ccw_or_cw < 0 ) )
@@ -1061,12 +1056,12 @@ Normal * Extrusion::AutoNormal::generateNormalsPerVertex(
                       bool closed_spine,
                       bool begin_cap,
                       bool end_cap ) {
-  
+
   bool closed_cross_section = false;
   if( ExtrusionInternals::coinc( cross_section.front(),
                                  cross_section.back() ) )
     closed_cross_section = true;
-                        
+
   AutoRef< Normal > normals_per_face_node;
   normals_per_face_node.reset( generateNormalsPerFace(
                                       vertex_vector,
@@ -1091,13 +1086,11 @@ Normal * Extrusion::AutoNormal::generateNormalsPerVertex(
     for( int j = 0; j < nr_of_cross_section_points; ++j ) {
       vector< H3DInt32 > indices;
       normal_vector.push_back( Vec3f( 0, 0, 0 ) );
-      bool begin_cap_summed = false;
-      bool end_cap_summed = false;
 
       indices = ExtrusionInternals::findSurroundingNormalIndex( i,
                                          j,
                                          closed_spine,
-                                         spine_size, 
+                                         spine_size,
                                          closed_cross_section,
                                          nr_of_cross_section_points_minone,
                                          if_caps_add,
@@ -1125,14 +1118,14 @@ Normal * Extrusion::AutoNormal::generateNormalsPerVertex(
                       H3DFloat crease_angle,
                       bool begin_cap,
                       bool end_cap ) {
-  
+
   bool closed_cross_section = false;
   if( ExtrusionInternals::coinc( cross_section.front(),
                                  cross_section.back() ) )
     closed_cross_section = true;
 
   AutoRef< Normal > normals_per_face_node;
-  normals_per_face_node.reset( generateNormalsPerFace( 
+  normals_per_face_node.reset( generateNormalsPerFace(
                                      vertex_vector,
                                      cross_section,
                                      y_axis,
@@ -1162,7 +1155,7 @@ Normal * Extrusion::AutoNormal::generateNormalsPerVertex(
     if( quad_normal * face_normal > cos_crease_angle )
       normal_vector.back() += quad_normal;
     if( closed_cross_section ) {
-      quad_normal = 
+      quad_normal =
         normals_per_face[ if_caps_add + 2* (nr_of_cross_section_points - 2) ];
       if( quad_normal * face_normal > cos_crease_angle )
         normal_vector.back() += quad_normal;
@@ -1213,8 +1206,6 @@ Normal * Extrusion::AutoNormal::generateNormalsPerVertex(
                              2 * ( i * (nr_of_cross_section_points - 1) + j );
       Vec3f face0_normal = normals_per_face[ lower_index ];
       Vec3f face1_normal = normals_per_face[ lower_index + 1 ];
-      bool begin_cap_summed = false;
-      bool end_cap_summed = false;
 
       //  vertex 0
       normal_vector.push_back( Vec3f( 0, 0, 0 ) );
@@ -1241,8 +1232,6 @@ Normal * Extrusion::AutoNormal::generateNormalsPerVertex(
 
 
       //  vertex 1
-      begin_cap_summed = false;
-      end_cap_summed = false;
       normal_vector.push_back( Vec3f( 0, 0, 0 ) );
       i_index = i;
       j_index = j + 1;
@@ -1262,8 +1251,6 @@ Normal * Extrusion::AutoNormal::generateNormalsPerVertex(
       normal_vector.back().normalizeSafe();
 
       //  vertex 2
-      begin_cap_summed = false;
-      end_cap_summed = false;
       normal_vector.push_back( Vec3f( 0, 0, 0 ) );
       Vec3f face1_vertex2( 0, 0, 0 );
       i_index = i + 1;
@@ -1287,8 +1274,6 @@ Normal * Extrusion::AutoNormal::generateNormalsPerVertex(
       face1_vertex2.normalizeSafe();
 
       //  vertex 3
-      begin_cap_summed = false;
-      end_cap_summed = false;
       normal_vector.push_back( face1_vertex0 );
       normal_vector.push_back( face1_vertex2 );
       normal_vector.push_back( Vec3f( 0, 0, 0 ) );
@@ -1317,16 +1302,16 @@ Normal * Extrusion::AutoNormal::generateNormalsPerVertex(
     H3DInt32 theLastVertexNormal = (H3DInt32)normals_per_face.size() - 1;
     Vec3f face_normal = normals_per_face.back();
     normal_vector.push_back( face_normal );
-    Vec3f quad_normal = 
+    Vec3f quad_normal =
       normals_per_face[ theLastVertexNormal - 2 * ( nr_of_cross_section_points - 1 ) + 1 ];
     if( quad_normal * face_normal > cos_crease_angle )
       normal_vector.back() += quad_normal;
     if( closed_cross_section ) {
-      quad_normal = 
+      quad_normal =
         normals_per_face[ theLastVertexNormal - 1 ];
       if( quad_normal * face_normal > cos_crease_angle )
         normal_vector.back() += quad_normal;
-      quad_normal = 
+      quad_normal =
         normals_per_face[ theLastVertexNormal - 2 ];
       if( quad_normal * face_normal > cos_crease_angle )
         normal_vector.back() += quad_normal;
@@ -1336,7 +1321,7 @@ Normal * Extrusion::AutoNormal::generateNormalsPerVertex(
     for( int i = 1; i < nr_of_cross_section_points - 1; ++i )  {
       normal_vector.push_back( face_normal );
 
-      quad_normal = 
+      quad_normal =
         normals_per_face[ theLastVertexNormal -
                           2 * ( nr_of_cross_section_points - i - 1 ) + 1 ];
       if( quad_normal * face_normal > cos_crease_angle )
@@ -1363,7 +1348,7 @@ Normal * Extrusion::AutoNormal::generateNormalsPerVertex(
     if( quad_normal * face_normal > cos_crease_angle )
       normal_vector.back() += quad_normal;
     if( closed_cross_section ) {
-      quad_normal = 
+      quad_normal =
         normals_per_face[ theLastVertexNormal - 2 * ( nr_of_cross_section_points - 1 ) + 1 ];
       if( quad_normal * face_normal > cos_crease_angle )
         normal_vector.back() += quad_normal;
@@ -1377,9 +1362,9 @@ Normal * Extrusion::AutoNormal::generateNormalsPerVertex(
 }
 
 void Extrusion::SFBound::update() {
-  const vector< Vec3f > &vertex_vectors = 
+  const vector< Vec3f > &vertex_vectors =
     static_cast< MFVec3f * >( routes_in[0] )->getValue();
-  
+
   BoxBound *bb = new BoxBound;
   bb->fitAroundPoints( vertex_vectors.begin(), vertex_vectors.end() );
   value = bb;
