@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -53,7 +53,7 @@ H3DNodeDatabase GLUTWindow::database( "GLUTWindow",
                                       &(H3DWindowNode::database) );
 
 namespace GLUTWindowInternals {
-  FIELDDB_ELEMENT( GLUTWindow, gameMode, INITIALIZE_ONLY );
+  FIELDDB_ELEMENT( GLUTWindow, gameMode, INITIALIZE_ONLY )
 }
 
 /////////////////////////////////////////////////////////////////////////////
@@ -102,7 +102,8 @@ GLUTWindow::GLUTWindow( Inst< SFInt32     > _width,
   last_x_pos( -1 ),
   last_y_pos( -1 ),
   last_width( -1 ),
-  last_height( -1 ) {
+  last_height( -1 ),
+  window_id( 0 ) {
   
   type_name = "GLUTWindow";
   database.initFields( this );
@@ -132,7 +133,8 @@ GLUTWindow::GLUTWindow( Inst< SFInt32     > _width,
   last_fullscreen = fullscreen->getValue();
 }
 
-GLUTWindow::~GLUTWindow() {
+void GLUTWindow::deinitWindow() {
+  H3DWindowNode::deinitWindow();
   if ( window_id > 0 ) {
     // If the window is closed with the x-button glut can be deinitialized
     // before getting here, so make sure glut is initialized before calling
@@ -142,14 +144,21 @@ GLUTWindow::~GLUTWindow() {
 #else
     if( GLUT_init ) {
 #endif
+
       glutDestroyWindow( window_id );
+      window_id = 0;
     }
   }
 }
 
+
+GLUTWindow::~GLUTWindow() {
+  deinitWindow();
+}
+
 void GLUTWindow::initWindow() {
   if( isInitialized() ) {
-    Console(4) << "GLUTWindow does not support changing pixel format from/to "
+    Console(LogLevel::Error) << "GLUTWindow does not support changing pixel format from/to "
                << "quad buffered stereo support after initialization." << endl;
     if( last_render_mode == RenderMode::QUAD_BUFFERED_STEREO ) {
       renderMode->setValue( "QUAD_BUFFERED_STEREO" );
@@ -173,13 +182,18 @@ void GLUTWindow::initWindow() {
   } else if( stereo_mode == RenderMode::QUAD_BUFFERED_STEREO ) {
     mode |= GLUT_STEREO;
   }
-  
+
   glutInitDisplayMode( mode );
-  if( gameMode->getValue() == "" ) {
+
+  const string& game_mode = gameMode->getValue();
+  if( game_mode == "" ) {
     glutInitWindowSize( width->getValue(), height->getValue() );
     glutInitWindowPosition( posX->getValue(), posY->getValue() );
+#ifdef FREEGLUT
+    glutInitContextVersion(3, 1); // There is no pure GLUT replacement for this since GLUT is too old.
+    glutInitContextFlags(GLUT_CORE_PROFILE);
+#endif
     window_id = glutCreateWindow( "H3D" );
-    setFullscreen( fullscreen->getValue() );
     // This code is here to check if we got a stereo window using stereo mode.
     // It seems like on some systems GLUT_MULTISAMPLE can not be combined with
     // GLUT_STEREO.
@@ -196,11 +210,12 @@ void GLUTWindow::initWindow() {
         glGetBooleanv( GL_STEREO, &quad_stereo_supported);
       }
     }
+    setFullscreen( fullscreen->getValue() );
     glutSetWindow( window_id );
   } else {
     glutGameModeString(gameMode->getValue().c_str());
     if( ! glutGameModeGet(GLUT_GAME_MODE_POSSIBLE) ) {
-      Console(4) << "The selected GLUT gamemode ("
+      Console(LogLevel::Error) << "The selected GLUT gamemode ("
                  << gameMode->getValue()
                  << ") is not available\n"
                  << std::endl;
@@ -234,6 +249,9 @@ void GLUTWindow::initWindow() {
    }
 #endif // H3D_WINDOWS
 
+
+  glutWMCloseFunc(glutWMCloseCallback);
+
 #ifdef FREEGLUT
   glutSetOption( GLUT_ACTION_ON_WINDOW_CLOSE, 
                  GLUT_ACTION_GLUTMAINLOOP_RETURNS );
@@ -252,6 +270,11 @@ void GLUTWindow::initWindow() {
 #endif // H3D_WINDOWS
 #endif // FREEGLUT
   window_is_made_active = true;
+
+  GLint num_samples;
+  glGetIntegerv(GL_SAMPLES, &num_samples);
+
+  numSamples->setValue(num_samples, id);
 }
 
 
@@ -267,9 +290,9 @@ GLUTWindow *GLUTWindow::getGLUTWindow( int glut_id ) {
   return NULL;
 }
 
-void GLUTWindow::setFullscreen( bool fullscreen ) {
+void GLUTWindow::setFullscreen( bool _fullscreen ) {
   if( gameMode->getValue() == "" ) {
-    if( fullscreen != last_fullscreen ) {
+    if( _fullscreen != last_fullscreen ) {
 // Note: This define check is needed since freeglut does not have a
 // working version compile time definition. freegluts svn repository
 // was checked and GLUT_FULL_SCREEN was added at the same time as
@@ -277,7 +300,7 @@ void GLUTWindow::setFullscreen( bool fullscreen ) {
 #if defined( FREEGLUT ) && defined( GLUT_FULL_SCREEN )
       glutFullScreenToggle();
 #else
-      if( fullscreen ) {
+      if(_fullscreen) {
         last_x_pos = glutGet( GLUT_WINDOW_X );
         last_y_pos = glutGet( GLUT_WINDOW_Y );
         last_width = width->getValue();
@@ -288,7 +311,7 @@ void GLUTWindow::setFullscreen( bool fullscreen ) {
         glutPositionWindow(last_x_pos,last_y_pos);
       }
 #endif
-      last_fullscreen = fullscreen;
+      last_fullscreen = _fullscreen;
     }
   }
 }
@@ -533,6 +556,18 @@ void GLUTWindow::glutMouseWheelCallback( int wheel,
   }
 }
 
+void GLUTWindow::glutWMCloseCallback() {
+  GLUTWindow *window = GLUTWindow::getGLUTWindow(glutGetWindow());
+  if (window) {
+    // Make sure we clean up our scene before deinitWindow, because after that the opengl context won't exist anymore
+    for (set< Scene * >::iterator i = Scene::scenes.begin(); i != Scene::scenes.end(); ++i) {
+      (*i)->setSceneRoot(NULL);
+    }
+    window->window_id = 0;
+    window->deinitWindow();
+  }
+}
+
 string GLUTWindow::getCursorForMode( const string &mode ) {
   if( mode == "ON_SENSOR_OVER" ) {
     return "CROSSHAIR";
