@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -52,13 +52,15 @@ H3DDisplayListObject::DisplayList::DisplayList():
   cache_mode( OPTIONS ),
   frustum_culling_mode( OPTIONS ),
   have_valid_display_list( false ),
-  isActive( new IsActive ){
+  isActive( new IsActive ),
+  graphic_options_previous( NULL ) {
   initGraphicOption();
   reset_delay_cache_counter = true;
   delay_cache_counter = cachingDelay();
   isActive->setValue( true );
   isActive->setName( "H3DDisplayListObject::isActive" );
   Scene::time->routeNoEvent( isActive );
+  break_list_field->setName( "H3DDisplayListObject::break_list_field" );
 
 #ifndef H3D_GENERATE_DOTROUTE_FILES
   break_list_field->route( this );
@@ -91,7 +93,7 @@ bool H3DDisplayListObject::DisplayList::tryBuildDisplayList( bool cache_broken )
       glNewList( display_list, GL_COMPILE_AND_EXECUTE );
       GLuint err = glGetError();
       if( err != GL_NO_ERROR ) {
-        Console(4) << "OpenGL error in glNewList() Error: \"" << gluErrorString( err ) 
+        Console(LogLevel::Error) << "OpenGL error in glNewList() Error: \"" << gluErrorString( err ) 
                    << "\" when rendering " << getFullName() << endl;
         return false;
       }
@@ -99,7 +101,7 @@ bool H3DDisplayListObject::DisplayList::tryBuildDisplayList( bool cache_broken )
       glEndList();
       err = glGetError();
       if( err != GL_NO_ERROR ) {
-        Console(4) << "OpenGL error in glEndList() Error: \"" << gluErrorString( err ) 
+        Console(LogLevel::Error) << "OpenGL error in glEndList() Error: \"" << gluErrorString( err ) 
                    << "\" when rendering " << getFullName() << endl;
         return false;
       }
@@ -119,17 +121,27 @@ void H3DDisplayListObject::DisplayList::propagateEvent( Event e ) {
   Field::propagateEvent( e );
   have_valid_display_list = false;
   reset_delay_cache_counter = true;
+#ifdef DEPRECATE_DISPLAYLIST_EVENT_COLLECT
+  // do nothing
+#else
   event_fields.insert( e.ptr );
-  /*Console(4)<<"display list object have a event from :"<<e.ptr->getFullName()<<endl;*/
+  /*Console(LogLevel::Error)<<"display list object have a event from :"<<e.ptr->getFullName()<<endl;*/
+#endif
 }
 
 void H3DDisplayListObject::DisplayList::callList( bool build_list ) {
+  #ifdef DISABLE_H3D_DISPLAYLIST
+  owner->render();
+  event_fields.clear();
+  #else
+  
+
   bool was_active = isActive->getValue();
   isActive->callListCalled();
 
   GLuint err = glGetError();
   if( err != GL_NO_ERROR ) {
-    Console(4) << "OpenGL error before H3DDisplayListObject::DisplayList::callList() Error: \"" << gluErrorString( err ) 
+    Console(LogLevel::Error) << "OpenGL error before H3DDisplayListObject::DisplayList::callList() Error: \"" << gluErrorString( err ) 
                << "\" when rendering parent of " << getFullName() << endl;
   }
 
@@ -165,24 +177,25 @@ void H3DDisplayListObject::DisplayList::callList( bool build_list ) {
   
   if ( using_caching && haveValidDisplayList() ) {
     glCallList( display_list );
-    GLuint err = glGetError();
+    err = glGetError();
     if( err != GL_NO_ERROR ) {
-      Console(4) << "OpenGL error in glCallList() Error: \"" << gluErrorString( err ) 
+      Console(LogLevel::Error) << "OpenGL error in glCallList() Error: \"" << gluErrorString( err ) 
                  << "\" when rendering " << getFullName() << endl;
       have_valid_display_list = false;
     }
   } else {
     owner->render();
     event_fields.clear();
-    GLuint err = glGetError();
+    err = glGetError();
     if( err != GL_NO_ERROR ) {
-      Console(4) << "OpenGL error in render() Error: \"" << gluErrorString( err ) 
+      Console(LogLevel::Error) << "OpenGL error in render() Error: \"" << gluErrorString( err ) 
                  << "\" when rendering " << getFullName() << endl;
     } else {
       if( delay_cache_counter > 0 ) 
         --delay_cache_counter;
     }
   }  
+  #endif // DISABLE_H3D_DISPLAYLIST
 }
 
 bool H3DDisplayListObject::DisplayList::childrenCachesReady( bool consider_active_field ) {
@@ -440,7 +453,7 @@ bool H3DDisplayListObject::DisplayList::isOutsideViewFrustum() {
       }
     }
     if( all_behind ) {
-      //Console(4) << "Culled " << plane << endl;
+      //Console(LogLevel::Error) << "Culled " << plane << endl;
       return true;
     }
   }
@@ -469,7 +482,7 @@ bool H3DDisplayListObject::DisplayList::usingCaching() {
   if( default_settings&&default_settings->optionNodesUpdated() ) {
     default_settings->getOptionNode( options );
     if( options ) {
-      //Console(4)<<"global setting update, update graphic option"<<endl;
+      //Console(LogLevel::Error)<<"global setting update, update graphic option"<<endl;
       // options exist in global setting and gs is updated
       // need to update graphic option
       graphic_options_previous = options;
