@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -45,39 +45,43 @@ H3DNodeDatabase H3DHapticsDevice::database( "H3DHapticsDevice",
                                          typeid( H3DHapticsDevice ) ); 
 
 namespace H3DHapticsDeviceInternals {
-  FIELDDB_ELEMENT( H3DHapticsDevice, initialized, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( H3DHapticsDevice, enabled, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( H3DHapticsDevice, set_enabled, INPUT_ONLY );
-  FIELDDB_ELEMENT( H3DHapticsDevice, deviceOrientation, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( H3DHapticsDevice, devicePosition, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( H3DHapticsDevice, trackerPosition, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( H3DHapticsDevice, trackerOrientation, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( H3DHapticsDevice, positionCalibration, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( H3DHapticsDevice, orientationCalibration, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( H3DHapticsDevice, proxyPosition, OUTPUT_ONLY );
+  FIELDDB_ELEMENT( H3DHapticsDevice, initialized, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( H3DHapticsDevice, enabled, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( H3DHapticsDevice, set_enabled, INPUT_ONLY )
+  FIELDDB_ELEMENT( H3DHapticsDevice, deviceOrientation, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( H3DHapticsDevice, devicePosition, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( H3DHapticsDevice, trackerPosition, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( H3DHapticsDevice, trackerOrientation, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( H3DHapticsDevice, positionCalibration, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( H3DHapticsDevice, orientationCalibration, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( H3DHapticsDevice, proxyPosition, OUTPUT_ONLY )
 #ifdef HAVE_PROFILER
-  FIELDDB_ELEMENT( H3DHapticsDevice, profiledResult, INPUT_OUTPUT);
+  FIELDDB_ELEMENT( H3DHapticsDevice, profiledResult, INPUT_OUTPUT)
 #endif 
-  FIELDDB_ELEMENT( H3DHapticsDevice, weightedProxyPosition, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( H3DHapticsDevice, proxyWeighting, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( H3DHapticsDevice, mainButton, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( H3DHapticsDevice, secondaryButton, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( H3DHapticsDevice, buttons, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( H3DHapticsDevice, force, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( H3DHapticsDevice, torque, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( H3DHapticsDevice, inputDOF, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( H3DHapticsDevice, outputDOF, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( H3DHapticsDevice, hapticsRate, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( H3DHapticsDevice, desiredHapticsRate, INITIALIZE_ONLY );
-  FIELDDB_ELEMENT( H3DHapticsDevice, stylus, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( H3DHapticsDevice, hapticsRenderer, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( H3DHapticsDevice, proxyPositions, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( H3DHapticsDevice, followViewpoint, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( H3DHapticsDevice, deviceVelocity, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( H3DHapticsDevice, trackerVelocity, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( H3DHapticsDevice, hapticsLoopTime, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( H3DHapticsDevice, forceLimit, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( H3DHapticsDevice, torqueLimit, INPUT_OUTPUT );
+  FIELDDB_ELEMENT( H3DHapticsDevice, weightedProxyPosition, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( H3DHapticsDevice, proxyWeighting, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( H3DHapticsDevice, mainButton, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( H3DHapticsDevice, secondaryButton, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( H3DHapticsDevice, buttons, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( H3DHapticsDevice, force, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( H3DHapticsDevice, torque, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( H3DHapticsDevice, inputDOF, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( H3DHapticsDevice, outputDOF, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( H3DHapticsDevice, hapticsRate, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( H3DHapticsDevice, desiredHapticsRate, INITIALIZE_ONLY )
+  FIELDDB_ELEMENT( H3DHapticsDevice, stylus, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( H3DHapticsDevice, hapticsRenderer, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( H3DHapticsDevice, proxyPositions, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( H3DHapticsDevice, followViewpoint, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( H3DHapticsDevice, deviceVelocity, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( H3DHapticsDevice, trackerVelocity, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( H3DHapticsDevice, hapticsLoopTime, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( H3DHapticsDevice, deadmansSwitch, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( H3DHapticsDevice, forceLimit, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( H3DHapticsDevice, torqueLimit, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( H3DHapticsDevice, forceScale, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( H3DHapticsDevice, deviceAngularVelocity, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( H3DHapticsDevice, trackerAngularVelocity, OUTPUT_ONLY )
 }
 
 
@@ -87,8 +91,8 @@ H3DHapticsDevice::H3DHapticsDevice(
                Inst< SFRotation      > _deviceOrientation      ,
                Inst< TrackerPosition > _trackerPosition        ,
                Inst< TrackerOrientation > _trackerOrientation  ,
-               Inst< PosCalibration  > _positionCalibration    ,
-               Inst< OrnCalibration  > _orientationCalibration ,
+               Inst< SFMatrix4f  > _positionCalibration    ,
+               Inst< SFRotation  > _orientationCalibration ,
                Inst< SFVec3f         > _proxyPosition          ,
                Inst< WeightedProxy   > _weightedProxyPosition  ,     
                Inst< SFFloat         > _proxyWeighting         ,
@@ -106,11 +110,12 @@ H3DHapticsDevice::H3DHapticsDevice(
                Inst< MFVec3f         > _proxyPositions,
                Inst< SFBool          > _followViewpoint,
                Inst< SFVec3f         > _deviceVelocity,
-               Inst< TrackerVelocity > _trackerVelocity
+               Inst< TrackerVelocity > _trackerVelocity,
                #ifdef HAVE_PROFILER
-               ,
-               Inst< SFString        > _profiledResult
+               Inst< SFString        > _profiledResult,
                #endif
+               Inst< SFVec3f         > _deviceAngularVelocity,
+               Inst< TrackerAngularVelocity > _trackerAngularVelocity
                ):
   devicePosition( _devicePosition ),
 #ifdef HAVE_PROFILER
@@ -120,9 +125,9 @@ H3DHapticsDevice::H3DHapticsDevice(
   trackerPosition( _trackerPosition ),
   trackerOrientation( _trackerOrientation ),
   positionCalibration( _positionCalibration ),
-  adjustedPositionCalibration( new PosCalibration ),
+  adjustedPositionCalibration( new SFMatrix4f ),
   orientationCalibration( _orientationCalibration ),
-  adjustedOrnCalibration( new OrnCalibration ),
+  adjustedOrnCalibration( new SFRotation ),
   proxyPosition( _proxyPosition ),
   weightedProxyPosition( _weightedProxyPosition ),
   proxyWeighting( _proxyWeighting ),
@@ -145,9 +150,13 @@ H3DHapticsDevice::H3DHapticsDevice(
   followViewpoint( _followViewpoint ),
   deviceVelocity( _deviceVelocity ),
   trackerVelocity( _trackerVelocity ),
+  deadmansSwitch( new SFBool ),
   forceLimit( new SFFloat ),
   torqueLimit( new SFFloat ),
-  error_msg_printed( false ) {
+  forceScale( new SFFloat ),
+  error_msg_printed( false ),
+  deviceAngularVelocity( _deviceAngularVelocity ),
+  trackerAngularVelocity( _trackerAngularVelocity ) {
 
   type_name = "H3DHapticsDevice";  
   database.initFields( this );
@@ -165,8 +174,10 @@ H3DHapticsDevice::H3DHapticsDevice(
   hapticsLoopTime->setValue( -1, id );
   inputDOF->setValue( 3, id );
   outputDOF->setValue( 3, id );
+  deadmansSwitch->setValue( false, id );
   forceLimit->setValue( -1, id );
   torqueLimit->setValue( -1, id );
+  forceScale->setValue( 1, id );
 
   // Even though this is an input only field a default value
   // should be set to false since onValueChanged is used which is
@@ -188,6 +199,8 @@ H3DHapticsDevice::H3DHapticsDevice(
   proxyWeighting->route( weightedProxyPosition, id );
 
   vp_initialized = 0;
+  orientationCalibration->route( trackerAngularVelocity, id );
+  deviceAngularVelocity->route( trackerAngularVelocity, id );
 }
 
 H3DHapticsDevice::ErrorCode H3DHapticsDevice::enableDevice() {
@@ -226,7 +239,7 @@ H3DHapticsDevice::ErrorCode H3DHapticsDevice::initDevice() {
         initialized->setValue( true, id );
       } else {
         if( !error_msg_printed ) {
-          Console(4) << hapi_device->getLastErrorMsg() << endl;
+          Console(LogLevel::Error) << hapi_device->getLastErrorMsg() << endl;
           error_msg_printed = true;
         }
       }
@@ -280,9 +293,8 @@ void H3DHapticsDevice::renderShapes(
 }
 
 void H3DHapticsDevice::updateDeviceValues() {
-  previuos_proxy_pos = proxyPositions->getValue();
+  previous_proxy_pos = proxyPositions->getValue();
   TimeStamp now = TimeStamp();
-  TimeStamp dt = now - last_update_values;
   last_update_values = now;
 
   if( hapi_device.get() ) { 
@@ -296,8 +308,18 @@ void H3DHapticsDevice::updateDeviceValues() {
     positionCalibration->upToDate();
     orientationCalibration->upToDate();
 
-    hapi_device->setForceLimit( forceLimit->getValue() );
-    hapi_device->setTorqueLimit( torqueLimit->getValue() );
+    if (deadmansSwitch->getValue() && !mainButton->getValue())
+    {
+        hapi_device->setForceLimit(0.0);
+        hapi_device->setTorqueLimit(0.0);
+    }
+    else
+    {
+        hapi_device->setForceLimit( forceLimit->getValue() );
+        hapi_device->setTorqueLimit( torqueLimit->getValue() );
+    }
+
+    hapi_device->setForceScale( forceScale->getValue() );
 
     X3DViewpointNode *vp = X3DViewpointNode::getActive();
     vp = H3DNavigation::viewpointToUse( vp, 0 );
@@ -346,20 +368,20 @@ void H3DHapticsDevice::updateDeviceValues() {
         setValue( adjust_matrix * positionCalibration->getValue() );
 
       hapi_device->setPositionCalibration( 
-        adjustedPositionCalibration->rt_pos_calibration );
+        adjustedPositionCalibration->getValue() );
       
-      // Create adjusted OrnCalibration and send to HAPI
+      // Create adjustedOrnCalibration and send to HAPI
       adjustedOrnCalibration->setValue(
         Rotation( vp_full_orn_mtx ) *
         orientationCalibration->getValue() );
       hapi_device->
-        setOrientationCalibration(adjustedOrnCalibration->rt_orn_calibration );
+        setOrientationCalibration(adjustedOrnCalibration->getValue() );
     }
     else {
       hapi_device->
-        setPositionCalibration( positionCalibration->rt_pos_calibration );
+        setPositionCalibration( positionCalibration->getValue() );
       hapi_device->setOrientationCalibration(
-        orientationCalibration->rt_orn_calibration );
+        orientationCalibration->getValue() );
     }
 
     H3DInt32 hr = hapi_device->getHapticsRate();
@@ -371,9 +393,12 @@ void H3DHapticsDevice::updateDeviceValues() {
     devicePosition->setValue( Vec3f( dv.position ), id);
     deviceVelocity->setValue( Vec3f( dv.velocity ), id);
     deviceOrientation->setValue( Rotation(dv.orientation), id);
+    deviceAngularVelocity->setValue( Vec3f( dv.angular_velocity ), id);
     force->setValue( (Vec3f)dv.force, id);
     torque->setValue( (Vec3f)dv.torque, id);
-    buttons->setValue( dv.button_status, id );
+    if( dv.button_status != buttons->getValue() ) {
+      buttons->setValue( dv.button_status, id );
+    }
     // set mainButton and secondaryButton
     bool temp_value = (dv.button_status & 0x01) != 0;
     if( temp_value != mainButton->getValue() )
@@ -382,6 +407,9 @@ void H3DHapticsDevice::updateDeviceValues() {
     if( temp_value != secondaryButton->getValue() )
       secondaryButton->setValue( temp_value, id );
 
+    if (deadmansSwitch->getValue() && !mainButton->getValue())
+      return;
+
     vector< Vec3f > proxies;
 
     for( unsigned int layer = 0; layer < hapi_device->nrLayers(); ++layer ) {
@@ -394,7 +422,6 @@ void H3DHapticsDevice::updateDeviceValues() {
         Vec3f proxy_pos = Vec3f( proxy_renderer->getProxyPosition() );
         proxies.push_back( proxy_pos );
         if( layer == 0 ) {
-          //cerr << proxy_pos << endl;
           proxyPosition->setValue( proxy_pos, id );
         }
       } else {
@@ -418,100 +445,100 @@ void H3DHapticsDevice::updateDeviceValues() {
           device_index = i;
       }
     }
-    assert( device_index != -1 );
 
-    HAPI::HAPIHapticsRenderer::Contacts all_contacts;
+    // Have to do this check since ClutchedHapticsDevice actually call H3DHapticsDevice::updateDeviceValues
+    // and since the ClutchedHapticsDevice is the one that is in DeviceInfo.
+    // Also what if we in the future create a similar device, having an assert will give issues in debug.
+    if( device_index > -1 ) {
 
-    for( unsigned int layer = 0; layer < hapi_device->nrLayers(); ++layer ) {
-      HAPI::HAPIHapticsRenderer *renderer = 
-        hapi_device->getHapticsRenderer( layer );
-      HAPI::HAPIHapticsRenderer::Contacts contacts;
-      if( renderer ) {
-        contacts = renderer->getContacts();
-        all_contacts.insert( all_contacts.end(),
-                             contacts.begin(),
-                             contacts.end() );
-      }
-      for( HAPI::HAPIHapticsRenderer::Contacts::iterator i = contacts.begin();
-           i != contacts.end(); ++i ) {
-        X3DGeometryNode *geom =
-          static_cast< X3DGeometryNode * >( (*i).first->getUserData() );
+      HAPI::HAPIHapticsRenderer::Contacts all_contacts;
+
+      for( unsigned int layer = 0; layer < hapi_device->nrLayers(); ++layer ) {
+        HAPI::HAPIHapticsRenderer *renderer = 
+          hapi_device->getHapticsRenderer( layer );
+        HAPI::HAPIHapticsRenderer::Contacts contacts;
+        if( renderer ) {
+          contacts = renderer->getContacts();
+          all_contacts.insert( all_contacts.end(),
+                               contacts.begin(),
+                               contacts.end() );
+        }
+        for( HAPI::HAPIHapticsRenderer::Contacts::iterator i = contacts.begin();
+             i != contacts.end(); ++i ) {
+          X3DGeometryNode *geom =
+            static_cast< X3DGeometryNode * >( (*i).first->getUserData() );
       
-        // make sure all fields have the right size
-        if( device_index > (int)geom->force->size() -1 )
-          geom->force->resize( device_index + 1, Vec3f( 0, 0, 0 ), geom->id );
-        if( device_index > (int)geom->contactPoint->size() -1 )
-          geom->contactPoint->resize( device_index + 1, Vec3f( 0, 0, 0 ),
-                                      geom->id );
-        if( device_index > (int)geom->contactNormal->size() -1 )
-          geom->contactNormal->resize( device_index + 1, Vec3f( 1, 0, 0 ),
-                                       geom->id );
-         if( device_index > (int)geom->contactTexCoord->size() -1 )
-          geom->contactTexCoord->resize( device_index + 1, Vec3f( 0, 0, 0 ),
+          // make sure all fields have the right size
+          if( device_index > (int)geom->force->size() -1 )
+            geom->force->resize( device_index + 1, Vec3f( 0, 0, 0 ), geom->id );
+          if( device_index > (int)geom->contactPoint->size() -1 )
+            geom->contactPoint->resize( device_index + 1, Vec3f( 0, 0, 0 ),
+                                        geom->id );
+          if( device_index > (int)geom->contactNormal->size() -1 )
+            geom->contactNormal->resize( device_index + 1, Vec3f( 1, 0, 0 ),
                                          geom->id );
-        if( device_index > (int)geom->isTouched->size() -1 )
-          geom->isTouched->resize( device_index + 1, false, geom->id );
+           if( device_index > (int)geom->contactTexCoord->size() -1 )
+            geom->contactTexCoord->resize( device_index + 1, Vec3f( 0, 0, 0 ),
+                                           geom->id );
+          if( device_index > (int)geom->isTouched->size() -1 )
+            geom->isTouched->resize( device_index + 1, false, geom->id );
         
-        HAPI::HAPISurfaceObject::ContactInfo ci = (*i).second;
+          HAPI::HAPISurfaceObject::ContactInfo ci = (*i).second;
 
-        // TODO: should be able to do it in a faster/better way.
-        Matrix4d global_to_local = (*i).first->getInverse();
-        Matrix3d global_vec_to_local = global_to_local.getRotationPart();
+          Matrix4d global_to_local = (*i).first->getInverse();
+          Matrix3d global_vec_to_local = global_to_local.getRotationPart();
 
-        Vec3f cp( global_to_local * ci.globalSurfaceContactPoint() );
+          Vec3f cp( global_to_local * ci.globalSurfaceContactPoint() );
 
-        if( geom->contactPoint->getValueByIndex( device_index ) != cp )
-          geom->contactPoint->setValue( device_index, cp, geom->id );
+          if( geom->contactPoint->getValueByIndex( device_index ) != cp )
+            geom->contactPoint->setValue( device_index, cp, geom->id );
 
-        Vec3f n( global_vec_to_local * ci.yAxis() );
+          Vec3f n( global_vec_to_local * ci.yAxis() );
 
-        if( geom->contactNormal->getValueByIndex( device_index ) != n ) 
-          geom->contactNormal->setValue( device_index, n, geom->id ); 
+          if( geom->contactNormal->getValueByIndex( device_index ) != n ) 
+            geom->contactNormal->setValue( device_index, n, geom->id ); 
 
-        if( geom->contactTexCoord->getValueByIndex( device_index ) !=
-            ci.contactPointTexCoord() ) 
-          geom->contactTexCoord->setValue( device_index,
-                                           (Vec3f) ci.contactPointTexCoord(),
-                                           geom->id ); 
+          if( geom->contactTexCoord->getValueByIndex( device_index ) !=
+              ci.contactPointTexCoord() ) 
+            geom->contactTexCoord->setValue( device_index,
+                                             (Vec3f) ci.contactPointTexCoord(),
+                                             geom->id ); 
         
-        Vec3f f( global_vec_to_local * ci.globalForce() );
+          Vec3f f( global_vec_to_local * ci.globalForce() );
 
-        if( geom->force->getValueByIndex( device_index ) != f ) 
-          geom->force->setValue(device_index, f, geom->id ); 
+          if( geom->force->getValueByIndex( device_index ) != f ) 
+            geom->force->setValue(device_index, f, geom->id ); 
 
-        bool contact_last_time = 
-          geom->isTouched->getValueByIndex( device_index );
-        if( !contact_last_time ) 
-          geom->isTouched->setValue( device_index, true, geom->id );
-      }
-    }
-
-    for( HAPI::HAPIHapticsRenderer::Contacts::iterator j =
-          last_contacts.begin();
-         j != last_contacts.end(); ++j ) {
-      bool still_in_contact = false;
-      for( HAPI::HAPIHapticsRenderer::Contacts::iterator i =
-            all_contacts.begin();
-           i != all_contacts.end(); ++i ) {
-        if( (*i).first->getUserData() == (*j).first->getUserData() ) {
-          still_in_contact = true;
-          break;
+          bool contact_last_time = 
+            geom->isTouched->getValueByIndex( device_index );
+          if( !contact_last_time ) 
+            geom->isTouched->setValue( device_index, true, geom->id );
         }
       }
-      if( !still_in_contact ) {
-        X3DGeometryNode *geom = 
-          static_cast< X3DGeometryNode * >((*j).first->getUserData() );
-        geom->isTouched->setValue( device_index, false, geom->id );
-        geom->force->setValue( device_index, Vec3f(), geom->id );
+
+      for( HAPI::HAPIHapticsRenderer::Contacts::iterator j =
+            last_contacts.begin();
+           j != last_contacts.end(); ++j ) {
+        bool still_in_contact = false;
+        for( HAPI::HAPIHapticsRenderer::Contacts::iterator i =
+              all_contacts.begin();
+             i != all_contacts.end(); ++i ) {
+          if( (*i).first->getUserData() == (*j).first->getUserData() ) {
+            still_in_contact = true;
+            break;
+          }
+        }
+        if( !still_in_contact ) {
+          X3DGeometryNode *geom = 
+            static_cast< X3DGeometryNode * >((*j).first->getUserData() );
+          geom->isTouched->setValue( device_index, false, geom->id );
+          geom->force->setValue( device_index, Vec3f(), geom->id );
+        }
       }
+      last_contacts.swap( all_contacts );
     }
-    last_contacts.swap( all_contacts );
 
   }
-
-  //cerr << hr << endl;
-  // TODO: lock
-  //proxyPosition->setValue( (Vec3f)proxy_position, id );
 }
 
 void H3DHapticsDevice::renderStylus() {
@@ -519,11 +546,13 @@ void H3DHapticsDevice::renderStylus() {
   if( stylus_node && initialized->getValue() ) {
     const Vec3f &pos = weightedProxyPosition->getValue();
     const Rotation &rot = trackerOrientation->getValue();
+    glMatrixMode( GL_MODELVIEW );
     glPushMatrix();
     glTranslatef( pos.x, pos.y, pos.z );
     glRotatef( (GLfloat)(rot.angle * 180 / Constants::pi), 
                rot.axis.x, rot.axis.y, rot.axis.z );
     stylus_node->render();
+    glMatrixMode( GL_MODELVIEW );
     glPopMatrix();
   }
 }
