@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -42,10 +42,11 @@ H3DNavigation::H3DNavigation() : haptic_device_nav( NULL ),
                                  sws_navigation( NULL ),
                                  linear_interpolate( false ),
                                  old_vp( NULL ),
-                                 last_time( 0 ) {
+                                 last_time( 0 ),
+                                 start_time( 0 ) {
   mouse_nav.reset( new MouseNavigation( this ) );
   keyboard_nav.reset( new KeyboardNavigation( this ) );
-
+  AUTOREF_DEBUG_NAME( old_vp, "H3DNavigation::old_vp")
   h3d_navigations.push_back( this );
 }
 
@@ -145,7 +146,7 @@ void H3DNavigation::doNavigation(
 
   if( !linear_interpolate ) {
 
-    Rotation vp_full_orientation = vp->totalOrientation->getValue();
+    vp_full_orientation = vp->totalOrientation->getValue();
     H3DTime delta_time = current_time - last_time;
     last_time = current_time;
 
@@ -165,9 +166,15 @@ void H3DNavigation::doNavigation(
           if( max_movement < Constants::f_epsilon ) {
             // Should never come here unless user manually aligns the
             // position of the viewpoint with the center of rotation.
-            // Just choose a direction to zoom out from.
-            direction = Vec3f( 1, 0, 0 );
-            max_movement = 1;
+            // If the translation_sum is greater than epsilon use that,
+            // otherwise just choose a direction to zoom out from.
+            if( move_info.translation_sum.length() < Constants::f_epsilon ){
+              direction = Vec3f( 0, 0, -1 );
+              max_movement = 1;
+            } else {
+              direction = -move_info.translation_sum;
+              max_movement = move_info.translation_sum.length();
+            }
           }
           direction = direction / max_movement;
           bool move_towards = true;
@@ -194,7 +201,7 @@ void H3DNavigation::doNavigation(
            }
            
            if( !isUniformScaling( scaling ) ) {
-              Console(3) << "Warning: Non-uniform scaling in the"
+              Console(LogLevel::Warning) << "Warning: Non-uniform scaling in the"
                          << " active X3DViewpointNode ( "
                          << vp->getName()
                          << " ) nodes local coordinate system. Speed and avatar size of "
@@ -233,7 +240,7 @@ void H3DNavigation::doNavigation(
       }
 
       if( !isUniformScaling( scaling ) ) {
-         Console(3) << "Warning: Non-uniform scaling in the"
+         Console(LogLevel::Warning) << "Warning: Non-uniform scaling in the"
                     << " active X3DViewpointNode ( "
                     << vp->getName()
                     << " ) nodes local coordinate system. Speed and avatar size of "
@@ -243,8 +250,6 @@ void H3DNavigation::doNavigation(
       H3DNavigationDevices::MoveInfo move_info;
       if( H3DNavigationDevices::getMoveInfo( move_info, this ) ) {
 
-        // TODO: maybe change this to allow for other devices to specify what
-        // to look at ( for example by supply a point and direction ).
         GLint viewport[4];
         GLdouble mvmatrix[16], projmatrix[16];
         GLdouble wx, wy, wz;
