@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -17,7 +17,7 @@
 //    along with H3D API; if not, write to the Free Software
 //    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
-//    A commercial license is also available. Please contact us at 
+//    A commercial license is also available. Please contact us at
 //    www.sensegraphics.com for more information.
 //
 //
@@ -38,36 +38,39 @@
 #include <H3D/CoordinateDouble.h>
 #include <H3D/Normal.h>
 #include <H3D/HAnimDisplacer.h>
+#include <H3DUtil/DualQuaternion.h>
 
 using namespace H3D;
+typedef H3DUtil::ArithmeticTypes::DualQuaternion DualQuaternion;
 
-H3DNodeDatabase HAnimHumanoid::database( 
-        "HAnimHumanoid", 
+H3DNodeDatabase HAnimHumanoid::database(
+        "HAnimHumanoid",
         newInstance<HAnimHumanoid>,
         typeid( HAnimHumanoid ),
-        &X3DChildNode::database 
+        &X3DChildNode::database
         );
 
+
 namespace HAnimHumanoidInternals {
-  FIELDDB_ELEMENT( HAnimHumanoid, bboxCenter, INITIALIZE_ONLY );
-  FIELDDB_ELEMENT( HAnimHumanoid, bboxSize, INITIALIZE_ONLY );
-  FIELDDB_ELEMENT( HAnimHumanoid, info, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( HAnimHumanoid, name, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( HAnimHumanoid, version, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( HAnimHumanoid, skin, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( HAnimHumanoid, joints, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( HAnimHumanoid, segments, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( HAnimHumanoid, sites, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( HAnimHumanoid, skeleton, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( HAnimHumanoid, viewpoints, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( HAnimHumanoid, skinCoord, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( HAnimHumanoid, skinNormal, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( HAnimHumanoid, center, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( HAnimHumanoid, rotation, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( HAnimHumanoid, scale, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( HAnimHumanoid, scaleOrientation, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( HAnimHumanoid, translation, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( HAnimHumanoid, renderMode, INPUT_OUTPUT );
+  FIELDDB_ELEMENT( HAnimHumanoid, bboxCenter, INITIALIZE_ONLY )
+  FIELDDB_ELEMENT( HAnimHumanoid, bboxSize, INITIALIZE_ONLY )
+  FIELDDB_ELEMENT( HAnimHumanoid, info, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( HAnimHumanoid, name, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( HAnimHumanoid, version, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( HAnimHumanoid, skin, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( HAnimHumanoid, joints, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( HAnimHumanoid, segments, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( HAnimHumanoid, sites, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( HAnimHumanoid, skeleton, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( HAnimHumanoid, viewpoints, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( HAnimHumanoid, skinCoord, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( HAnimHumanoid, skinNormal, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( HAnimHumanoid, center, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( HAnimHumanoid, rotation, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( HAnimHumanoid, scale, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( HAnimHumanoid, scaleOrientation, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( HAnimHumanoid, translation, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( HAnimHumanoid, renderMode, INPUT_OUTPUT )
 }
 
 HAnimHumanoid::HAnimHumanoid(  Inst< SFNode         > _metadata    ,
@@ -112,7 +115,7 @@ HAnimHumanoid::HAnimHumanoid(  Inst< SFNode         > _metadata    ,
   renderMode( _renderMode ),
   use_union_bound( false ),
   root_transform( NULL ),
-  joint_matrix_changed( new Field ) {
+  jointMatrixChanged( new Field ) {
 
   type_name = "HAnimHumanoid";
   database.initFields( this );
@@ -130,32 +133,211 @@ HAnimHumanoid::HAnimHumanoid(  Inst< SFNode         > _metadata    ,
   renderMode->addValidValue( "SKIN" );
   renderMode->addValidValue( "JOINTS" );
   renderMode->addValidValue( "SKELETON" );
+  renderMode->addValidValue( "SKIN_DLB" );
   renderMode->setValue( "SKIN" );
 
-  renderMode->route( displayList );
- }
+  jointMatrixChanged->setOwner( this );
+  jointMatrixChanged->setName( "jointMatrixChanged" );
 
+  renderMode->route( displayList );
+  renderMode->route( jointMatrixChanged );
+ }
 
 template< class VectorType >
 void HAnimHumanoid::updateCoordinates( const VectorType &orig_points,
                                        const vector< Vec3f > &orig_normals,
                                        VectorType &modified_points,
                                        vector< Vec3f > &modified_normals ) {
+  if( renderMode->getValue() == "SKIN_DLB" ) {
+    updateCoordinatesDLB( orig_points, orig_normals, modified_points, modified_normals );
+  } else {
+    updateCoordinatesLBS2( orig_points, orig_normals, modified_points, modified_normals );
+  }
+
+  const NodeVector &jts = joints->getValue();
+  // do displacer movements
+  for( unsigned int i = 0; i < jts.size(); ++i ) {
+    HAnimJoint *joint = static_cast< HAnimJoint* >( jts[i]);
+    if( joint ) {
+      const NodeVector &disp = joint->displacers->getValue();
+      if( disp.size() > 0 ) {
+        for( unsigned int j = 0; j < disp.size(); ++j ) {
+          HAnimDisplacer *displacer = static_cast< HAnimDisplacer* >( disp[j]);
+          if( displacer ) {
+            displacer->displaceCoordinates(modified_points,
+                                           joint->accumulatedJointMatrix->getValue() );
+          }
+        }
+      }
+    }
+  }
+
+}
+
+template< class VectorType >
+void HAnimHumanoid::updateCoordinatesDLB( const VectorType &orig_points,
+                                         const vector< Vec3f > &orig_normals,
+                                         VectorType &modified_points,
+                                         vector< Vec3f > &modified_normals ) {
 
   unsigned int p_size = (unsigned int) orig_points.size();
   unsigned int n_size = (unsigned int) orig_normals.size();
   unsigned int max_size = H3DMax( p_size, n_size );
-  vector< bool  > point_written( max_size, false ); 
+  vector< bool  > point_written( max_size, false );
+  vector< DualQuaternion > deform_quat ( max_size, DualQuaternion( Quaternion( 0, 0, 0, 0 ),
+                                                                   Quaternion( 0, 0, 0, 0 ) ) );
+  const NodeVector &jts = joints->getValue();
+
+  DualQuaternion first_joint_dq;
+
+  // blend dual quaternions
+  for( unsigned int i = 0; i < jts.size(); ++i ) {
+    HAnimJoint *joint = static_cast< HAnimJoint* >( jts[i]);
+    if( joint ) {
+      const vector<int> &indices = joint->skinCoordIndex->getValue();
+      const vector<H3DFloat> &weights = joint->skinCoordWeight->getValue();
+      Matrix4f joint_to_humanoid = joint->accumulatedJointMatrix->getValue();
+
+      // q and -q represents the same rigid transform. For blending we need to choose
+      // the representation with the same signs so we check here and make sure that
+      // they are.
+      DualQuaternion joint_dq( joint_to_humanoid );
+      if( i == 0 ) {
+        first_joint_dq = joint_dq;
+      } else {
+        if( first_joint_dq.q0.dotProduct( joint_dq.q0 ) < 0 ) {
+          joint_dq = -joint_dq;
+        }
+      }
+
+      for( unsigned int j = 0; j < indices.size(); ++j ) {
+        unsigned int index = indices[j];
+
+        if( index < p_size && j < weights.size() ) {
+           deform_quat[index] += weights[j] * joint_dq;
+        }
+
+        if( index < max_size ) {
+          point_written[index] = true;
+        }
+      }
+    }
+  }
+
+  // apply deform transform to all vertices
+  for( unsigned int i = 0; i < max_size; ++i ) {
+    if( point_written[i] ) {
+      //(see "Skinning with Dual Quaternion" paper by Kavan) for details
+      DualQuaternion &b = deform_quat[i];
+
+      /* This does not work. Don't know why as it is implemented from article. Code example from article is different though and
+      is what is used instead
+      H3DFloat b0_norm = b.q0.norm();
+      Quaternion c0 = b.q0 / b0_norm;
+      Quaternion ce = b.qe / b0_norm;
+
+      H3DFloat t0 = 2 * (-ce.w*c0.v.x + ce.v.x*c0.w   - ce.v.y*c0.v.z + ce.v.z*c0.v.y );
+      H3DFloat t1 = 2 * (-ce.w*c0.v.y + ce.v.x*c0.v.z + ce.v.y*c0.w   - ce.v.z*c0.v.x );
+      H3DFloat t2 = 2 * (-ce.w*c0.v.z - ce.v.x*c0.v.y + ce.v.y*c0.v.x + ce.v.z*c0.w   );
+
+      Matrix4f deform_matrix( 1 - 2*H3DSqr(c0.v.y) - 2*H3DSqr(c0.v.z), 2*c0.v.x*c0.v.y - 2*c0.w*c0.v.z        , 2*c0.v.x*c0.v.z + 2*c0.w*c0.v.y        , t0,
+                              2*c0.v.x*c0.v.y + 2*c0.w*c0.v.z        , 1 - 2*H3DSqr(c0.v.x) - 2*H3DSqr(c0.v.z), 2*c0.v.y*c0.v.z - 2*c0.w*c0.v.x        , t1,
+                              2*c0.v.x*c0.v.z - 2*c0.w*c0.v.y        , 2*c0.v.y*c0.v.z + 2*c0.w*c0.v.x        , 1 - 2*H3DSqr(c0.v.x) - 2*H3DSqr(c0.v.y), t2,
+                              0                                      , 0                                      , 0                                      , 1 );
+    */
+
+      Quaternion &c0 = b.q0;
+      Quaternion &ce = b.qe;
+
+      H3DFloat t0 = 2 * (-ce.w*c0.v.x + ce.v.x*c0.w   - ce.v.y*c0.v.z + ce.v.z*c0.v.y );
+      H3DFloat t1 = 2 * (-ce.w*c0.v.y + ce.v.x*c0.v.z + ce.v.y*c0.w   - ce.v.z*c0.v.x );
+      H3DFloat t2 = 2 * (-ce.w*c0.v.z - ce.v.x*c0.v.y + ce.v.y*c0.v.x + ce.v.z*c0.w   );
+
+      H3DFloat w = b.q0.w;
+      H3DFloat x = b.q0.v.x;
+      H3DFloat y = b.q0.v.y;
+      H3DFloat z = b.q0.v.z;
+
+      Matrix4f deform_matrix( w*w + x*x - y*y - z*z, 2*(x*y - w*z)        , 2*(x*z + w*y)        , t0,
+                              2*(x*y + w*z)        , w*w + y*y - x*x - z*z, 2*(y*z - w*x )       , t1,
+                              2*(x*z - w*y)        , 2*(y*z + w*x)        , w*w + z*z - x*x - y*y, t2,
+                              0                    , 0                    , 0                     , 1 );
+
+      H3DFloat len2 = c0.dotProduct(c0);
+	    if (len2 > 0.0f) len2 = (H3DFloat) 1.0 / len2;
+
+      if( i < p_size ) modified_points[i] = len2*(deform_matrix * orig_points[i]);
+      if( i < n_size ) modified_normals[i] = len2*(deform_matrix.getScaleRotationPart() * orig_normals[i]);
+    }
+  }
+}
+
+template< class VectorType >
+void HAnimHumanoid::updateCoordinatesLBS2( const VectorType &orig_points,
+                                         const vector< Vec3f > &orig_normals,
+                                         VectorType &modified_points,
+                                         vector< Vec3f > &modified_normals ) {
+
+  unsigned int p_size = (unsigned int) orig_points.size();
+  unsigned int n_size = (unsigned int) orig_normals.size();
+  unsigned int max_size = H3DMax( p_size, n_size );
+  vector< bool  > point_written( max_size, false );
+  vector< Matrix4f > deform_matrix ( max_size, Matrix4f( 0, 0, 0, 0,
+                                                         0, 0, 0, 0,
+                                                         0, 0, 0, 0,
+                                                         0, 0, 0, 0 ) );
+  const NodeVector &jts = joints->getValue();
+
+  // blend transform matrices
+  for( unsigned int i = 0; i < jts.size(); ++i ) {
+    HAnimJoint *joint = static_cast< HAnimJoint* >( jts[i]);
+    if( joint ) {
+      const vector<int> &indices = joint->skinCoordIndex->getValue();
+      const vector<H3DFloat> &weights = joint->skinCoordWeight->getValue();
+      Matrix4f joint_to_humanoid = joint->accumulatedJointMatrix->getValue();
+
+      for( unsigned int j = 0; j < indices.size(); ++j ) {
+        unsigned int index = indices[j];
+
+        if( index < p_size && j < weights.size() ) {
+            deform_matrix[index] += weights[j] * joint_to_humanoid;
+        }
+
+        if( index < max_size ) {
+          point_written[index] = true;
+        }
+      }
+    }
+  }
+
+  // apply deform transform to all vertices
+  for( unsigned int i = 0; i < max_size; ++i ) {
+    if( point_written[i] ) {
+      if( i < p_size ) modified_points[i] = deform_matrix[i] * orig_points[i];
+      if( i < n_size ) modified_normals[i] = deform_matrix[i].getScaleRotationPart() * orig_normals[i];
+    }
+  }
+}
+
+template< class VectorType >
+void HAnimHumanoid::updateCoordinatesLBS( const VectorType &orig_points,
+                                         const vector< Vec3f > &orig_normals,
+                                         VectorType &modified_points,
+                                         vector< Vec3f > &modified_normals ) {
+
+  unsigned int p_size = (unsigned int) orig_points.size();
+  unsigned int n_size = (unsigned int) orig_normals.size();
+  unsigned int max_size = H3DMax( p_size, n_size );
+  vector< bool  > point_written( max_size, false );
 
   const NodeVector &jts = joints->getValue();
- 
+
   // do joint movements
   for( unsigned int i = 0; i < jts.size(); ++i ) {
     HAnimJoint *joint = static_cast< HAnimJoint* >( jts[i]);
     if( joint ) {
       const vector<int> &indices = joint->skinCoordIndex->getValue();
       const vector<H3DFloat> &weights = joint->skinCoordWeight->getValue();
-      const Matrix4f &joint_to_global = joint->accumulatedForward->getValue();
       Matrix4f joint_to_humanoid = joint->accumulatedJointMatrix->getValue();
       Matrix3f joint_to_humanoid_rot = joint_to_humanoid.getRotationPart();
 
@@ -164,8 +346,8 @@ void HAnimHumanoid::updateCoordinates( const VectorType &orig_points,
 
         // point calculation
         if( index < p_size ) {
-          
-          typename VectorType::value_type weighted_point = 
+
+          typename VectorType::value_type weighted_point =
             joint_to_humanoid * orig_points[index];
           if( j < weights.size() )
             weighted_point *= weights[j];
@@ -187,33 +369,13 @@ void HAnimHumanoid::updateCoordinates( const VectorType &orig_points,
             modified_normals[index] += weighted_normal;
           } else {
             modified_normals[index] = weighted_normal;
-          } 
+          }
         }
 
-        
+
         if( index < max_size ) {
           point_written[index] = true;
         }
-
-        
-      }
-    }
-  }
-
-  // do displacer movements
-  for( unsigned int i = 0; i < jts.size(); ++i ) {
-    HAnimJoint *joint = static_cast< HAnimJoint* >( jts[i]);
-    if( joint ) {
-      const NodeVector &disp = joint->displacers->getValue();  
-      if( disp.size() > 0 ) {
-        for( unsigned int i = 0; i < disp.size(); ++i ) {
-          HAnimDisplacer *displacer = 
-            static_cast< HAnimDisplacer* >( disp[i]);
-          if( displacer ) {
-            displacer->displaceCoordinates(modified_points, 
-                                           joint->accumulatedJointMatrix->getValue() );
-          }
-        }
       }
     }
   }
@@ -242,15 +404,15 @@ void HAnimHumanoid::initialize() {
   root_transform->bound->route( bound );
 }
 
-void HAnimHumanoid::render()     { 
+void HAnimHumanoid::render()     {
   X3DChildNode::render();
 
   const string &render_type = renderMode->getValue();
 
-  
+
 
   if( render_type == "JOINTS" || render_type == "SKELETON" ) {
-    HAnimJoint::RenderType type = 
+    HAnimJoint::RenderType type =
       render_type == "JOINTS" ? HAnimJoint::JOINTS : HAnimJoint::SKELETON;
     glMatrixMode( GL_MODELVIEW );
     glPushMatrix();
@@ -260,6 +422,7 @@ void HAnimHumanoid::render()     {
       HAnimJoint *joint = dynamic_cast< HAnimJoint* >( skel[i]);
       if( joint ) joint->renderSkeleton( type );
     }
+    glMatrixMode(GL_MODELVIEW);
     glPopMatrix();
   } else {
     // render skinned version
@@ -271,7 +434,7 @@ void HAnimHumanoid::render()     {
     }
   }
 
-};
+}
 
 void HAnimHumanoid::traverseSG( TraverseInfo &ti ) {
 
@@ -289,7 +452,7 @@ void HAnimHumanoid::traverseSG( TraverseInfo &ti ) {
         points_single.clear();
         points_double = c->point->getValue();
       } else {
-        Console(4) << "Unsupported X3DCoordinateNode: \"" 
+        Console(LogLevel::Error) << "Unsupported X3DCoordinateNode: \""
                    << coord->getTypeName() << "\" in HAnimHumanoid." << endl;
         points_single.clear();
         points_double.clear();
@@ -305,31 +468,30 @@ void HAnimHumanoid::traverseSG( TraverseInfo &ti ) {
     if( normal ) {
       normals_single = normal->vector->getValue();
     } else if( base_normal ) {
-      Console(4) << "Unsupported X3DNormalNode: \"" 
+      Console(LogLevel::Error) << "Unsupported X3DNormalNode: \""
                    << base_normal->getTypeName() << "\" in HAnimHumanoid." << endl;
       normals_single.clear();
     }
   }
-  
+
   X3DChildNode::traverseSG( ti );
   root_transform->traverseSG( ti );
 
   const NodeVector &skel = skeleton->getValue();
-  const NodeVector &jts = joints->getValue();
 
-  // traverse skeleton 
+  // traverse skeleton
   for( unsigned int i = 0; i < skel.size(); ++i ) {
     Node *n = skel[i];
     if( n ) n->traverseSG( ti );
   }
 
-  if( !joint_matrix_changed->isUpToDate() ) {
-    joint_matrix_changed->upToDate();
+  if( !jointMatrixChanged->isUpToDate() ) {
+    jointMatrixChanged->upToDate();
 
     if( CoordinateDouble *c = dynamic_cast< CoordinateDouble * >( coord ) ) {
       vector< Vec3d > modified_points = points_double;
       vector< Vec3f > modified_normals = normals_single;
-      updateCoordinates( points_double, normals_single, 
+      updateCoordinates( points_double, normals_single,
                          modified_points, modified_normals );
       c->point->swap( modified_points );
       if( normal ) normal->vector->swap( modified_normals );
@@ -338,7 +500,7 @@ void HAnimHumanoid::traverseSG( TraverseInfo &ti ) {
       vector< Vec3f > modified_normals = normals_single;
       updateCoordinates( points_single, normals_single,
                          modified_points, modified_normals );
-      if( Coordinate *c = dynamic_cast< Coordinate * >( coord )) { 
+      if( Coordinate *c = dynamic_cast< Coordinate * >( coord )) {
         c->point->swap( modified_points );
       }
       if( normal ) normal->vector->swap( modified_normals );
@@ -347,8 +509,8 @@ void HAnimHumanoid::traverseSG( TraverseInfo &ti ) {
 }
 
 bool HAnimHumanoid::lineIntersect(
-                  const Vec3f &from, 
-                  const Vec3f &to,    
+                  const Vec3f &from,
+                  const Vec3f &to,
                   LineIntersectResult &result ) {
   return root_transform->lineIntersect( from, to, result );
 }
@@ -359,25 +521,24 @@ void HAnimHumanoid::closestPoint( const Vec3f &p,
 }
 
 bool HAnimHumanoid::movingSphereIntersect( H3DFloat radius,
-                                           const Vec3f &from, 
+                                           const Vec3f &from,
                                            const Vec3f &to,
                                            NodeIntersectResult &result ) {
   return root_transform->movingSphereIntersect( radius,
                                                 from,
-                                                to, 
+                                                to,
                                                 result );
 }
 
 
 void HAnimHumanoid::MFSkeletonNode::onAdd( Node *n ) {
-  HAnimHumanoid *humanoid = static_cast< HAnimHumanoid * >( getOwner() );
   if( n ) {
-    if( HAnimJoint *joint = dynamic_cast< HAnimJoint *>( n ) ) {
-      
+    if( dynamic_cast< HAnimJoint *>( n ) ) {
+
     } else if( dynamic_cast< HAnimSite * >( n ) ) {
-      
+
     } else {
-      Console(4) << "Invalid Node type: \"" << n->getTypeName() 
+      Console(LogLevel::Error) << "Invalid Node type: \"" << n->getTypeName()
                  << "\" in HAnimHumanoid.skeleton field. Must be HAnimJoint or HAnimSite" << endl;
       return;
     }
@@ -390,13 +551,13 @@ void HAnimHumanoid::MFJoint::onAdd( Node *n ) {
   HAnimHumanoid *humanoid = static_cast< HAnimHumanoid * >( getOwner() );
   if( n ) {
     if( HAnimJoint *joint = dynamic_cast< HAnimJoint *>( n ) ) {
-      joint->accumulatedJointMatrix->route( humanoid->joint_matrix_changed );
-      joint->displacers->route( humanoid->joint_matrix_changed );
+      joint->accumulatedJointMatrix->route( humanoid->jointMatrixChanged );
+      joint->displacers->route( humanoid->jointMatrixChanged );
       joint->displayList->route( humanoid->displayList );
     } else if( HAnimSite *site = dynamic_cast< HAnimSite * >( n ) ) {
       site->displayList->route( humanoid->displayList );
     } else {
-      Console(4) << "Invalid Node type: \"" << n->getTypeName() 
+      Console(LogLevel::Error) << "Invalid Node type: \"" << n->getTypeName()
                  << "\" in HAnimHumanoid.skeleton field. Must be HAnimJoint or HAnimSite" << endl;
       return;
     }
@@ -409,8 +570,8 @@ void HAnimHumanoid::MFJoint::onRemove( Node *n ) {
   HAnimHumanoid *humanoid = static_cast< HAnimHumanoid * >( getOwner() );
 
   if( HAnimJoint *joint = dynamic_cast< HAnimJoint *>( n ) ) {
-    joint->accumulatedJointMatrix->unroute( humanoid->joint_matrix_changed );
-    joint->displacers->unroute( humanoid->joint_matrix_changed );
+    joint->accumulatedJointMatrix->unroute( humanoid->jointMatrixChanged );
+    joint->displacers->unroute( humanoid->jointMatrixChanged );
     joint->displayList->unroute( humanoid->displayList );
   } else if( HAnimSite *site = dynamic_cast< HAnimSite *>( n ) ) {
     site->displayList->unroute( humanoid->displayList );
