@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -43,8 +43,8 @@ H3DNodeDatabase IndexedTriangleFanSet::database(
                                     &X3DComposedGeometryNode::database );
 
 namespace IndexedTriangleFanSetInternals {
-  FIELDDB_ELEMENT( IndexedTriangleFanSet, set_index, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( IndexedTriangleFanSet, index, INPUT_OUTPUT );
+  FIELDDB_ELEMENT( IndexedTriangleFanSet, set_index, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( IndexedTriangleFanSet, index, INPUT_OUTPUT )
 }
 
 IndexedTriangleFanSet::IndexedTriangleFanSet( 
@@ -402,39 +402,39 @@ void IndexedTriangleFanSet::traverseSG( TraverseInfo &ti ) {
 void IndexedTriangleFanSet::AutoNormal::update() {
   bool normals_per_vertex = 
     static_cast< SFBool * >( routes_in[0] )->getValue();
-  X3DCoordinateNode *coord = 
+  X3DCoordinateNode *_coord = 
     static_cast< X3DCoordinateNode * >( static_cast< SFCoordinateNode * >
                                         ( routes_in[1] )->getValue() );
-  const vector<int> &index = 
+  const vector<int> &_index = 
     static_cast< MFInt32 * >( routes_in[2] )->getValue();
-  bool ccw = static_cast< SFBool * >( routes_in[3] )->getValue();
+  bool _ccw = static_cast< SFBool * >( routes_in[3] )->getValue();
 
   if( normals_per_vertex ) 
-    value = generateNormalsPerVertex( coord, index, ccw );
+    value = generateNormalsPerVertex( _coord, _index, _ccw);
   else
-    value = generateNormalsPerFace( coord, index, ccw );
+    value = generateNormalsPerFace( _coord, _index, _ccw);
 }
 
 X3DNormalNode *IndexedTriangleFanSet::AutoNormal::generateNormalsPerVertex( 
-                                   X3DCoordinateNode *coord,
-                                   const vector< int > &index,
-                                   bool ccw ) {
-  Normal *normal = new Normal;
-  if( coord ) {
-    vector< Vec3f > normals( coord->nrAvailableCoords(), 
+                                   X3DCoordinateNode *_coord,
+                                   const vector< int > &_index,
+                                   bool _ccw ) {
+  Normal *_normal = new Normal;
+  if( _coord ) {
+    vector< Vec3f > normals(_coord->nrAvailableCoords(),
                              Vec3f( 0, 0, 0 ) );
     // the start index of the current triangle fan
     unsigned int fan_root = 0;
-    for( unsigned int j = 0; j < index.size() - 2; ++j ) {
-      if( j+2 < index.size() && 
-          index[j] != -1 && 
-          index[j+1] != -1 && 
-          index[j+2] != -1 ) {
+    for( unsigned int j = 0; j < _index.size() - 2; ++j ) {
+      if( j+2 < _index.size() &&
+          _index[j] != -1 &&
+          _index[j+1] != -1 &&
+          _index[j+2] != -1 ) {
         Vec3f norm, A, B, C, AB, BC;
         // calculate a normal 
-        A = coord->getCoord( index[ fan_root ] );
-        B = coord->getCoord( index[ j+1 ] );
-        C = coord->getCoord( index[ j+2 ] );
+        A = _coord->getCoord( _index[ fan_root ] );
+        B = _coord->getCoord( _index[ j+1 ] );
+        C = _coord->getCoord( _index[ j+2 ] );
         
         AB = B - A;
         BC = C - B;
@@ -442,11 +442,11 @@ X3DNormalNode *IndexedTriangleFanSet::AutoNormal::generateNormalsPerVertex(
         norm = AB % BC;
 
         norm.normalizeSafe();
-        if( !ccw ) 
+        if( !_ccw ) 
           norm = -norm;
-        normals[index[ fan_root ]] += norm;
-        normals[index[ j+1 ]] += norm;
-        normals[index[ j+2 ]] += norm;
+        normals[_index[ fan_root ]] += norm;
+        normals[_index[ j+1 ]] += norm;
+        normals[_index[ j+2 ]] += norm;
       } else {
         fan_root = j+1;
       }
@@ -457,32 +457,32 @@ X3DNormalNode *IndexedTriangleFanSet::AutoNormal::generateNormalsPerVertex(
          ++i ) {
         (*i).normalizeSafe();
     }
-    normal->vector->setValue( normals );
+    _normal->vector->setValue( normals );
   }
-  return normal;
+  return _normal;
 }
 
 X3DNormalNode *IndexedTriangleFanSet::AutoNormal::generateNormalsPerFace( 
-                                             X3DCoordinateNode *coord,
-                                             const vector< int > &index,
-                                             bool ccw ) {
-  Normal *normal = new Normal;
-  if( coord ) {
+                                             X3DCoordinateNode *_coord,
+                                             const vector< int > &_index,
+                                             bool _ccw ) {
+  Normal *_normal = new Normal;
+  if( _coord ) {
     vector< Vec3f > normals;
     // the start index of the current triangle fan
     unsigned int fan_root = 0;
-    for( unsigned int j = 0; j < index.size() - 2; ++j ) {
+    for( unsigned int j = 0; j < _index.size() - 2; ++j ) {
       
-      if( j+2 < index.size() && 
-          index[j] != -1 && 
-          index[j+1] != -1 && 
-          index[j+2] != -1 ) {
+      if( j+2 < _index.size() &&
+          _index[j] != -1 &&
+          _index[j+1] != -1 &&
+          _index[j+2] != -1 ) {
         
         Vec3f norm, A, B, C, AB, BC;
         // calculate a normal for the triangle
-        A = coord->getCoord( index[ fan_root ] );
-        B = coord->getCoord( index[ j + 1 ] );
-        C = coord->getCoord( index[ j + 2 ] );
+        A = _coord->getCoord(_index[ fan_root ] );
+        B = _coord->getCoord(_index[ j + 1 ] );
+        C = _coord->getCoord(_index[ j + 2 ] );
         
         AB = B - A;
         BC = C - B;
@@ -495,7 +495,7 @@ X3DNormalNode *IndexedTriangleFanSet::AutoNormal::generateNormalsPerFace(
           norm = Vec3f( 1, 0, 0 );
         }
         
-        if( !ccw ) 
+        if( !_ccw )
           norm = -norm;
         
         normals.push_back( norm );
@@ -503,9 +503,9 @@ X3DNormalNode *IndexedTriangleFanSet::AutoNormal::generateNormalsPerFace(
         fan_root = j+1;
       }
     }
-    normal->vector->setValue( normals );
+    _normal->vector->setValue( normals );
   }
-  return normal;
+  return _normal;
 }
 
 
