@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -50,39 +50,39 @@ H3DNodeDatabase RazerHydraSensor::database(
                                           );
 
 namespace RazerHydraSensorInternal {
-  FIELDDB_ELEMENT( RazerHydraSensor, c0Position, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( RazerHydraSensor, c0Orientation, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( RazerHydraSensor, c0Joystick, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( RazerHydraSensor, c0Trigger, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( RazerHydraSensor, c0Buttons, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( RazerHydraSensor, c0Connected, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( RazerHydraSensor, c0Docked, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( RazerHydraSensor, c0WhichHand, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( RazerHydraSensor, c0HemiTrackingEnabled, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( RazerHydraSensor, c0Button1, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( RazerHydraSensor, c0Button2, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( RazerHydraSensor, c0Button3, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( RazerHydraSensor, c0Button4, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( RazerHydraSensor, c0ButtonStart, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( RazerHydraSensor, c0ButtonBumper, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( RazerHydraSensor, c0ButtonJoystick, OUTPUT_ONLY );
+  FIELDDB_ELEMENT( RazerHydraSensor, c0Position, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( RazerHydraSensor, c0Orientation, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( RazerHydraSensor, c0Joystick, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( RazerHydraSensor, c0Trigger, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( RazerHydraSensor, c0Buttons, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( RazerHydraSensor, c0Connected, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( RazerHydraSensor, c0Docked, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( RazerHydraSensor, c0WhichHand, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( RazerHydraSensor, c0HemiTrackingEnabled, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( RazerHydraSensor, c0Button1, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( RazerHydraSensor, c0Button2, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( RazerHydraSensor, c0Button3, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( RazerHydraSensor, c0Button4, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( RazerHydraSensor, c0ButtonStart, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( RazerHydraSensor, c0ButtonBumper, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( RazerHydraSensor, c0ButtonJoystick, OUTPUT_ONLY )
 
-  FIELDDB_ELEMENT( RazerHydraSensor, c1Position, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( RazerHydraSensor, c1Orientation, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( RazerHydraSensor, c1Joystick, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( RazerHydraSensor, c1Trigger, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( RazerHydraSensor, c1Buttons, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( RazerHydraSensor, c1Connected, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( RazerHydraSensor, c1Docked, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( RazerHydraSensor, c1WhichHand, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( RazerHydraSensor, c1HemiTrackingEnabled, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( RazerHydraSensor, c1Button1, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( RazerHydraSensor, c1Button2, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( RazerHydraSensor, c1Button3, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( RazerHydraSensor, c1Button4, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( RazerHydraSensor, c1ButtonStart, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( RazerHydraSensor, c1ButtonBumper, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( RazerHydraSensor, c1ButtonJoystick, OUTPUT_ONLY );
+  FIELDDB_ELEMENT( RazerHydraSensor, c1Position, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( RazerHydraSensor, c1Orientation, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( RazerHydraSensor, c1Joystick, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( RazerHydraSensor, c1Trigger, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( RazerHydraSensor, c1Buttons, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( RazerHydraSensor, c1Connected, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( RazerHydraSensor, c1Docked, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( RazerHydraSensor, c1WhichHand, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( RazerHydraSensor, c1HemiTrackingEnabled, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( RazerHydraSensor, c1Button1, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( RazerHydraSensor, c1Button2, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( RazerHydraSensor, c1Button3, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( RazerHydraSensor, c1Button4, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( RazerHydraSensor, c1ButtonStart, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( RazerHydraSensor, c1ButtonBumper, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( RazerHydraSensor, c1ButtonJoystick, OUTPUT_ONLY )
 
   H3D_API_EXCEPTION( CouldNotInitRazerHydra );
   H3D_API_EXCEPTION( CreateWindowError );
@@ -186,7 +186,7 @@ X3DSensorNode( _enabled, _metadata, _isActive ),
   c1HemiTrackingEnabled->setValue( false, id );
   
 #ifndef HAVE_SIXENSE
-  Console(4) << "Warning: H3D API compiled without Sixense SDK. RazerHydraSensor node "
+  Console(LogLevel::Error) << "Warning: H3D API compiled without Sixense SDK. RazerHydraSensor node "
              << "will be unusable." << endl;
 #endif
 }
@@ -211,7 +211,7 @@ void RazerHydraSensor::EnableHydraSensor::onValueChange( const bool &new_value )
     if( !thread_handle.get() ) {
       int sixense_initialized = sixenseInit();
       if( sixense_initialized == SIXENSE_FAILURE ) {
-        Console(4) << "Warning: Could not initialize Sixense SDK. RazerHydraSensor node "
+        Console(LogLevel::Error) << "Warning: Could not initialize Sixense SDK. RazerHydraSensor node "
                    << "will be unusable." << endl;
         rhs->isActive->setValue( false, rhs->id );
         return;
@@ -235,7 +235,7 @@ void RazerHydraSensor::EnableHydraSensor::onValueChange( const bool &new_value )
   }
 #ifdef H3D_WINDOWS
   } else {
-    Console(4) << "Warning: Sixense SDK not found on this system. "
+    Console(LogLevel::Error) << "Warning: Sixense SDK not found on this system. "
                << "RazerHydraSensor node will be unusable until sixense.dll "
                << "is added to PATH environment variable." << endl;
   }
@@ -292,7 +292,7 @@ void RazerHydraSensor::traverseSG( TraverseInfo &ti ) {
           c1ButtonJoystick->setValue( (current_data.buttons & SIXENSE_BUTTON_JOYSTICK) > 0, id );
         }
       } else {
-        Console(3) << "Warning: Developer forgot to take care of a new case of controller index for RazerHydraSensor " << getName() << " make sure the lazy **** fixes it." << endl;
+        Console(LogLevel::Warning) << "Warning: Developer forgot to take care of a new case of controller index for RazerHydraSensor " << getName() << " make sure the lazy **** fixes it." << endl;
       }
       tmp_hydra_data.pop_front();
     }
@@ -304,7 +304,7 @@ void RazerHydraSensor::traverseSG( TraverseInfo &ti ) {
 void RazerHydraSensor::transferValues( const vector< RazerHydraData > &new_data ) {
   // Only call this function when new_data is not empty.
   data_lock.lock();
-  for( int i = new_data.size() -1; i >= 0; --i )
+  for( size_t i = new_data.size() -1; i >= 0; --i )
     hydra_data.push_back( new_data[i] );
   data_lock.unlock();
 }
