@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -60,6 +60,13 @@
 #include <H3D/SAIFunctions.h>
 #include <H3D/Shape.h>
 #include <H3D/Inline.h>
+#include <H3D/H3DSingleTextureNode.h>
+#include <H3D/X3DProgrammableShaderObject.h>
+#include <H3DUtil/ThreadPool.h>
+
+#ifndef H3D_WINDOWS
+#include <unistd.h>
+#endif
 
 using namespace H3D;
 
@@ -80,18 +87,26 @@ H3DNodeDatabase Scene::database(
                                 typeid( Scene ) );
 
 namespace SceneInternals {
-  FIELDDB_ELEMENT( Scene, sceneRoot, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( Scene, window, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( Scene, frameRate, OUTPUT_ONLY );
+  FIELDDB_ELEMENT( Scene, sceneRoot, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( Scene, window, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( Scene, frameRate, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( Scene, maxFrameRate, INPUT_OUTPUT )
 #ifdef HAVE_PROFILER
-  FIELDDB_ELEMENT( Scene, profiledResult, INPUT_OUTPUT );
+  FIELDDB_ELEMENT( Scene, profiledResult, INPUT_OUTPUT )
 #endif
+#ifdef HAVE_NVAPI
+  FIELDDB_ELEMENT( Scene, nvidiaGraphicsOptions, OUTPUT_ONLY )
+#endif // HAVE_NVAPI
 }
 
 
 set< Scene* > Scene::scenes;
 auto_ptr< SFTime > Scene::time(new SFTime( TimeStamp() ) );
 auto_ptr< Scene::EventSink > Scene::eventSink(new EventSink);
+#ifdef HAVE_NVAPI
+bool Scene::NVOptions_applied( false );
+string Scene::nvidia_graphics_options_file_url( "NVSettings.x3d" );
+#endif // HAVE_NVAPI
 
 //SFTime *Scene::time = new SFTime( TimeStamp() );
 //Scene::EventSink *Scene::eventSink = new EventSink;
@@ -109,7 +124,10 @@ namespace SceneInternal {
     }
     catch (const Exception::QuitAPI &) {
 #ifdef FREEGLUT
-      glutLeaveMainLoop();
+      for (set< Scene * >::iterator i = Scene::scenes.begin(); i != Scene::scenes.end(); ++i) {
+        (*i)->setSceneRoot(NULL);
+      }
+        glutLeaveMainLoop();
 #else
       // normal glut does not support exiting main loop. Have to quit 
       // using exit. This will not call any destructors.
@@ -118,8 +136,11 @@ namespace SceneInternal {
     }
     
     catch (const Exception::H3DException &e) {
-      Console(4) << e << endl;
+      Console(LogLevel::Error) << e << endl;
 #ifdef FREEGLUT
+      for (set< Scene * >::iterator i = Scene::scenes.begin(); i != Scene::scenes.end(); ++i) {
+        (*i)->setSceneRoot(NULL);
+      }
       glutLeaveMainLoop();
 #else
       // normal glut does not support exiting main loop. Have to quit 
@@ -138,6 +159,89 @@ std::string Scene::generateProfileResult()
   
   std::stringstream result;
 
+  // Compute exclusive times for each group node
+  // i.e. Time for group minus total time reported by children
+  const H3DTime update_interval = 1;
+  if( time->getValue () - exclusive_times_update_time > update_interval ) {
+    exclusive_times_update_time = time->getValue();
+    if( X3DGroupingNode* g = dynamic_cast <X3DGroupingNode*> (sceneRoot->getValue()) ) {
+      ProfileNodeResultVec exclusive_times;
+      computeExclusiveTimes( *g, exclusive_times );
+
+      // Sort descending and trim
+      const int max_nodes = 10;
+      const H3DTime max_time = 1;
+
+      ProfileNodeResultVec exclusive_times_render;
+      exclusive_times_render.reserve( exclusive_times.size() );
+      for( ProfileNodeResultVec::iterator i = exclusive_times.begin(); i != exclusive_times.end(); ++i ) {
+        const ProfileNodeResult& r = *i;
+        if( TimeStamp() - r.render_time_last < max_time ) {
+          exclusive_times_render.push_back( r );
+        }
+      }
+      std::sort( exclusive_times_render.begin(), exclusive_times_render.end(), compareRenderTime );
+      if( exclusive_times_render.size() > max_nodes ) {
+        exclusive_times_render.erase( exclusive_times_render.begin() + max_nodes, exclusive_times_render.end() );
+      }
+
+      ProfileNodeResultVec exclusive_times_traverse;
+      exclusive_times_traverse.reserve( exclusive_times.size() );
+      for( ProfileNodeResultVec::iterator i = exclusive_times.begin(); i != exclusive_times.end(); ++i ) {
+        const ProfileNodeResult& r = *i;
+        if( TimeStamp() - r.traverse_time_last < max_time ) {
+          exclusive_times_traverse.push_back( r );
+        }
+      }
+      std::sort( exclusive_times_traverse.begin(), exclusive_times_traverse.end(), compareTraverseTime );
+      if( exclusive_times_traverse.size() > max_nodes ) {
+        exclusive_times_traverse.erase( exclusive_times_traverse.begin() + max_nodes, exclusive_times_traverse.end() );
+      }
+
+      // Format
+      std::stringstream ss;
+      ss << "Render:" << std::endl;
+      for( ProfileNodeResultVec::iterator i = exclusive_times_render.begin(); i != exclusive_times_render.end(); ++i ) {
+        ProfileNodeResult& r = *i;
+
+        std::string child_names;
+        if( !r.node->hasName() ) {
+          if( X3DGroupingNode* group = dynamic_cast <X3DGroupingNode*> (r.node) ) {
+            for( NodeVector::const_iterator j = group->children->begin(); j != group->children->end(); ++j ) {
+              if( *j ) {
+                child_names += (*j)->getName() + " ";
+              }
+            }
+          }
+        }
+
+        ss << std::setprecision( 2 ) << r.render_time*1000.0 << " ms " << r.node->getName() << "(" << r.node->getTypeName() << ") " << child_names << endl;
+      }
+      ss << std::endl;
+      ss << "Traverse:" << std::endl;
+      for( ProfileNodeResultVec::iterator i = exclusive_times_traverse.begin(); i != exclusive_times_traverse.end(); ++i ) {
+        ProfileNodeResult& r = *i;
+
+        std::string child_names;
+        if( !r.node->hasName() ) {
+          if( X3DGroupingNode* group = dynamic_cast <X3DGroupingNode*> (r.node) ) {
+            for( NodeVector::const_iterator j = group->children->begin(); j != group->children->end(); ++j ) {
+              if( *j ) {
+                child_names += (*j)->getName() + " ";
+              }
+            }
+          }
+        }
+
+        ss << std::setprecision( 2 ) << r.traverse_time*1000.0 << " ms " << r.node->getName() << "(" << r.node->getTypeName() << ") " << child_names << endl;
+      }
+      exclusive_times_string = ss.str();
+    }
+  }
+
+  result << "====================================Node Times===================================" << std::endl;
+  result << exclusive_times_string;
+  result << "=======================================END=======================================" << std::endl;
   
   if(!H3D_scene_result.isEmpty())
   {
@@ -158,22 +262,80 @@ std::string Scene::generateProfileResult()
     result<<H3D_sofa_result.getResult();
     result<<"=======================================END======================================="<<std::endl;
   }
+
   return result.str();
 }
+
+void Scene::computeExclusiveTimes( Node& _node, ProfileNodeResultVec& _result ) {
+
+  if( X3DGroupingNode* group = dynamic_cast <X3DGroupingNode*> ( &_node ) ) {
+
+    std::pair < H3DTime, H3DTime > times = group->getChildTimes();
+    ProfileNodeResult r = { 
+      group, group->time_in_last_render - times.first, 
+      group->time_in_last_traverseSG - times.second,
+      group->time_last_render,
+      group->time_last_traverseSG
+    };
+    _result.push_back( r );
+
+    // Recurse
+    for( NodeVector::const_iterator i = group->children->begin(); i != group->children->end(); ++i ) {
+      if( *i ) {
+        computeExclusiveTimes( **i, _result );
+      }
+    }
+
+  } else if( Inline* inl = dynamic_cast <Inline*> (&_node) ) {
+
+    // Recurse
+    for( NodeVector::const_iterator i = inl->loadedScene->begin(); i != inl->loadedScene->end(); ++i ) {
+      if( *i ) {
+        computeExclusiveTimes( **i, _result );
+      }
+    }
+
+  }
+
+  
+}
+
 #endif
 void Scene::idle() {
   // calculate and set frame rate
 #ifdef HAVE_PROFILER
   H3DUtil::H3DTimer::begin("H3D_scene");
-  H3DUtil::H3DTimer::stepBegin("H3D_scene_loop");
 
 #endif // HAVE_PROFILER
-  
+  H3DTIMER_BEGIN("H3D_scene_loop","H3D")
+  H3DFloat max_fr= maxFrameRate->getValue();
+  if ( max_fr > 0 ) {
+    H3DFloat min_frame_time= 1.0f / max_fr;
+
+    TimeStamp t0;
+    TimeStamp dt0= t0 - last_time;
+
+    H3DTime sleep_time= min_frame_time-dt0;
+    if ( sleep_time > 0 ) {
+#ifdef H3D_WINDOWS
+      Sleep ( (DWORD)(sleep_time*1000.0) );
+#else
+      usleep( sleep_time*1e6 );
+#endif
+    }
+  }
+
   TimeStamp t;
   TimeStamp dt = t - last_time;
   frameRate->setValue( 1.0f / (H3DFloat)(dt), id );
   last_time = t;
+#ifdef HAVE_PROFILER
+  H3DUtil::H3DTimer::stepBegin("Time_update");
+#endif
   time->setValue( t, id );
+#ifdef HAVE_PROFILER
+  H3DUtil::H3DTimer::stepEnd("Time_update");
+#endif
 
 #ifdef THREAD_LOCK_DEBUG
 #ifndef HAVE_PROFILER
@@ -183,27 +345,27 @@ void Scene::idle() {
   main_info.total_run_time += dt;
   main_info.period_start_time = t;
   if( t - last_update_time > THREAD_LOCK_DEBUG_UPDATE_INTERVAL ) {
-    Console(4) << "*************************************************" << endl;
+    Console(LogLevel::Error) << "*************************************************" << endl;
     for( ThreadBase::ThreadLockInfoMap::const_iterator i = ThreadBase::thread_lock_info.begin();
          i != ThreadBase::thread_lock_info.end(); ++i ) {
       ThreadBase *thread = H3DUtil::ThreadBase::getThreadById( (*i).first );
       const ThreadBase::ThreadLockInfo &info = (*i).second;
       if( thread ) {
-        Console(4) << "Thread:        " << thread->getThreadName() << endl;
+        Console(LogLevel::Error) << "Thread:        " << thread->getThreadName() << endl;
       } else {
         if( pthread_equal( (*i).first, ThreadBase::getMainThreadId() ) ) {
-          Console(4) << "Thread:        " << "Main Thread" << endl;
+          Console(LogLevel::Error) << "Thread:        " << "Main Thread" << endl;
           static double last_main_thread_lock_time = 0;
-          Console(4) << "Lock (%) inst: " << (info.total_run_time == 0 ? 0 : (info.total_lock_time - last_main_thread_lock_time )/ (t - last_update_time)) *100  << endl;
+          Console(LogLevel::Error) << "Lock (%) inst: " << (info.total_run_time == 0 ? 0 : (info.total_lock_time - last_main_thread_lock_time )/ (t - last_update_time)) *100  << endl;
           last_main_thread_lock_time = info.total_lock_time;
 
         } else {
-          Console(4) << "Thread:        " << "Non ThreadBase" << endl;
+          Console(LogLevel::Error) << "Thread:        " << "Non ThreadBase" << endl;
         }
       }
-      Console(4)   << "Tot runtime:   " << info.total_run_time << endl;
-      Console(4)   << "Lock (s):      " << info.total_lock_time << endl;
-      Console(4)   << "Lock (%):      " << (info.total_run_time == 0 ? 0 : (info.total_lock_time / info.total_run_time) *100)  << endl << endl;
+      Console(LogLevel::Error)   << "Tot runtime:   " << info.total_run_time << endl;
+      Console(LogLevel::Error)   << "Lock (s):      " << info.total_lock_time << endl;
+      Console(LogLevel::Error)   << "Lock (%):      " << (info.total_run_time == 0 ? 0 : (info.total_lock_time / info.total_run_time) *100)  << endl << endl;
     }
     last_update_time = t;
   }
@@ -325,21 +487,16 @@ void Scene::idle() {
       
     }
     // traverse the scene graph to collect the HapticObject instances to render.
-    TraverseInfo *ti = new TraverseInfo( hds );
+    auto_ptr<TraverseInfo> ti(new TraverseInfo(hds));
 
     ti->setUserData( "ShadowCaster", shadow_caster.get() );
 
-#ifdef HAVE_PROFILER
-    H3DUtil::H3DTimer::stepBegin("Scene_traverse");
-#endif
+    H3DTIMER_BEGIN( "Scene_traverse", "H3D" );
     if( scene_root ) {
       scene_root->traverseSG( *ti );
     }
-#ifdef HAVE_PROFILER
-    H3DUtil::H3DTimer::stepEnd("Scene_traverse");
-#endif
-    /// traverse the stylus of all haptics devices
-    DeviceInfo *di = DeviceInfo::getActive();
+    H3DTIMER_END( "Scene_traverse" );
+    // traverse the stylus of all haptics devices
     if( di ) {
       for( DeviceInfo::MFDevice::const_iterator i = di->device->begin();
            i != di->device->end(); ++i ) {
@@ -365,25 +522,27 @@ void Scene::idle() {
     unsigned int nr_devices = (unsigned int) ti->getHapticsDevices().size();
     for( unsigned int i = 0; i < nr_devices; ++i ) {
       H3DHapticsDevice *hd = ti->getHapticsDevice( i );
-      // nr_of_layers is the biggest of the nr of layers in this traverseinfo
-      // and last_traverseinfo.
-      unsigned int nr_of_layers = ti->nrLayers();
-      if( last_traverseinfo && last_traverseinfo->nrLayers() > nr_of_layers )
-        nr_of_layers = last_traverseinfo->nrLayers();
-      if( hd->initialized->getValue() ) {
-        for( unsigned int l = 0; l < nr_of_layers; ++l ) {
-          if( l < ti->nrLayers() ) {
-            ti->setCurrentLayer( l );
-            hd->renderShapes( ti->getHapticShapes( i ), l );
-          } else {
-            // This layer no longer exists, set the shapes to an empty vector
-            // to remove from haptics loop.
-            HapticShapeVector tmp_vector;
-            hd->renderShapes( tmp_vector, l );
+      if( hd->getHAPIDevice() && (hd->getHAPIDevice()->getDeviceState() == HAPI::HAPIHapticsDevice::ENABLED ) ) {
+        // nr_of_layers is the biggest of the nr of layers in this traverseinfo
+        // and last_traverseinfo.
+        unsigned int nr_of_layers = ti->nrLayers();
+        if( last_traverseinfo && last_traverseinfo->nrLayers() > nr_of_layers )
+          nr_of_layers = last_traverseinfo->nrLayers();
+        if( hd->initialized->getValue() ) {
+          for( unsigned int l = 0; l < nr_of_layers; ++l ) {
+            if( l < ti->nrLayers() ) {
+              ti->setCurrentLayer( l );
+              hd->renderShapes( ti->getHapticShapes( i ), l );
+            } else {
+              // This layer no longer exists, set the shapes to an empty vector
+              // to remove from haptics loop.
+              HapticShapeVector tmp_vector;
+              hd->renderShapes( tmp_vector, l );
+            }
           }
+          hd->renderEffects( ti->getForceEffects( i ) );
+          hd->postRender();
         }
-        hd->renderEffects( ti->getForceEffects( i ) );
-        hd->postRender();
       }
     }
 
@@ -393,31 +552,27 @@ void Scene::idle() {
     // one scenegraph loop.
     if( last_traverseinfo )
       delete last_traverseinfo;
-    last_traverseinfo = ti;
+    last_traverseinfo = ti.release();
 
 
   } else {
     // no HapticDevices exist, but we still have to traverse the scene-graph.
     // Haptics is disabled though to avoid unnecessary calculations.
-    TraverseInfo *ti = new TraverseInfo( vector< H3DHapticsDevice * >() );
+    auto_ptr<TraverseInfo> ti(new TraverseInfo( vector< H3DHapticsDevice * >()));
     ti->setUserData( "ShadowCaster", shadow_caster.get() );
     ti->disableHaptics();
-#ifdef HAVE_PROFILER
-    H3DUtil::H3DTimer::stepBegin("Scene_traverse");
-#endif
+    H3DTIMER_BEGIN( "Scene_traverse","H3D" );
     if( scene_root ) {
       scene_root->traverseSG( *ti );
     }
-#ifdef HAVE_PROFILER
-    H3DUtil::H3DTimer::stepEnd("Scene_traverse");
-#endif
+    H3DTIMER_END( "Scene_traverse" );
     // remove the TraverseInfo instance from the last loop. TraverseInfo 
     // instances must be kept alive until its HapticShapes and 
     // HAPIForceEffects are not rendered anymore, which in this case is 
     // one scenegraph loop.
     if( last_traverseinfo )
       delete last_traverseinfo;
-    last_traverseinfo = ti;
+    last_traverseinfo = ti.release();
   }
   
   // call the callback functions added during callback.
@@ -441,52 +596,66 @@ void Scene::idle() {
           vp->accForwardMatrix->getValue().getRotationPart() * 
           (vp->totalOrientation->getValue() * Vec3f( 0, 0, -1 ));
       }
-      DirectionalLight *light = new DirectionalLight();
+      auto_ptr<DirectionalLight> light(new DirectionalLight());
       light->direction->setValue( direction );
-      shadow_caster->light->push_back( light );
+      shadow_caster->light->push_back( light.release() );
     }
+    // default shadow caster doesn't call traverse so call the update function directly
+    shadow_caster->updateShadowInformation( *last_traverseinfo, shadow_caster.get() );
   }
-#ifdef HAVE_PROFILER
-  H3DUtil::H3DTimer::stepBegin("Graphic_rendering");
-#endif
+  H3DTIMER_BEGIN( "Graphic_rendering", "H3D" );
   // call window's render function
   for( MFWindow::const_iterator w = window->begin(); 
        w != window->end(); ++w ) {
-    H3DWindowNode *window = static_cast< H3DWindowNode * >(*w);
-    bool used_mpt = window->getMultiPassTransparency();
-    window->setMultiPassTransparency( 
+    H3DWindowNode *_window = static_cast< H3DWindowNode * >(*w);
+    bool used_mpt = _window->getMultiPassTransparency();
+    _window->setMultiPassTransparency(
                        last_traverseinfo->getMultiPassTransparency() );
-    if( used_mpt != window->getMultiPassTransparency() ) {
+    if( used_mpt != _window->getMultiPassTransparency() ) {
       H3DDisplayListObject::DisplayList::rebuildAllDisplayLists();
     }
-    window->render( scene_root );
+    _window->render( scene_root );
   }
-#ifdef HAVE_PROFILER
-  H3DUtil::H3DTimer::stepEnd("Graphic_rendering");
-#endif
+  H3DTIMER_END( "Graphic_rendering" );
   // update the eventSink
-#ifdef HAVE_PROFILER
-  H3DUtil::H3DTimer::stepBegin("Event_sink_update");
-#endif
+
+  H3DTIMER_BEGIN( "Event_sink_update","H3D" );
   eventSink->upToDate();
-#ifdef HAVE_PROFILER
-  H3DUtil::H3DTimer::stepEnd("Event_sink_update");
-#endif
+  H3DTIMER_END( "Event_sink_update" );
+
+  H3DTIMER_BEGIN( "Scene_callbacks", "H3D" );
   callback_lock.lock();
-  // execute callbacks
-  for( CallbackList::iterator i = callbacks.begin();
-       i != callbacks.end(); ) {
-    CallbackCode c = (*i).first( (*i).second );
-    if( c == CALLBACK_DONE ) {
-      i = callbacks.erase( i );
-    } else {
-      ++i;
+  try {
+    // execute callbacks
+    for( CallbackList::iterator i = callbacks.begin();
+         i != callbacks.end(); ) {
+      CallbackCode c = (*i).first( (*i).second );
+      if( c == CALLBACK_DONE ) {
+        i = callbacks.erase( i );
+      } else {
+        ++i;
+      }
     }
+  } catch (const Exception::QuitAPI &) {
+    callback_lock.unlock();
+    throw Exception::QuitAPI();
+  } catch( const Exception::H3DException &exp ) {
+    callback_lock.unlock();
+    H3DUtil::Console( LogLevel::Error ) << exp.message << std::endl;
+  } catch( const std::exception &e ) {
+    callback_lock.unlock();
+    H3DUtil::Console( LogLevel::Error ) << e.what() << std::endl;
+  } catch( ... ) {
+    callback_lock.unlock();
+    H3DUtil::Console( LogLevel::Error ) << "Unknown exception" << std::endl;
+    throw Exception::QuitAPI();
   }
-  callback_lock.unlock();
   
+  callback_lock.unlock();
+  H3DTIMER_END( "Scene_callbacks" );
+  H3DTIMER_END( "H3D_scene_loop" );
   #ifdef HAVE_PROFILER
-  H3DUtil::H3DTimer::stepEnd("H3D_scene_loop");
+
   std::stringstream profiled_result_scene_temp;
   H3DUtil::H3DTimer::end("H3D_scene",profiled_result_scene_temp);
   if(!profiled_result_scene_temp.str().empty())
@@ -501,6 +670,24 @@ void Scene::idle() {
 
   Anchor::replaceSceneRoot( this );
 
+  if ( X3DProgrammableShaderObject::use_bindless_textures ) {
+    // Periodically check if there are any resident textures which
+    // have not been used recently, and make them non-resident
+    H3DTime unused_texture_time ( 5 );
+    if( default_settings ) {
+      GraphicsOptions* gfx_options;
+      default_settings->getOptionNode( gfx_options );
+      if ( gfx_options ) {
+        unused_texture_time= gfx_options->bindlessTexturesUnusedTime->getValue();
+      }
+    }
+
+    H3DTime cur_time= time->getValue();
+    if ( cur_time - last_unused_texture_check > unused_texture_time ) {
+      H3DSingleTextureNode::makeUnusedTexturesNonResident ( unused_texture_time );
+      last_unused_texture_check= cur_time;
+    }
+  }
 }
 
 
@@ -517,24 +704,41 @@ Scene::Scene( Inst< SFChildNode >  _sceneRoot,
               ,
               Inst< MFString    >  _profiledResult 
 #endif
+              ,Inst< SFFloat     >  _maxFrameRate
+#ifdef HAVE_NVAPI
+              ,
+              Inst< SFNvidiaGraphicsOptions > _nvidiaGraphicsOptions
+#endif // HAVE_NVAPI
               ) :
   sceneRoot( _sceneRoot ),
   window   ( _window    ),
   frameRate( _frameRate ),
+  maxFrameRate ( _maxFrameRate ),
 #ifdef HAVE_PROFILER
   profiledResult( _profiledResult ),
+  exclusive_times_update_time ( 0 ),
 #endif
+#ifdef HAVE_NVAPI
+  nvidiaGraphicsOptions( _nvidiaGraphicsOptions ),
+#endif // HAVE_NVAPI
+
   active( true ),
   last_traverseinfo( NULL ),
-  SAI_browser( this ),
-  shadow_caster( new ShadowCaster ) {
+  SAI_browser( NULL ),
+  shadow_caster( new ShadowCaster ),
+  last_unused_texture_check ( 0 ) {
 
+  SAI_browser = this;
 #ifdef HAVE_PROFILER
   H3DUtil::H3DTimer::setEnabled("H3D_scene",true);
   H3DUtil::H3DTimer::setInterval("H3D_scene",10);
 #endif //HAVE_PROFILER
+  if( scenes.empty() ) {
+    H3DUtil::ThreadPool::global_pool.resize( 8 ); 
+  }
   scenes.insert( this );
   
+  
   shadow_caster->algorithm->setValue( "ZFAIL" );
 
   type_name = "Scene";
@@ -543,13 +747,39 @@ Scene::Scene( Inst< SFChildNode >  _sceneRoot,
   Scene::time->setName( "Scene::time" );
   time->setAccessType( Field::OUTPUT_ONLY );
   frameRate->setValue( 0, id );
+  maxFrameRate->setValue ( -1 );
   ThreadBase::setThreadName( ThreadBase::getMainThreadId(), 
                              "H3D API Main Thread" );
   Scene::time->route( eventSink );
+#ifdef HAVE_NVAPI
+  if( !Scene::NVOptions_applied ) {
+    // apply nv setting is not done yet
+    AutoRef<Node> n;
+    string result;
+    try {
+      n = X3D::createX3DNodeFromURL( nvidia_graphics_options_file_url );
+    } catch( const Exception::H3DException &e ) {
+      Console(LogLevel::Warning) << nvidia_graphics_options_file_url << " file parsing failed, please check if the file exist and if the content is valid! The full error was: " << endl << e << std::endl;
+    }
+    if( n.get() ) {
+      if( dynamic_cast<NvidiaGraphicsOptions*>(n.get()) ) {
+        // NvidiaGraphicsOptions node is accessed correctly
+        // Settings are applied when the node is initialized
+        nvidiaGraphicsOptions->setValue( n, id );
+      } else {
+        Console(LogLevel::Warning) << nvidia_graphics_options_file_url << " file must have NvidiaGraphicsOptions node as the topmost node!" << std::endl;
+      }
+    }
+    NVOptions_applied = true;
+  }
+#endif // HAVE_NVAPI
 }
 
 Scene::~Scene() {
   scenes.erase( this );
+  if( scenes.empty() ) {
+    ThreadPool::global_pool.resize(0);
+  }
   setSceneRoot( NULL );
 
   if( last_traverseinfo )
@@ -581,7 +811,6 @@ void Scene::EventSink::update() {
 void Scene::loadSceneRoot( const string &url ) {
   SAI::SAIScene *scene = new SAI::SAIScene;
   try {
-  // TODO: fill out scene with all values
   scene->root_node.reset( X3D::createX3DFromURL( url,
                                                  &scene->named_nodes,
                                                  &scene->exported_nodes,
@@ -680,9 +909,9 @@ bool Scene::removeProgramSetting( Field *field ) {
       field->markProgramSetting( false );
       removed = true;
       // call callbacks
-      for( ProgramSettingsCallbackList::iterator i = program_settings_callbacks.begin();
-           i != program_settings_callbacks.end(); ++i ) {
-        (*i).first( REMOVE_SETTING, setting, (*i).second );
+      for( ProgramSettingsCallbackList::iterator j = program_settings_callbacks.begin();
+           j != program_settings_callbacks.end(); ++j ) {
+        (*j).first( REMOVE_SETTING, setting, (*j).second );
       }
       
       // remove any more existants of field
@@ -693,7 +922,7 @@ bool Scene::removeProgramSetting( Field *field ) {
   return removed;
 }
 
-H3D::Node* Scene::findChildNode(H3D::Group *group, const std::string &nodeType, const std::string &nodeName){
+H3D::Node* Scene::findChildNode(H3D::X3DGroupingNode *group, const std::string &nodeType, const std::string &nodeName){
   if(group)
   {
     const NodeVector &c = group->children->getValue();
@@ -711,7 +940,7 @@ H3D::Node* Scene::findChildNode(H3D::Group *group, const std::string &nodeType,
     }
     for( unsigned int i = 0; i < c.size(); ++i ) {
       if( c[i]){
-        H3D::Group *childgroup = dynamic_cast<H3D::Group*>(c[i]);
+        H3D::X3DGroupingNode *childgroup = dynamic_cast<H3D::X3DGroupingNode*>(c[i]);
         if (childgroup){
           H3D::Node *node = findChildNode(childgroup, nodeType, nodeName);
           if (node)
@@ -745,10 +974,10 @@ H3D::Node* Scene::findNodeType(H3D::Node *node, const std::string &nodeType, con
         }
       }
       for( unsigned int i = 0; i < c.size(); ++i ) {
-        if( c[i]){
-          H3D::Node *node = findNodeType(c[i], nodeType, nodeName);
-          if (node)
-            return node;
+        if( c[i] ){
+          H3D::Node *_node = findNodeType(c[i], nodeType, nodeName);
+          if( _node )
+            return _node;
         }
       }
       return NULL;
@@ -777,9 +1006,9 @@ H3D::Node* Scene::findNodeType(H3D::Node *node, const std::string &nodeType, con
         }
         else
         {
-          H3D::Node *node = findNodeType(appearance, nodeType, nodeName);
-          if (node)
-            return node;
+          H3D::Node *_node = findNodeType(appearance, nodeType, nodeName);
+          if( _node )
+            return _node;
         }
       }
       return NULL;
@@ -820,9 +1049,9 @@ H3D::Node* Scene::findNodeType(H3D::Node *node, const std::string &nodeType, con
         for( unsigned int i = 0; i < inlinenode->loadedScene->size(); ++i ) {
           Group *g = inlinenode->loadedScene->getValueByIndex( i );
           if( g){
-            H3D::Node *node = findNodeType(g, nodeType, nodeName);
-            if (node)
-              return node;
+            H3D::Node *_node = findNodeType(g, nodeType, nodeName);
+            if( _node )
+              return _node;
           }
         }
       }
