@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -144,7 +144,6 @@ float  SimplexNoise1234::grad( int hash, float x, float y, float z, float t ) {
 
 // A lookup table to traverse the simplex around a given point in 4D.
 // Details can be found where this table is used, in the 4D noise method.
-/* TODO: This should not be required, backport it from Bill's GLSL code! */
 unsigned char SimplexNoise1234::simplex[][4] = {
   {0,1,2,3},{0,1,3,2},{0,0,0,0},{0,2,3,1},{0,0,0,0},{0,0,0,0},{0,0,0,0},{1,2,3,0},
   {0,2,1,3},{0,0,0,0},{0,3,1,2},{0,3,2,1},{0,0,0,0},{0,0,0,0},{0,0,0,0},{1,3,2,0},
@@ -246,7 +245,7 @@ float SimplexNoise1234::noise(float x, float y) {
 
   // Add contributions from each corner to get the final noise value.
   // The result is scaled to return values in the interval [-1,1].
-  return 40.0f * (n0 + n1 + n2); // TODO: The scale factor is preliminary!
+  return 40.0f * (n0 + n1 + n2);
 }
 
 // 3D simplex noise
@@ -344,7 +343,7 @@ float SimplexNoise1234::noise(float x, float y, float z) {
 
   // Add contributions from each corner to get the final noise value.
   // The result is scaled to stay just inside [-1,1]
-  return 32.0f * (n0 + n1 + n2 + n3); // TODO: The scale factor is preliminary!
+  return 32.0f * (n0 + n1 + n2 + n3);
 }
 
 
@@ -383,7 +382,7 @@ float SimplexNoise1234::noise(float x, float y, float z, float w) {
   // To find out which of the 24 possible simplices we're in, we need to
   // determine the magnitude ordering of x0, y0, z0 and w0.
   // The method below is a good way of finding the ordering of x,y,z,w and
-  // then find the correct traversal order for the simplex were in.
+  // then find the correct traversal order for the simplex we're in.
   // First, six pair-wise comparisons are performed between each possible pair
   // of the four coordinates, and the results are used to add up binary bits
   // for an integer index.
@@ -480,5 +479,5 @@ float SimplexNoise1234::noise(float x, float y, float z, float w) {
   }
 
   // Sum up and scale the result to cover the range [-1,1]
-  return 27.0f * (n0 + n1 + n2 + n3 + n4); // TODO: The scale factor is preliminary!
+  return 27.0f * (n0 + n1 + n2 + n3 + n4);
 }
