@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -31,7 +31,7 @@
 
 #include <H3D/SpaceWareSensor.h>
 
-#ifdef WIN32
+#ifdef H3D_WINDOWS
 
 // si.h requires us to specify an OS type when including it.
 #define OS_WIN32
@@ -45,7 +45,7 @@ extern "C" {
 
 #include <process.h>
 
-#endif // WIN32
+#endif // H3D_WINDOWS
 
 #if defined( __linux )
 #include <X11/Xlib.h>
@@ -74,36 +74,36 @@ H3DNodeDatabase SpaceWareSensor::database(
                                           );
 
 namespace SpaceWareSensorInternal {
-  FIELDDB_ELEMENT( SpaceWareSensor, rawYaw, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( SpaceWareSensor, rawPitch, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( SpaceWareSensor, rawRoll, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( SpaceWareSensor, rawRotation, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( SpaceWareSensor, rawTranslation, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( SpaceWareSensor, instantTranslation, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( SpaceWareSensor, instantYaw, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( SpaceWareSensor, instantPitch, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( SpaceWareSensor, instantRoll, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( SpaceWareSensor, instantRotation, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( SpaceWareSensor, accumulatedTranslation, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( SpaceWareSensor, accumulatedYaw, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( SpaceWareSensor, accumulatedPitch, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( SpaceWareSensor, accumulatedRoll, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( SpaceWareSensor, accumulatedRotation, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( SpaceWareSensor, translationScale, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( SpaceWareSensor, rotationScale, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( SpaceWareSensor, buttons, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( SpaceWareSensor, latestButtonPress, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( SpaceWareSensor, latestButtonRelease, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( SpaceWareSensor, accumulateTimeDependent, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( SpaceWareSensor, resetAccumulatedTranslation, INPUT_ONLY );
-  FIELDDB_ELEMENT( SpaceWareSensor, resetAccumulatedRotation, INPUT_ONLY );
+  FIELDDB_ELEMENT( SpaceWareSensor, rawYaw, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( SpaceWareSensor, rawPitch, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( SpaceWareSensor, rawRoll, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( SpaceWareSensor, rawRotation, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( SpaceWareSensor, rawTranslation, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( SpaceWareSensor, instantTranslation, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( SpaceWareSensor, instantYaw, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( SpaceWareSensor, instantPitch, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( SpaceWareSensor, instantRoll, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( SpaceWareSensor, instantRotation, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( SpaceWareSensor, accumulatedTranslation, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( SpaceWareSensor, accumulatedYaw, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( SpaceWareSensor, accumulatedPitch, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( SpaceWareSensor, accumulatedRoll, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( SpaceWareSensor, accumulatedRotation, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( SpaceWareSensor, translationScale, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( SpaceWareSensor, rotationScale, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( SpaceWareSensor, buttons, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( SpaceWareSensor, latestButtonPress, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( SpaceWareSensor, latestButtonRelease, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( SpaceWareSensor, accumulateTimeDependent, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( SpaceWareSensor, resetAccumulatedTranslation, INPUT_ONLY )
+  FIELDDB_ELEMENT( SpaceWareSensor, resetAccumulatedRotation, INPUT_ONLY )
 
   H3D_API_EXCEPTION( CouldNotInitSpaceWare );
   H3D_API_EXCEPTION( CreateWindowError );
   H3D_API_EXCEPTION( WindowGetMessageError );
 
 #ifdef HAVE_3DXWARE
-#ifdef WIN32
+#ifdef H3D_WINDOWS
   // Help function for getting a string of the current
   // error returned from GetLastError.
   string getWindowErrorMessage() {
@@ -236,18 +236,18 @@ namespace SpaceWareSensorInternal {
           // the event is a 3DxWare event
           switch (si_event.type) {
           case SI_MOTION_EVENT: {
-            const SiSpwData & data = si_event.u.spwData;
+            const SiSpwData & _data = si_event.u.spwData;
             // we have to negate the z component since H3D API
             // and the SpaceWare have different coordinate systems.
             space_ware_sensor->thread_translation =
-              Vec3f( (H3DFloat)data.mData[ SI_TX ],
-                     (H3DFloat)data.mData[ SI_TY ],
-                     (H3DFloat)-data.mData[ SI_TZ ] );
+              Vec3f( (H3DFloat)_data.mData[ SI_TX ],
+                     (H3DFloat)_data.mData[ SI_TY ],
+                     (H3DFloat)-_data.mData[ SI_TZ ] );
             // convert to radians
             space_ware_sensor->thread_rotation = 
-              Vec3f( (H3DFloat)data.mData[SI_RX],
-                     (H3DFloat)data.mData[SI_RY],
-                     (H3DFloat)-data.mData[SI_RZ] ) * ( Constants::pi / 180 );
+              Vec3f( (H3DFloat)_data.mData[SI_RX],
+                     (H3DFloat)_data.mData[SI_RY],
+                     (H3DFloat)-_data.mData[SI_RZ] ) * ( Constants::pi / 180 );
             space_ware_sensor->thread_motion_event = true;
             break;
           }
@@ -255,10 +255,10 @@ namespace SpaceWareSensorInternal {
             break;
     
           case SI_BUTTON_EVENT: {
-            const SiSpwData & data = si_event.u.spwData;
-            SpaceWareSensor::ButtonData bd( data.bData.pressed,
-                                            data.bData.released,
-                                            data.bData.current );
+            const SiSpwData & _data = si_event.u.spwData;
+            SpaceWareSensor::ButtonData bd( _data.bData.pressed,
+                                            _data.bData.released,
+                                            _data.bData.current );
             space_ware_sensor->thread_buttons.push_back( bd );
             break;
           }
@@ -278,12 +278,12 @@ namespace SpaceWareSensorInternal {
       SpaceWareSensor *space_ware_sensor = 
         static_cast< SpaceWareSensor * >( data );
       space_ware_sensor->thread_is_active = false;
-      Console(4) << "Warning: SpaceWareSensor error. Node will be unusable."<<endl;
-      Console(4) << e << endl;
+      Console(LogLevel::Error) << "Warning: SpaceWareSensor error. Node will be unusable."<<endl;
+      Console(LogLevel::Error) << e << endl;
     }
     return NULL;
   }
-#endif // WIN32
+#endif // H3D_WINDOWS
 
 #if defined( __linux )
   void * spaceWareThread( void* data ) {
@@ -419,8 +419,8 @@ namespace SpaceWareSensorInternal {
       XCloseDisplay( display );
     }
     catch (const Exception::H3DException &e) {
-      Console(4) << "Warning: SpaceWareSensor error. Node will be unusable."<<endl;
-      Console(4) << e << endl;
+      Console(LogLevel::Error) << "Warning: SpaceWareSensor error. Node will be unusable."<<endl;
+      Console(LogLevel::Error) << e << endl;
     }
     return NULL;
   }
@@ -538,13 +538,13 @@ SpaceWareSensor::SpaceWareSensor(
 #endif 
 
 #ifndef HAVE_3DXWARE
-#ifdef WIN32
-  Console(4) << "Warning: H3D API compiled without 3Dxware. SpaceWareSensor node "
+#ifdef H3D_WINDOWS
+  Console(LogLevel::Error) << "Warning: H3D API compiled without 3Dxware. SpaceWareSensor node "
              << "will be unusable." << endl;
 #else
-  Console(4) << "Warning: SpaceWareSensor only supported on Windows platform." 
+  Console(LogLevel::Error) << "Warning: SpaceWareSensor only supported on Windows platform." 
              << endl;
-#endif
+#endif //H3D_WINDOWS
 #endif
   if( !sws_instance )
     sws_instance = this;
@@ -552,7 +552,6 @@ SpaceWareSensor::SpaceWareSensor(
 
 void SpaceWareSensor::updateValues(){
   using namespace SpaceWareSensorInternal;
-  // TODO: lock
 
 
   if( isActive->getValue() != thread_is_active )
