@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -17,7 +17,7 @@
 //    along with H3D API; if not, write to the Free Software
 //    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
-//    A commercial license is also available. Please contact us at 
+//    A commercial license is also available. Please contact us at
 //    www.sensegraphics.com for more information.
 //
 //
@@ -34,23 +34,23 @@
 using namespace H3D;
 
 // Add this node to the H3DNodeDatabase system.
-H3DNodeDatabase Text::database( 
-                               "Text", 
-                               &(newInstance<Text>), 
+H3DNodeDatabase Text::database(
+                               "Text",
+                               &(newInstance<Text>),
                                typeid( Text ),
                                &X3DGeometryNode::database );
 
 AutoRef<FontStyle > Text::default_font_style( NULL );
 
 namespace TextInternals {
-  FIELDDB_ELEMENT( Text, fontStyle, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( Text, length, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( Text, maxExtent, INPUT_OUTPUT );
+  FIELDDB_ELEMENT( Text, fontStyle, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( Text, length, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( Text, maxExtent, INPUT_OUTPUT )
   FieldDBInsert string( INPUT_OUTPUT( &Text::database, "string", &Text::stringF ) );
-  FIELDDB_ELEMENT( Text, lineBounds, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( Text, origin, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( Text, textBounds, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( Text, solid, INPUT_OUTPUT );
+  FIELDDB_ELEMENT( Text, lineBounds, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( Text, origin, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( Text, textBounds, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( Text, solid, INPUT_OUTPUT )
 }
 
 Text::Text( Inst< SFNode          > _metadata,
@@ -77,12 +77,12 @@ Text::Text( Inst< SFNode          > _metadata,
   outputUpdater( _outputUpdater ) {
 
   type_name = "Text";
-  
+
   database.initFields( this );
-  
+
   maxExtent->setValue( 0.0 );
   solid->setValue( false );
-  
+
   fontStyle->route( displayList );
   length->route( displayList );
   maxExtent->route( displayList );
@@ -108,7 +108,7 @@ Text::Text( Inst< SFNode          > _metadata,
 void Text::justifyMinor( const vector< string > &lines,
                          X3DFontStyleNode *font ) {
   if( lines.size() == 0 ) return;
-  
+
   X3DFontStyleNode::Alignment alignment = font->getAlignment();
   bool left_to_right = font->isLeftToRight();
   bool top_to_bottom = font->isTopToBottom();
@@ -116,7 +116,7 @@ void Text::justifyMinor( const vector< string > &lines,
 
   // dimension of the first line of text
   Vec3f line1_size = font->stringDimensions( lines[0], alignment );
-  // dimension of all lines combined  
+  // dimension of all lines combined
   Vec3f total_size = font->stringDimensions( lines, alignment );
 
   // horizontal text
@@ -125,8 +125,8 @@ void Text::justifyMinor( const vector< string > &lines,
       if( minor == X3DFontStyleNode::BEGIN ) {
         glTranslatef( 0, -line1_size.y, 0 );
       } else if( minor == X3DFontStyleNode::MIDDLE ) {
-        glTranslatef( 0, 
-                      total_size.y/2 - line1_size.y - font->descender(), 
+        glTranslatef( 0,
+                      total_size.y/2 - line1_size.y - font->descender(),
                       0 );
       } else if( minor == X3DFontStyleNode::END ) {
         glTranslatef( 0, total_size.y - line1_size.y , 0 );
@@ -169,9 +169,9 @@ void Text::justifyLine( const string& text,
       H3DFloat size = font->charDimensions( text[0] ).x;
       if( major == X3DFontStyleNode::MIDDLE ) {
         Vec3f dim = font->stringDimensions( text, alignment );
-        if( left_to_right ) 
+        if( left_to_right )
           glTranslatef( -dim.x/2, 0, 0 );
-        else 
+        else
           glTranslatef( dim.x/2 - size, 0, 0 );
       } else if( major == X3DFontStyleNode::END ) {
         Vec3f dim = font->stringDimensions( text, alignment );
@@ -180,15 +180,15 @@ void Text::justifyLine( const string& text,
       } else if( major == X3DFontStyleNode::BEGIN ||
                  major == X3DFontStyleNode::FIRST )
         if( !left_to_right ) glTranslatef( -size, 0, 0 );
-      
+
       // vertical text
     } else if( alignment == X3DFontStyleNode::VERTICAL ) {
       H3DFloat size = font->charDimensions( text[0] ).y;
       if( major == X3DFontStyleNode::MIDDLE ) {
         Vec3f dim = font->stringDimensions( text, alignment );
-        if( top_to_bottom ) 
+        if( top_to_bottom )
           glTranslatef( 0, dim.y/2 - size - font->descender(), 0 );
-        else 
+        else
           glTranslatef( 0, -dim.y/2 - font->descender(), 0 );
       } else if( major == X3DFontStyleNode::END ) {
         Vec3f dim = font->stringDimensions( text, alignment );
@@ -198,7 +198,7 @@ void Text::justifyLine( const string& text,
                  major == X3DFontStyleNode::FIRST )
         if( top_to_bottom ) glTranslatef( 0, -size, 0 );
     }
-  }  
+  }
 }
 
 void Text::moveToNewLine( const string &text, X3DFontStyleNode *font ) {
@@ -210,12 +210,12 @@ void Text::moveToNewLine( const string &text, X3DFontStyleNode *font ) {
   if( alignment == X3DFontStyleNode::HORIZONTAL ) {
     if( top_to_bottom )
       glTranslatef( 0, -dim.y * spacing, 0 );
-    else 
+    else
       glTranslatef( 0, dim.y * spacing, 0 );
   } else if( alignment == X3DFontStyleNode::VERTICAL ) {
     if( left_to_right )
       glTranslatef( dim.x * spacing, 0, 0 );
-    else 
+    else
       glTranslatef( -dim.x * spacing, 0, 0 );
   }
 }
@@ -227,12 +227,12 @@ void Text::renderTextLine( const string& text,
   X3DFontStyleNode::Alignment alignment = font->getAlignment();
   bool left_to_right = font->isLeftToRight();
   bool top_to_bottom = font->isTopToBottom();
-  
+
   switch( alignment ) {
   case X3DFontStyleNode::HORIZONTAL: {
     if( left_to_right ) {
       // Using special function for normal case of horizontal lines since this
-      // can be optimized by FTGL and render much faster than doing it 
+      // can be optimized by FTGL and render much faster than doing it
       // character by character.
       font->renderHorizontalLine( text );
     } else {
@@ -283,27 +283,27 @@ void Text::scaleToMaxExtent( const vector< string >& text,
       Vec3f total_size = font->stringDimensions( text, alignment );
       if( alignment == X3DFontStyleNode::HORIZONTAL ) {
         if( total_size.x > max_extent )
-          glScalef( max_extent / total_size.x, 1, 1 ); 
+          glScalef( max_extent / total_size.x, 1, 1 );
 
       } else if( alignment == X3DFontStyleNode::VERTICAL ) {
           if( total_size.y > max_extent )
-            glScalef( 1, max_extent / total_size.y, 1 ); 
-      }        
+            glScalef( 1, max_extent / total_size.y, 1 );
+      }
     } else {
       // we have length values so we have to inspect those to know the
       // length of a string value.
       H3DFloat max_length = 0;
       vector< H3DFloat >::const_iterator l = line_length.begin();
       vector< string >::const_iterator s = text.begin();
-      
-      // find the maximum value in the length field and skip as many 
+
+      // find the maximum value in the length field and skip as many
       // strings.
-      for( ; 
+      for( ;
            l != line_length.end() && s != text.end();
            ++l, ++s ) {
         if( max_length < *l ) max_length = *l;
       }
-      
+
       // check the remaining strings to see if any of them is longer
       // than the maximum length field and scale in order to fit within
       // max extent value.
@@ -314,8 +314,8 @@ void Text::scaleToMaxExtent( const vector< string >& text,
           ++s;
         }
         if( max_length > max_extent )
-          glScalef( max_extent / max_length, 1, 1 ); 
-        
+          glScalef( max_extent / max_length, 1, 1 );
+
       } else if( alignment == X3DFontStyleNode::VERTICAL ) {
         while( s != text.end() ) {
           Vec3f dims = font->stringDimensions( *s, alignment );
@@ -323,16 +323,16 @@ void Text::scaleToMaxExtent( const vector< string >& text,
           ++s;
         }
         if( max_length > max_extent )
-          glScalef( 1, max_extent / max_length, 1 ); 
+          glScalef( 1, max_extent / max_length, 1 );
       }
     }
   }
-} 
+}
 
 void Text::render() {
-  X3DFontStyleNode *font = 
+  X3DFontStyleNode *font =
     static_cast< X3DFontStyleNode * >( fontStyle->getValue() );
- 
+
   if( !font ) {
     font = default_font_style.get();
   }
@@ -345,7 +345,7 @@ void Text::render() {
   try {
     font->buildFonts();
   } catch( const Exception::H3DException &e ) {
-    Console(3) << "Warning: Text node " << getName() << " will not be rendered. "
+    Console(LogLevel::Warning) << "Warning: Text node " << getName() << " will not be rendered. "
                << "FontStyle::buildFonts message is: " << e.message << endl;
   }
 
@@ -353,7 +353,7 @@ void Text::render() {
     // get the field values
     X3DFontStyleNode::Alignment alignment = font->getAlignment();
     //X3DFontStyleNode::Justification major = font->getMajorJustification();
-    const vector< string >& text = stringF->getValue(); 
+    const vector< string >& text = stringF->getValue();
     //bool top_to_bottom = font->isTopToBottom();
     //bool left_to_right = font->isLeftToRight();
     //H3DFloat max_extent = maxExtent->getValue();
@@ -369,55 +369,55 @@ void Text::render() {
     for( vector< string >::const_iterator line = text.begin();
          line != text.end();
          ++line ) {
-      
-      // translate to the next line. What the next line is depends on the 
+
+      // translate to the next line. What the next line is depends on the
       // values in the font node.
       if ( !( alignment == X3DFontStyleNode::VERTICAL && font->isLeftToRight() ) )
         if( line != text.begin() )
           moveToNewLine( *line, font );
-      
+
       glMatrixMode(GL_MODELVIEW);
-      glPushMatrix();   
-      
+      glPushMatrix();
+
       // if we have values in the length field we scale in order to have
       // the text line be the same size as the length.
       if( l != line_length.end() ) {
-        if( alignment == X3DFontStyleNode::HORIZONTAL ) 
-          glScalef( (*l)/ font->stringDimensions( *line, alignment ).x, 
+        if( alignment == X3DFontStyleNode::HORIZONTAL )
+          glScalef( (*l)/ font->stringDimensions( *line, alignment ).x,
                     1, 1 );
-        else 
+        else
           glScalef( 1,
-                    (*l)/ font->stringDimensions( *line, alignment ).y, 
+                    (*l)/ font->stringDimensions( *line, alignment ).y,
                     1 );
         ++l;
       }
 
       // justify the line in the major alignment depending on the justify
       // values in font.
-      justifyLine( *line, font ); 
+      justifyLine( *line, font );
       // render the line of text in the style of the font.
       renderTextLine( *line, font );
-    
+
       glMatrixMode(GL_MODELVIEW);
       glPopMatrix();
-      // translate to the next line. What the next line is depends on the 
+      // translate to the next line. What the next line is depends on the
       // values in the font node.
-      if( alignment == X3DFontStyleNode::VERTICAL && font->isLeftToRight() ) 
+      if( alignment == X3DFontStyleNode::VERTICAL && font->isLeftToRight() )
         moveToNewLine( *line, font );
     }
   }
   glMatrixMode(GL_MODELVIEW);
   glPopMatrix();
 }
-  
+
 void Text::DisplayList::callList( bool build_list ) {
   Text *text_node = static_cast< Text * >( getOwner() );
-  X3DFontStyleNode *font = 
+  X3DFontStyleNode *font =
     static_cast< X3DFontStyleNode * >(text_node->fontStyle->getValue());
   if( font && !font->fontsBuilt() ) {
     try {
       font->buildFonts();
-    } catch( const Exception::H3DException &e ) {
+    } catch( const Exception::H3DException ) {
     }
   }
   X3DGeometryNode::DisplayList::callList( build_list );
@@ -431,19 +431,19 @@ void Text::traverseSG( TraverseInfo &ti ) {
 }
 
 void Text::SFBound::update() {
-  X3DFontStyleNode *font_style = 
-    static_cast< SFFontStyleNode * >( routes_in[0] )->getValue(); 
-  const vector<H3DFloat> &length = 
+  X3DFontStyleNode *font_style =
+    static_cast< SFFontStyleNode * >( routes_in[0] )->getValue();
+  const vector<H3DFloat> &_length =
     static_cast< MFFloat * >( routes_in[1] )->getValue();
   H3DFloat max_extent = static_cast< SFFloat * >( routes_in[2] )->getValue();
   const vector< string > &text = static_cast< MFString * >( routes_in[3] )->getValue();
 
   if( !font_style ) font_style = default_font_style.get();
-  
+
 
   try {
     font_style->buildFonts();
-  } catch( const Exception::H3DException &e ) {
+  } catch( const Exception::H3DException ) {
     // Not doing anything here.
   }
 
@@ -451,19 +451,19 @@ void Text::SFBound::update() {
     X3DFontStyleNode::Alignment alignment = font_style->getAlignment();
     X3DFontStyleNode::Justification minor = font_style->getMinorJustification();
     X3DFontStyleNode::Justification major = font_style->getMajorJustification();
-    
+
     Vec3f box_center( 0, 0, 0 );
     Vec3f text_dims = font_style->stringDimensions( text,
                                                     alignment,
-                                                    length );
+                                                    _length );
     if( alignment == X3DFontStyleNode::HORIZONTAL ) {
       if( max_extent > 0 && text_dims.x > max_extent ) text_dims.x = max_extent;
-      if( major == X3DFontStyleNode::BEGIN || 
-          major == X3DFontStyleNode::FIRST ) 
+      if( major == X3DFontStyleNode::BEGIN ||
+          major == X3DFontStyleNode::FIRST )
         box_center.x = text_dims.x / 2;
       else if( major == X3DFontStyleNode::END )
         box_center.x = -text_dims.x / 2;
-      if( minor == X3DFontStyleNode::BEGIN ) 
+      if( minor == X3DFontStyleNode::BEGIN )
         box_center.y = -text_dims.y / 2;
       else if( minor == X3DFontStyleNode::END )
         box_center.y = text_dims.y / 2;
@@ -471,7 +471,7 @@ void Text::SFBound::update() {
         box_center.y = -font_style->descender();
       else if( minor == X3DFontStyleNode::FIRST ) {
         box_center.y = -text_dims.y / 2;
-        if( font_style->isTopToBottom() && text.size() > 0 ) { 
+        if( font_style->isTopToBottom() && text.size() > 0 ) {
           Vec3f line_dims = font_style->stringDimensions( text[0],
                                                           alignment );
           box_center.y += line_dims.y;
@@ -479,23 +479,23 @@ void Text::SFBound::update() {
       }
     } else {
       if( max_extent > 0 && text_dims.y > max_extent ) text_dims.y = max_extent;
-      if( major == X3DFontStyleNode::BEGIN || 
-          major == X3DFontStyleNode::FIRST  ) 
+      if( major == X3DFontStyleNode::BEGIN ||
+          major == X3DFontStyleNode::FIRST  )
         box_center.y = -text_dims.y / 2;
       else if( major == X3DFontStyleNode::END )
         box_center.y = text_dims.y / 2;
       else if( major == X3DFontStyleNode::MIDDLE )
         box_center.y = font_style->descender();
-      if( minor == X3DFontStyleNode::BEGIN || 
-          minor == X3DFontStyleNode::FIRST ) 
+      if( minor == X3DFontStyleNode::BEGIN ||
+          minor == X3DFontStyleNode::FIRST )
         box_center.x = text_dims.x / 2;
       else if( minor == X3DFontStyleNode::END )
         box_center.x = -text_dims.x / 2;
     }
-  
+
     if( !font_style->isLeftToRight() ) box_center.x = -box_center.x;
     if( !font_style->isTopToBottom() ) box_center.y = -box_center.y;
-  
+
     BoxBound *bb= new BoxBound;
     box_center.y += font_style->descender();
     bb->size->setValue( text_dims );
@@ -506,8 +506,8 @@ void Text::SFBound::update() {
 }
 
 bool Text::lineIntersect(
-                  const Vec3f &from, 
-                  const Vec3f &to,    
+                  const Vec3f &from,
+                  const Vec3f &to,
                   LineIntersectResult &result ) {
   if( result.detect_pt_device && !result.hasCurrentPointingDevice() ) {
     // If this function is called because we are detecting pointing device
@@ -534,41 +534,36 @@ bool Text::lineIntersect(
 void Text::OutputUpdater::update() {
   Text * text_node = static_cast< Text * >( getOwner() );
   // Do not bother if length and maxExtent are not defaults
-  if ( text_node->length->getValue().size() != 0 || 
-       text_node->maxExtent->getValue() != 0 ) 
+  if ( text_node->length->getValue().size() != 0 ||
+       text_node->maxExtent->getValue() != 0 )
     return;
 
   BoxBound * bb = dynamic_cast< BoxBound *>( text_node->bound->getValue() );
   if ( bb == NULL ) return;
 
-  X3DFontStyleNode * font_style = 
-    static_cast< SFFontStyleNode * >( routes_in[0] )->getValue();
-  const vector< string > &text = 
-    static_cast< MFString * >( routes_in[1] )->getValue();
-  
   const Vec3f &size = bb->size->getValue();
   Vec3f center = bb->center->getValue();
 
   // update textBounds
-  text_node->textBounds->setValue( 
+  text_node->textBounds->setValue(
     Vec2f(size.x, size.y), text_node->id );
   // update origin
-  text_node->origin->setValue( 
+  text_node->origin->setValue(
     Vec3f(center.x-(size.x/2), center.y+(size.y/2), 0), text_node->id );
   // update line bounds
   text_node->updateLineBounds();
 }
 
-void Text::updateLineBounds() {  
+void Text::updateLineBounds() {
   BoxBound * bb = dynamic_cast< BoxBound * >( bound->getValue() );
   if ( bb == NULL ) return;
 
-  X3DFontStyleNode * font_style 
+  X3DFontStyleNode * font_style
     = static_cast< X3DFontStyleNode * >( fontStyle->getValue() );
   if( !font_style ) font_style = default_font_style.get();
-  X3DFontStyleNode::Alignment alignment 
+  X3DFontStyleNode::Alignment alignment
     = font_style->getAlignment();
-  X3DFontStyleNode::Justification justification 
+  X3DFontStyleNode::Justification justification
     = font_style->getMajorJustification();
 
   vector< string > text = stringF->getValue();
@@ -577,7 +572,7 @@ void Text::updateLineBounds() {
   if ( alignment == X3DFontStyleNode::HORIZONTAL ) {
     // Case of horizontal alignment
     H3DFloat offset = bb->size->getValue().x;
-    for ( vector<string>::const_iterator i = text.begin(); 
+    for ( vector<string>::const_iterator i = text.begin();
       i != text.end(); ++i ) {
       const string &line = *i;
       Vec3f dim = font_style->
@@ -602,7 +597,7 @@ void Text::updateLineBounds() {
   } else {
     // Case of vertical alignment
     H3DFloat offset = bb->size->getValue().y;
-    for ( vector<string>::const_iterator i = text.begin(); 
+    for ( vector<string>::const_iterator i = text.begin();
       i != text.end(); ++i ) {
       const string &line = *i;
       Vec3f dim = font_style->
