@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -68,11 +68,31 @@ Group* X3D::createX3DFromURL( const string &url,
                               DEFNodes *exported_nodes,
                               PrototypeVector *prototypes,
                               bool change_base_path_during_parsing ) {
+  H3DTIMER_BEGIN( "createX3DFromURL (" + url + ")", "PARSING" ) 
   // First try to resolve the url to file contents and load via string buffer
   // Otherwise fallback on using temp files
   string url_contents= ResourceResolver::resolveURLAsString ( url );
   if ( url_contents != "" ) {
-    return createX3DFromString ( url_contents, dn, exported_nodes, prototypes );
+    // Ensure that we set the base URL even when returning file contents
+    string old_base = ResourceResolver::getBaseURL();
+    string::size_type to = url.find_last_of( "/\\" );
+    if( to != string::npos ) {
+      string base = url.substr( 0, to + 1 );
+      string new_base = old_base + base;
+      if( ResourceResolver::getURNResolver() ) {
+        base = ResourceResolver::getURNResolver()->resolveURN( base );
+        if( base.size() > 1 && base[1] == ':' ) {
+          new_base = base;
+        }
+      }
+      ResourceResolver::setBaseURL( new_base );
+    }
+
+    Group* g= createX3DFromString ( url_contents, dn, exported_nodes, prototypes );
+
+    ResourceResolver::setBaseURL( old_base );
+    H3DTIMER_END( "createX3DFromURL (" + url + ")" ) 
+    return g;
   }
 
   bool is_tmp_file;
@@ -87,7 +107,7 @@ Group* X3D::createX3DFromURL( const string &url,
   }
   string old_base = ResourceResolver::getBaseURL();
 
-#ifdef WIN32
+#ifdef H3D_WINDOWS
   // needed when running H3DAPI as a plugin to a web-browser
   // and if url points to a local file.
   // Needed for plugin for IE
@@ -101,7 +121,7 @@ Group* X3D::createX3DFromURL( const string &url,
   if( path.find( remove_string ) == 0 ) {
     path = path.substr( remove_string.size(), pos + 1 );
   }
-#endif
+#endif // H3D_WINDOWS
 
   if( change_base_path_during_parsing )
     ResourceResolver::setBaseURL( path ); 
@@ -162,6 +182,7 @@ Group* X3D::createX3DFromURL( const string &url,
       if( is_tmp_file ) 
         ResourceResolver::releaseTmpFileName( resolved_url );
       ResourceResolver::setBaseURL( old_base );
+      H3DTIMER_END( "createX3DFromURL (" + url + ")" ) 
       return g;
     }
     istest.close();
@@ -179,6 +200,7 @@ Group* X3D::createX3DFromURL( const string &url,
       parser->parse( IStreamInputSource( is, url_ch ) );
     } catch(...) {
       delete[] url_ch;
+      H3DTIMER_END( "createX3DFromURL (" + url + ")" ) 
       throw;
     }
     delete[] url_ch;
@@ -194,9 +216,10 @@ Group* X3D::createX3DFromURL( const string &url,
   AutoRef< Node > n = handler.getResultingNode();
   if( n.get() )
     g->children->push_back( n.get() );
+  H3DTIMER_END( "createX3DFromURL (" + url + ")" ) 
   return g;
 #else
-  Console(3) << "H3D API compiled without HAVE_XERCES flag. X3D-XML files "
+  Console(LogLevel::Warning) << "H3D API compiled without HAVE_XERCES flag. X3D-XML files "
        << "are not supported" << endl;
   return 0;
 #endif
@@ -224,6 +247,7 @@ AutoRef< Node > X3D::createX3DNodeFromString( const string &str,
     return createVRMLNodeFromString( str, dn, exported_nodes, prototypes );
   else {
 #ifdef HAVE_XERCES
+    H3DTIMER_BEGIN( "createX3DNodeFromString", "PARSING" ) 
     auto_ptr< SAX2XMLReader > parser( getNewXMLParser() );
     X3DSAX2Handlers handler( dn, exported_nodes, prototypes );
     stringstream s;
@@ -232,9 +256,10 @@ AutoRef< Node > X3D::createX3DNodeFromString( const string &str,
     parser->setErrorHandler(&handler); 
     parser->setLexicalHandler( &handler );
     parser->parse( IStreamInputSource( s, (const XMLCh*)L"<string input>" ) );
+    H3DTIMER_END( "createX3DNodeFromString") 
     return handler.getResultingNode();
 #else
-  Console(3) << "H3D API compiled without HAVE_XERCES flag. X3D-XML files "
+  Console(LogLevel::Warning) << "H3D API compiled without HAVE_XERCES flag. X3D-XML files "
        << "are not supported" << endl;
   return AutoRef< Node >(NULL);
 #endif
@@ -246,11 +271,31 @@ AutoRef< Node > X3D::createX3DNodeFromURL( const string &url,
                                            DEFNodes *exported_nodes,
                                            PrototypeVector *prototypes,
                                            bool change_base_path_during_parsing ) {
+  H3DTIMER_BEGIN( "createX3DNodeFromURL(" + url + ")", "PARSING" ) 
   // First try to resolve the url to file contents and load via string buffer
   // Otherwise fallback on using temp files
   string url_contents= ResourceResolver::resolveURLAsString ( url );
   if ( url_contents != "" ) {
-    return createX3DNodeFromString ( url_contents, dn, exported_nodes, prototypes );
+    // Ensure that we set the base URL even when returning file contents
+    string old_base = ResourceResolver::getBaseURL();
+    string::size_type to = url.find_last_of( "/\\" );
+    if ( to != string::npos ) {
+      string base = url.substr ( 0, to+1 );
+      string new_base = old_base + base;
+      if( ResourceResolver::getURNResolver() ) {
+        base = ResourceResolver::getURNResolver()->resolveURN( base );
+        if( base.size() > 1 && base[1] == ':' ) {
+          new_base = base;
+        }
+      }
+      ResourceResolver::setBaseURL( new_base );
+    }
+      
+     AutoRef< Node > n = createX3DNodeFromString ( url_contents, dn, exported_nodes, prototypes );
+    
+    ResourceResolver::setBaseURL( old_base );
+    H3DTIMER_END( "createX3DNodeFromURL(" + url + ")" )
+    return n;
   }
 
   bool is_tmp_file;
@@ -265,7 +310,7 @@ AutoRef< Node > X3D::createX3DNodeFromURL( const string &url,
   }
   string old_base = ResourceResolver::getBaseURL();
 
-#ifdef WIN32
+#ifdef H3D_WINDOWS
   // needed when running H3DAPI as a plugin to a web-browser
   // and if url points to a local file.
   // Needed for plugin for IE
@@ -279,7 +324,7 @@ AutoRef< Node > X3D::createX3DNodeFromURL( const string &url,
   if( path.find( remove_string ) == 0 ) {
     path = path.substr( remove_string.size(), pos + 1 );
   }
-#endif
+#endif // H3D_WINDOWS
 
   if( change_base_path_during_parsing )
     ResourceResolver::setBaseURL( path ); 
@@ -343,6 +388,7 @@ AutoRef< Node > X3D::createX3DNodeFromURL( const string &url,
       if( is_tmp_file ) 
         ResourceResolver::releaseTmpFileName( resolved_url );
       ResourceResolver::setBaseURL( old_base );
+      H3DTIMER_END( "createX3DNodeFromURL(" + url + ")" )
       return n;
     }
     istest.close();
@@ -364,11 +410,13 @@ AutoRef< Node > X3D::createX3DNodeFromURL( const string &url,
   if( is_tmp_file ) 
     ResourceResolver::releaseTmpFileName( resolved_url );
   ResourceResolver::setBaseURL( old_base );
-
+  
+  H3DTIMER_END( "createX3DNodeFromURL(" + url + ")" )
+ 
 #ifdef HAVE_XERCES
   return handler.getResultingNode();
 #else
-  Console(3) << "H3D API compiled without HAVE_XERCES flag. X3D-XML files "
+  Console(LogLevel::Warning) << "H3D API compiled without HAVE_XERCES flag. X3D-XML files "
        << "are not supported" << endl;
   return AutoRef< Node >(NULL);
 #endif
@@ -394,7 +442,7 @@ AutoRef< Node > X3D::createX3DNodeFromStream( istream &is,
   delete[] system_id_ch;
   return handler.getResultingNode();
 #else
-  Console(3) << "H3D API compiled without HAVE_XERCES flag. X3D-XML files "
+  Console(LogLevel::Warning) << "H3D API compiled without HAVE_XERCES flag. X3D-XML files "
        << "are not supported" << endl;
   return AutoRef< Node >( NULL );
 #endif
@@ -465,7 +513,7 @@ void X3D::writeNodeAsX3DHelp( ostream& os,
     }
   } else if( output_type == VRML_OUTPUT ) {
     if( !use_node ) {
-      os << prefix << "DEF " << name << " " << node_name << " {" << endl;;
+      os << prefix << "DEF " << name << " " << node_name << " {" << endl;
       visited_nodes.insert( node );
     } else {
       os << prefix << "USE " << name << endl;
@@ -485,68 +533,70 @@ void X3D::writeNodeAsX3DHelp( ostream& os,
     H3DDynamicFieldsObject * dyn_f_obj =
       dynamic_cast< H3DDynamicFieldsObject * >(node);
     AutoRef< Node > default_value_node( H3DNodeDatabase::createNode( node_name ) );
-    for( H3DNodeDatabase::FieldDBConstIterator i = db->fieldDBBegin();
-         i != db->fieldDBEnd(); ++i ) {
-      Field *f = node->getField( *i );
-      // In the case of dynamic fields a field in the field database might not
-      // exist for this node, in that case just check next.
-      if( !f )
-        continue;
-      Field::AccessType access_type = f->getAccessType();
-      // check if field is a dynamic field.
-      bool dynamic_field = false;
-      H3DDynamicFieldsObject::field_iterator j;
-      if( dyn_f_obj ) {
-        for( j = dyn_f_obj->firstField();
-             j != dyn_f_obj->endField(); ++j )
-          if( (*j) == f ) {
-            dynamic_field = true;
-            break;
-          }
-      }
-
-      if( dynamic_field ) {
-        // Dynamic fields are added to other vectors.
-        if( MFNode *mf_node = dynamic_cast< MFNode * >( f ) ) {
-          dyn_mf_nodes.push_back( make_pair( (*j)->getName(), mf_node ) );
-        } else if( SFNode *sf_node = dynamic_cast< SFNode * >( f ) ) {
-          dyn_sf_nodes.push_back( make_pair( (*j)->getName(), sf_node ) );
-        } else if( ParsableField *p_field = 
-                   dynamic_cast< ParsableField * >( f ) ) {
-          dyn_parse_fields.push_back( make_pair( (*j)->getName(), p_field ) );
+    if( default_value_node.get() ) {
+      for( H3DNodeDatabase::FieldDBConstIterator k = db->fieldDBBegin();
+           k != db->fieldDBEnd(); ++k ) {
+        Field *f = node->getField( *k );
+        // In the case of dynamic fields a field in the field database might not
+        // exist for this node, in that case just check next.
+        if( !f )
+          continue;
+        Field::AccessType access_type = f->getAccessType();
+        // check if field is a dynamic field.
+        bool dynamic_field = false;
+        H3DDynamicFieldsObject::field_iterator j;
+        if( dyn_f_obj ) {
+          for( j = dyn_f_obj->firstField();
+               j != dyn_f_obj->endField(); ++j )
+            if( (*j) == f ) {
+              dynamic_field = true;
+              break;
+            }
         }
-      } else if( access_type != Field::INPUT_ONLY &&
-                 access_type != Field::OUTPUT_ONLY ) {
-        if( MFNode *mf_node = dynamic_cast< MFNode * >( f ) ) {
-          if( mf_node->size() > 0 )
-            mf_nodes.push_back( make_pair( *i, mf_node ) );
-        } else if( SFNode *sf_node = dynamic_cast< SFNode * >( f ) ) {
-          if( sf_node->getValue() )
-            sf_nodes.push_back( make_pair( *i, sf_node ) );
-        } else if( ParsableField *p_field = 
-                   dynamic_cast< ParsableField * >( f ) ){
-          // only add value if it is different from the default value.
-          ParsableField *default_field = dynamic_cast< ParsableField * >(
-               default_value_node->getField( *i ) );
-          string v = p_field->getValueAsString();
-          if( default_field && default_field->getValueAsString() != v ) {
-            if( output_type == X3D_OUTPUT ) {
-              os << *i << "=\'" << v << "\' ";
-            } else if( output_type == VRML_OUTPUT ) {
-              // VRML versions of true and false are upper case.
-              if( dynamic_cast< MFBool * >( default_field ) || 
-                  dynamic_cast< SFBool * >( default_field ) ){
-                for (size_t j=0; j<v.length(); ++j) {
-                  v[j]=toupper(v[j]);
-                }
-              }
 
-              if( dynamic_cast< MFieldClass * >( default_field ) ) { 
-                os << new_prefix << *i << " [" << v << "]" << endl;
-              } else if( dynamic_cast< SFString *>( default_field ) ) {
-                os << new_prefix << *i << " \"" << v << "\"" << endl;
-              } else {
-                os << new_prefix << *i << " " << v << endl;
+        if( dynamic_field ) {
+          // Dynamic fields are added to other vectors.
+          if( MFNode *mf_node = dynamic_cast< MFNode * >( f ) ) {
+            dyn_mf_nodes.push_back( make_pair( (*j)->getName(), mf_node ) );
+          } else if( SFNode *sf_node = dynamic_cast< SFNode * >( f ) ) {
+            dyn_sf_nodes.push_back( make_pair( (*j)->getName(), sf_node ) );
+          } else if( ParsableField *p_field = 
+                     dynamic_cast< ParsableField * >( f ) ) {
+            dyn_parse_fields.push_back( make_pair( (*j)->getName(), p_field ) );
+          }
+        } else if( access_type != Field::INPUT_ONLY &&
+                   access_type != Field::OUTPUT_ONLY ) {
+          if( MFNode *mf_node = dynamic_cast< MFNode * >( f ) ) {
+            if( mf_node->size() > 0 )
+              mf_nodes.push_back( make_pair( *k, mf_node ) );
+          } else if( SFNode *sf_node = dynamic_cast< SFNode * >( f ) ) {
+            if( sf_node->getValue() )
+              sf_nodes.push_back( make_pair( *k, sf_node ) );
+          } else if( ParsableField *p_field = 
+                     dynamic_cast< ParsableField * >( f ) ){
+            // only add value if it is different from the default value.
+            ParsableField *default_field = dynamic_cast< ParsableField * >(
+                 default_value_node->getField( *k ) );
+            string v = p_field->getValueAsString();
+            if( default_field && default_field->getValueAsString() != v ) {
+              if( output_type == X3D_OUTPUT ) {
+                os << *k << "=\'" << v << "\' ";
+              } else if( output_type == VRML_OUTPUT ) {
+                // VRML versions of true and false are upper case.
+                if( dynamic_cast< MFBool * >( default_field ) || 
+                    dynamic_cast< SFBool * >( default_field ) ){
+                  for (size_t l=0; l<v.length(); ++l) {
+                    v[l]=toupper(v[l]);
+                  }
+                }
+
+                if( dynamic_cast< MFieldClass * >( default_field ) ) { 
+                  os << new_prefix << *k << " [" << v << "]" << endl;
+                } else if( dynamic_cast< SFString *>( default_field ) ) {
+                  os << new_prefix << *k << " \"" << v << "\"" << endl;
+                } else {
+                  os << new_prefix << *k << " " << v << endl;
+                }
               }
             }
           }
@@ -573,13 +623,13 @@ void X3D::writeNodeAsX3DHelp( ostream& os,
     }
 
     // Handle dynamic parsable fields
-    for( unsigned int i = 0; i < dyn_parse_fields.size(); ++i ) {
+    for( unsigned int k = 0; k < dyn_parse_fields.size(); ++k ) {
       if( output_type == X3D_OUTPUT ) {
         Field::AccessType access_type =
-          dyn_parse_fields[i].second->getAccessType();
-        os << "<field name=\"" << dyn_parse_fields[i].first << "\" type=\""
-           << dyn_parse_fields[i].second->getTypeName()
-           << "\" value=\"" << dyn_parse_fields[i].second->getValueAsString()
+          dyn_parse_fields[k].second->getAccessType();
+        os << "<field name=\"" << dyn_parse_fields[k].first << "\" type=\""
+           << dyn_parse_fields[k].second->getTypeName()
+           << "\" value=\"" << dyn_parse_fields[k].second->getValueAsString()
            << "\" accessType=\"";
         if( access_type == Field::INPUT_OUTPUT )
           os << "inputOutput";
@@ -591,22 +641,22 @@ void X3D::writeNodeAsX3DHelp( ostream& os,
           os << "initializeOnly";
         os << "\"/>" << endl;
       } else if( output_type == VRML_OUTPUT ) {
-        string v = dyn_parse_fields[i].second->getValueAsString();
+        string v = dyn_parse_fields[k].second->getValueAsString();
 
         // VRML versions of true and false are upper case.
-        if( dynamic_cast< MFBool * >( dyn_parse_fields[i].second ) ||
-            dynamic_cast< SFBool * >( dyn_parse_fields[i].second ) ){
+        if( dynamic_cast< MFBool * >( dyn_parse_fields[k].second ) ||
+            dynamic_cast< SFBool * >( dyn_parse_fields[k].second ) ){
           for (size_t j=0; j<v.length(); ++j) {
             v[j]=toupper(v[j]);
           }
         }
 
         os << new_prefix << "field "
-                         << dyn_parse_fields[i].second->getTypeName();
+                         << dyn_parse_fields[k].second->getTypeName();
 
-        if( dynamic_cast< MFieldClass * >( dyn_parse_fields[i].second ) ) {
+        if( dynamic_cast< MFieldClass * >( dyn_parse_fields[k].second ) ) {
           os << " [" << v << "]" << endl;
-        } else if( dynamic_cast< SFString *>( dyn_parse_fields[i].second ) ) {
+        } else if( dynamic_cast< SFString *>( dyn_parse_fields[k].second ) ) {
           os << " \"" << v << "\"" << endl;
         } else {
           os << " " << v << endl;
@@ -615,29 +665,29 @@ void X3D::writeNodeAsX3DHelp( ostream& os,
     }
 
     // process child nodes
-    for( unsigned int i = 0; i < sf_nodes.size(); ++i ) {
+    for( unsigned int k = 0; k < sf_nodes.size(); ++k ) {
       if( output_type == VRML_OUTPUT ) {
-        os << new_prefix << sf_nodes[i].first << " ";
+        os << new_prefix << sf_nodes[k].first << " ";
       }
 
       X3D::writeNodeAsX3DHelp( os, 
-             sf_nodes[i].second->getValue(), 
-             sf_nodes[i].first,
+             sf_nodes[k].second->getValue(), 
+             sf_nodes[k].first,
              new_prefix,
              visited_nodes,
            output_type);
     }
 
-    for( unsigned int i = 0; i < mf_nodes.size(); ++i ) {
+    for( unsigned int k = 0; k < mf_nodes.size(); ++k ) {
       if( output_type == VRML_OUTPUT ) {
-         os << new_prefix << mf_nodes[i].first << " ["<< endl;
+         os << new_prefix << mf_nodes[k].first << " ["<< endl;
       }
 
-      for( MFNode::const_iterator n = mf_nodes[i].second->begin();
-           n != mf_nodes[i].second->end(); ++n ) {
+      for( MFNode::const_iterator n = mf_nodes[k].second->begin();
+           n != mf_nodes[k].second->end(); ++n ) {
         X3D::writeNodeAsX3DHelp( os, 
          *n, 
-         mf_nodes[i].first, 
+         mf_nodes[k].first, 
          new_prefix + "  ", 
          visited_nodes,
          output_type );  
@@ -651,12 +701,12 @@ void X3D::writeNodeAsX3DHelp( ostream& os,
     // Handle dynamic sf node fields, might not work when exporting
     // to wrl because I do not know the syntax when not using the
     // USE attribute, or if it is even possible.
-    for( unsigned int i = 0; i < dyn_sf_nodes.size(); ++i ) {
+    for( unsigned int k = 0; k < dyn_sf_nodes.size(); ++k ) {
       if( output_type == X3D_OUTPUT ) {
         Field::AccessType access_type =
-          dyn_sf_nodes[i].second->getAccessType();
-        os << "<field name=\"" << dyn_sf_nodes[i].first << "\" type=\""
-           << dyn_sf_nodes[i].second->getTypeName() << "\" accessType=\"";
+          dyn_sf_nodes[k].second->getAccessType();
+        os << "<field name=\"" << dyn_sf_nodes[k].first << "\" type=\""
+           << dyn_sf_nodes[k].second->getTypeName() << "\" accessType=\"";
         if( access_type == Field::INPUT_OUTPUT )
           os << "inputOutput";
         else if( access_type == Field::INPUT_ONLY )
@@ -668,11 +718,11 @@ void X3D::writeNodeAsX3DHelp( ostream& os,
         os << "\" >" << endl;
       } else if( output_type == VRML_OUTPUT ) {
         os << new_prefix << "exposedField "
-           << dyn_sf_nodes[i].second->getTypeName();
+           << dyn_sf_nodes[k].second->getTypeName();
       }
 
       X3D::writeNodeAsX3DHelp( os, 
-             dyn_sf_nodes[i].second->getValue(), 
+             dyn_sf_nodes[k].second->getValue(), 
              "",
              new_prefix,
              visited_nodes,
@@ -686,11 +736,11 @@ void X3D::writeNodeAsX3DHelp( ostream& os,
     // Handle dynamic mf node fields, might not work when exporting
     // to wrl because I do not know the syntax when not using the
     // USE attribute, or if it is even possible.
-    for( unsigned int i = 0; i < dyn_mf_nodes.size(); ++i ) {
+    for( unsigned int k = 0; k < dyn_mf_nodes.size(); ++k ) {
       if( output_type == X3D_OUTPUT ) {
         Field::AccessType access_type =
-          dyn_mf_nodes[i].second->getAccessType();
-        os << "<field name=\"" << dyn_mf_nodes[i].first
+          dyn_mf_nodes[k].second->getAccessType();
+        os << "<field name=\"" << dyn_mf_nodes[k].first
            << "\" type=\"MFNode\" accessType=\"";
         if( access_type == Field::INPUT_OUTPUT )
           os << "inputOutput";
@@ -703,11 +753,11 @@ void X3D::writeNodeAsX3DHelp( ostream& os,
         os << "\"/>" << endl;
       } else if( output_type == VRML_OUTPUT ) {
          os << new_prefix << "exposedField "
-           << dyn_sf_nodes[i].second->getTypeName() << "[";
+           << dyn_sf_nodes[k].second->getTypeName() << "[";
       }
 
-      for( MFNode::const_iterator n = dyn_mf_nodes[i].second->begin();
-           n != dyn_mf_nodes[i].second->end(); ++n ) {
+      for( MFNode::const_iterator n = dyn_mf_nodes[k].second->begin();
+           n != dyn_mf_nodes[k].second->end(); ++n ) {
         X3D::writeNodeAsX3DHelp( os, 
          *n, 
          "", 
