@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -60,12 +60,13 @@ H3DNodeDatabase X3DGeometryNode::database( "X3DGeometryNode",
                                            &X3DChildNode::database );
 
 namespace X3DGeometryNodeInternals {
-  FIELDDB_ELEMENT( X3DGeometryNode, isTouched, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( X3DGeometryNode, force, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( X3DGeometryNode, contactPoint, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( X3DGeometryNode, contactNormal, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( X3DGeometryNode, options, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( X3DGeometryNode, contactTexCoord, OUTPUT_ONLY );
+  FIELDDB_ELEMENT( X3DGeometryNode, isTouched, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( X3DGeometryNode, force, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( X3DGeometryNode, contactPoint, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( X3DGeometryNode, contactNormal, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( X3DGeometryNode, options, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( X3DGeometryNode, contactTexCoord, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( X3DGeometryNode, centerOfMass, INPUT_OUTPUT )
 }
 
 X3DGeometryNode::X3DGeometryNode( 
@@ -77,7 +78,8 @@ X3DGeometryNode::X3DGeometryNode(
                                  Inst< MFVec3f > _contactPoint,
                                  Inst< MFVec3f > _contactNormal,
                                  Inst< MFVec3f > _contactTexCoord,
-                                 Inst< SFBoundTree > _boundTree ) :
+                                 Inst< SFBoundTree > _boundTree,
+                                 Inst< SFVec3f > _centerOfMass ) :
   X3DChildNode( _metadata ),
   H3DBoundedObject( _bound ),
   H3DDisplayListObject( _displayList ),
@@ -87,6 +89,7 @@ X3DGeometryNode::X3DGeometryNode(
   contactTexCoord( _contactTexCoord ),
   contactNormal( _contactNormal ),
   boundTree( _boundTree ),
+  centerOfMass ( _centerOfMass ),
   options( new MFOptionsNode ),
   shadow_volume( NULL ),
   use_culling( false ),
@@ -104,6 +107,7 @@ X3DGeometryNode::X3DGeometryNode(
   database.initFields( this );
 
   displayList->route( boundTree );
+  centerOfMass->setValue(Vec3f(0.0,0.0,0.0));
 }
 
 void X3DGeometryNode::initialize() {
@@ -202,25 +206,12 @@ HAPI::HAPIHapticShape *X3DGeometryNode::getOpenGLHapticShape(
     } else if( shape == "DEPTH_BUFFER" ) {
       type = 1;
     } else {
-      Console(4) << "Warning: Invalid OpenHaptics GLShape type: "
+      Console(LogLevel::Error) << "Warning: Invalid OpenHaptics GLShape type: "
                  << shape 
                  << ". Must be \"FEEDBACK_BUFFER\" or \"DEPTH_BUFFER\" "
                  << "(in \"" << getName() << "\")" << endl;
     }
-    // todo: fix touchable_face
-/*    const string &face = openhaptics_options->touchableFace->getValue();
-    if( face == "FRONT" ) touchable_face = HL_FRONT;
-    else if( face == "BACK" ) touchable_face = HL_BACK;
-    else if( face == "FRONT_AND_BACK" ) touchable_face = HL_FRONT_AND_BACK;
-    else if( face == "AS_GRAPHICS" ) {
-      // default values are the same as graphics
-    } else {
-      Console(4) << "Warning: Invalid default OpenHaptics touchable face: "
-                 << face 
-                 << ". Must be \"FRONT\", \"BACK\" or \"FRONT_AND_BACK\" "
-                 << "(in active OpenHapticsSettings node\")" << endl;
-                 }*/
-    
+
     adaptive_viewport = openhaptics_options->useAdaptiveViewport->getValue();
     camera_view = openhaptics_options->useHapticCameraView->getValue();
   }
@@ -264,8 +255,8 @@ void X3DGeometryNode::DisplayList::callList( bool build_list ) {
   GLboolean culling_enabled;
   glGetBooleanv( GL_CULL_FACE, &culling_enabled );
 
-  GLint cull_face;
-  glGetIntegerv( GL_CULL_FACE_MODE, &cull_face );
+  GLint old_cull_face;
+  glGetIntegerv( GL_CULL_FACE_MODE, &old_cull_face );
 
   if( geom->usingCulling() && geom->allowingCulling() ) {
     glEnable( GL_CULL_FACE );
@@ -315,6 +306,7 @@ void X3DGeometryNode::DisplayList::callList( bool build_list ) {
         glMatrixMode( GL_MODELVIEW );
         glPushMatrix();
         tree->renderBounds( render_depth );
+        glMatrixMode( GL_MODELVIEW );
         glPopMatrix();
       }
     }
@@ -326,31 +318,32 @@ void X3DGeometryNode::DisplayList::callList( bool build_list ) {
       }
     }
   }
-  
+
+
   // restore previous values for culling
   if( culling_enabled ) glEnable( GL_CULL_FACE );
   else glDisable( GL_CULL_FACE );
   
-  glCullFace( cull_face );
+  glCullFace( old_cull_face );
 
 }
 
 unsigned int X3DGeometryNode::DisplayList::cachingDelay(){
   // use local graphic option first, if local graphic option is null, check global one
-  GraphicsOptions *options = NULL;
+  GraphicsOptions *_options = NULL;
   X3DGeometryNode *geom = static_cast< X3DGeometryNode * >( getOwner() );
-  geom->getOptionNode( options );
-  if( options ) {// This geometry contains graphic option 
-    return options->cachingDelay->getValue();
+  geom->getOptionNode( _options );
+  if( _options ) {// This geometry contains graphic option 
+    return _options->cachingDelay->getValue();
   }
   else {// check global graphic option instead
     GlobalSettings *default_settings = GlobalSettings::getActive();
     if( default_settings&&default_settings->optionNodesUpdated() ) {
-      default_settings->getOptionNode( options );
-      if( options ) {// update graphic options
-        graphic_options_previous = options;
+      default_settings->getOptionNode( _options );
+      if( _options ) {// update graphic options
+        graphic_options_previous = _options;
         //cache_delay_previous = options->cachingDelay->getValue();
-        return options->cachingDelay->getValue();
+        return _options->cachingDelay->getValue();
       }else{
         // global setting change in last frame, but no graphic in it now
         graphic_options_previous = NULL;
@@ -379,16 +372,16 @@ bool X3DGeometryNode::DisplayList::usingCaching(){
   if( cache_mode == ON ) return true;
   if( cache_mode == OFF ) return false;
 
-  GraphicsOptions *options = NULL;
+  GraphicsOptions *_options = NULL;
 
-  geom->getOptionNode( options );
-  if( !options ) {// no local option, try to use global setting
+  geom->getOptionNode( _options );
+  if( !_options ) {// no local option, try to use global setting
     GlobalSettings *default_settings = GlobalSettings::getActive();
     if( default_settings&&default_settings->optionNodesUpdated() ) {
-      default_settings->getOptionNode( options );
-      if( options ) { // global graphic option exist and need update
-        graphic_options_previous = options;
-        return options->useCaching->getValue();
+      default_settings->getOptionNode( _options );
+      if( _options ) { // global graphic option exist and need update
+        graphic_options_previous = _options;
+        return _options->useCaching->getValue();
       }else{
         // no graphic option in updated global setting
         graphic_options_previous = NULL;
@@ -406,7 +399,7 @@ bool X3DGeometryNode::DisplayList::usingCaching(){
     }
   }else{
     // local option exist, use local option
-    return options->useCaching->getValue();
+    return _options->useCaching->getValue();
   }
 }
 
@@ -425,18 +418,18 @@ void X3DGeometryNode::SFBoundTree::update() {
                                                     lines, 
                                                     points );
   
-  GeometryBoundTreeOptions *options = NULL;
-  geometry->getOptionNode( options );
-  if( !options ) {
+  GeometryBoundTreeOptions *_options = NULL;
+  geometry->getOptionNode( _options );
+  if( !_options ) {
     GlobalSettings *default_settings = GlobalSettings::getActive();
     if( default_settings ) {
-      default_settings->getOptionNode( options );
+      default_settings->getOptionNode( _options );
     }
   }
 
-  if( options ) {
-    const string &type = options->boundType->getValue();
-    H3DInt32 max_triangles = options->maxTrianglesInLeaf->getValue();
+  if( _options ) {
+    const string &type = _options->boundType->getValue();
+    H3DInt32 max_triangles = _options->maxTrianglesInLeaf->getValue();
     if( type == "AABB" ) {
       value = new HAPI::Collision::AABBTree( triangles,
                                           lines,
@@ -453,7 +446,7 @@ void X3DGeometryNode::SFBoundTree::update() {
                                                  points,
                                                  max_triangles );
     } else {
-      Console(4) << "Warning: Invalid boundType: "
+      Console(LogLevel::Error) << "Warning: Invalid boundType: "
                  << type
                  << ". Must be \"SPHERE\", \"OBB\" or \"AABB\" "
                  << "(in active GeometryBoundTreeOptions node for \" "
@@ -474,6 +467,16 @@ void X3DGeometryNode::SFBoundTree::update() {
 void X3DGeometryNode::traverseSG( TraverseInfo &ti ) {
   X3DChildNode::traverseSG( ti );
   
+  if( !displayList->isUpToDate() ) {
+    Bound *b = bound->getValue();
+    if( b ) {
+      BoxBound *bb = dynamic_cast<BoxBound*>(b);
+      if( bb ) {
+        centerOfMass->setValue( bb->center->getValue() );
+      }
+    }
+  }
+
   if( H3DRenderModeGroupNode::current_render_mode_group ) {
     // We do not know what OpenGL states are changed  by the 
     // H3DRenderModeGroupNode, so we cannot have it cached in a display
@@ -573,7 +576,7 @@ void X3DGeometryNode::createAndAddHapticShapes(
     else if( face == "AS_GRAPHICS" ) {
       // default values are the same as graphics
     } else {
-      Console(4) << "Warning: Invalid default touchable face: "
+      Console(LogLevel::Error) << "Warning: Invalid default touchable face: "
         << face 
         << ". Must be \"FRONT\", \"BACK\" or \"FRONT_AND_BACK\" "
         << "(in active HapticsOptions node\" )" << endl;
@@ -657,7 +660,7 @@ void X3DGeometryNode::createAndAddHapticShapes(
   if( print_negative_scaling_warning ) {
     Matrix3f m3 = ti.getAccForwardMatrix().getScaleRotationPart();
     if( ( m3.getRow( 0 ) % m3.getRow( 1 ) ) * m3.getRow(2) < 0 ) {
-      Console(3) << "Warning: A parent transform node to the X3DGeometryNode "
+      Console(LogLevel::Warning) << "Warning: A parent transform node to the X3DGeometryNode "
                  << getName() << " contains a negative scaling coefficient. "
                  << " Haptics will most likely not be rendered correctly."
                  << endl;
@@ -694,7 +697,7 @@ void X3DGeometryNode::createAndAddHapticShapes(
         type = HAPI::OpenHapticsRenderer::OpenHapticsOptions::CUSTOM;
       } else {
         type = HAPI::OpenHapticsRenderer::OpenHapticsOptions::FEEDBACK_BUFFER;
-        Console(4) << "Warning: Invalid OpenHaptics GLShape type: "
+        Console(LogLevel::Error) << "Warning: Invalid OpenHaptics GLShape type: "
           << shape 
           << ". Must be \"FEEDBACK_BUFFER\", \"DEPTH_BUFFER\" "
           << "or \"CUSTOM\""
@@ -745,7 +748,7 @@ void X3DGeometryNode::createAndAddHapticShapes(
         type = HAPI::OpenHapticsRenderer::OpenHapticsOptions::CUSTOM;
       } else {
         type = HAPI::OpenHapticsRenderer::OpenHapticsOptions::FEEDBACK_BUFFER;
-        Console(4) << "Warning: Invalid OpenHaptics GLShape type: "
+        Console(LogLevel::Error) << "Warning: Invalid OpenHaptics GLShape type: "
           << shape 
           << ". Must be \"FEEDBACK_BUFFER\", \"DEPTH_BUFFER\" "
           << "or \"CUSTOM\""
@@ -797,7 +800,7 @@ void X3DGeometryNode::createAndAddHapticShapes(
         type = HAPI::OpenHapticsRenderer::OpenHapticsOptions::CUSTOM;
       } else {
         type = HAPI::OpenHapticsRenderer::OpenHapticsOptions::FEEDBACK_BUFFER;
-        Console(4) << "Warning: Invalid OpenHaptics GLShape type: "
+        Console(LogLevel::Error) << "Warning: Invalid OpenHaptics GLShape type: "
           << shape 
           << ". Must be \"FEEDBACK_BUFFER\", \"DEPTH_BUFFER\" "
           << "or \"CUSTOM\""
@@ -908,7 +911,7 @@ void X3DGeometryNode::addDynamicInfoToShape( TraverseInfo &ti,
           shape->setForceDynamic( true );
         else  if( !(dynamic_mode == "TRANSFORM_CHANGED" ||
                     dynamic_mode == "ALWAYS" ) ) {
-          Console(4) << "Warning: Invalid dynamic mode: "
+          Console(LogLevel::Error) << "Warning: Invalid dynamic mode: "
                      << dynamic_mode 
                      << ". Must be \"ALWAYS\", \"NEVER\" or \"TRANSFORM_CHANGED\" "
                      << "(in active HapticsOptions node\" )" << endl;
