@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -38,3 +38,7 @@ X3DMetadataObject::X3DMetadataObject( Inst< SFString > _name,
   reference( _reference ) {
 }
 
+ X3DMetadataObject *X3DMetadataObject::getMetadataByName( const string &_name ) {
+   if( _name == nameF->getValue() ) return this; 
+   return NULL;
+ }
