@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -174,7 +174,7 @@ void X3DSAX2Handlers::handleProtoInterfaceFieldElement( const Attributes &attrs
       } else if( !field_value &&  name == "value" ) {
         field_value = attrs.getValue( i );
       } else {
-        Console(3) << "Warning: Unknown attribute \"" << name  
+        Console(LogLevel::Warning) << "Warning: Unknown attribute \"" << name  
              << "\" in 'field' element " << getLocationString() << endl;
       }
     }
@@ -222,7 +222,7 @@ void X3DSAX2Handlers::handleProtoInterfaceFieldElement( const Attributes &attrs
       X3DTypes::stringToType(  toString( field_type ).c_str() );
 
     if( x3d_type == X3DTypes::UNKNOWN_X3D_TYPE ) {
-      Console(3) << "Warning: Invalid value for \"type\" attribute of \"field\" element "
+      Console(LogLevel::Warning) << "Warning: Invalid value for \"type\" attribute of \"field\" element "
            << getLocationString() << endl;
     } else {
       string v = "";
@@ -232,13 +232,13 @@ void X3DSAX2Handlers::handleProtoInterfaceFieldElement( const Attributes &attrs
         v = toString( field_value );
       }
       string error_message = proto_declaration->addFieldDeclaration(
-                                                                                                                        toString( field_name ),
-                              x3d_type,
-                              access_type,
-                              v );
-                        if( error_message != "" ) {
-                                Console(3) << error_message << " " << getLocationString() << endl;
-                        }
+                                                                     toString( field_name ),
+                                                                     x3d_type,
+                                                                     access_type,
+                                                                     v );
+      if( error_message != "" ) {
+        Console(LogLevel::Warning) << error_message << " " << getLocationString() << endl;
+      }
     }
     FieldValue *fv = new FieldValue( toString( field_name ), NULL );
     node_stack.push( NodeElement( fv ) ); 
@@ -267,7 +267,7 @@ Field * X3DSAX2Handlers::handleFieldElement( const Attributes &attrs,
       } else if( !field_value && name == "value" ) {
         field_value = attrs.getValue( i );
       } else {
-        Console(3) << "Warning: Unknown attribute \"" << name  
+        Console(LogLevel::Warning) << "Warning: Unknown attribute \"" << name  
              << "\" in 'field' element " << getLocationString() << endl;
       }
     }
@@ -362,7 +362,7 @@ Field * X3DSAX2Handlers::handleFieldElement( const Attributes &attrs,
                                     (int)locator->getLineNumber() ); 
         }
       } else {
-        Console(3) << "Warning: 'value' attribute ignored. Only used if "
+        Console(LogLevel::Warning) << "Warning: 'value' attribute ignored. Only used if "
              << "accesstype is initializeOnly, inputOnly or inputOutput " 
              << getLocationString() << endl;
       }
@@ -374,7 +374,7 @@ Field * X3DSAX2Handlers::handleFieldElement( const Attributes &attrs,
     node_stack.push( NodeElement( fv ) ); 
     return f;
   }  else {
-    Console(3) << "Warning: 'field' declaration in Node that is does not support"
+    Console(LogLevel::Warning) << "Warning: 'field' declaration in Node that is does not support"
          << " it " << getLocationString() << endl;
     node_stack.push( NodeElement( NULL ) ); 
   }
@@ -402,7 +402,7 @@ void X3DSAX2Handlers::handleRouteElement( const Attributes &attrs,
     } else if( !to_field_name && name == "toField" ) {
       to_field_name = attrs.getValue( i );
     } else {
-      Console(3) << "Warning: Unknown attribute \"" << name  
+      Console(LogLevel::Warning) << "Warning: Unknown attribute \"" << name  
            << "\" in ROUTE element " << getLocationString() << endl;
     }
   }
@@ -446,25 +446,25 @@ void X3DSAX2Handlers::handleRouteElement( const Attributes &attrs,
             else
               from_field->routeNoEvent( to_field );
           } else {
-            Console(3) << "Warning: Route error. Could not find field named \"" 
+            Console(LogLevel::Warning) << "Warning: Route error. Could not find field named \"" 
                  << to_field_name
                  << "\" in \"" << to_node_name << "\" Node " 
                        << getLocationString() << endl;
           }
         } else {
-          Console(3) << "Warning: Route error. Could not find Node named \"" 
+          Console(LogLevel::Warning) << "Warning: Route error. Could not find Node named \"" 
                << to_node_name
                << "\" specified in \"toNode\" attribute " 
                << getLocationString() << endl;
         }
       } else {
-        Console(3) << "Warning: Route error. Could not find field named \"" 
+        Console(LogLevel::Warning) << "Warning: Route error. Could not find field named \"" 
              << from_field_name
              << "\" in \"" << from_node_name << "\" Node " 
              << getLocationString() << endl;
       }
     } else {
-      Console(3) << "Warning: Route error. Could not find Node named \"" 
+      Console(LogLevel::Warning) << "Warning: Route error. Could not find Node named \"" 
            << from_node_name
            << "\" specified in \"fromNode\" attribute " 
            << getLocationString() << endl;
@@ -496,7 +496,7 @@ void X3DSAX2Handlers::handleProgramSettingElement( const Attributes &attrs ) {
     } else if( name == "section" ) {
       setting_section = attrs.getValue( i );
     } else {
-      Console(3) << "Warning: Unknown attribute \"" << name  
+      Console(LogLevel::Warning) << "Warning: Unknown attribute \"" << name  
            << "\" in PROGRAM_SETTING element " << getLocationString() << endl;
     }
   }
@@ -524,13 +524,13 @@ void X3DSAX2Handlers::handleProgramSettingElement( const Attributes &attrs ) {
                                   toString( setting_section )  ); 
                                   
       } else {
-        Console(3) << "Warning: Program setting error. Could not find field named \"" 
+        Console(LogLevel::Warning) << "Warning: Program setting error. Could not find field named \"" 
              << field_name
              << "\" in \"" << node_name << "\" Node " 
              << getLocationString() << endl;
       }
     } else {
-      Console(3) << "Warning: program setting error. Could not find Node named \"" 
+      Console(LogLevel::Warning) << "Warning: program setting error. Could not find Node named \"" 
            << node_name
            << "\" specified in \"node\" attribute " 
            << getLocationString() << endl;
@@ -558,7 +558,7 @@ void X3DSAX2Handlers::handleImportElement( const Attributes &attrs  ) {
     } else if( !as_name && name == "AS" ) {
       as_name = attrs.getValue( i );
     } else {
-      Console(3) << "Warning: Unknown attribute \"" << name  
+      Console(LogLevel::Warning) << "Warning: Unknown attribute \"" << name  
            << "\" in IMPORT element " << getLocationString() << endl;
     }
   }
@@ -590,7 +590,7 @@ void X3DSAX2Handlers::handleImportElement( const Attributes &attrs  ) {
         DEF_map->addNode( toString( as_name ), 
                           import_node ); 
       } else {
-        Console(3) << "Warning: IMPORT error. H3D_EXPORTS " 
+        Console(LogLevel::Warning) << "Warning: IMPORT error. H3D_EXPORTS " 
              << "does not include \""
              << exported_def_name << "\"" 
              << getLocationString() << endl;
@@ -605,18 +605,18 @@ void X3DSAX2Handlers::handleImportElement( const Attributes &attrs  ) {
           DEF_map->addNode( toString( as_name ), 
                             import_node ); 
         } else {
-          Console(3) << "Warning: IMPORT error. Inline node \"" 
+          Console(LogLevel::Warning) << "Warning: IMPORT error. Inline node \"" 
                << inline_def_name << "\" does not EXPORT \""
                << exported_def_name << "\"" 
                << getLocationString() << endl;
         }
       } else {
         if( n ) {
-          Console(3) << "Warning: IMPORT error. Node \"" 
+          Console(LogLevel::Warning) << "Warning: IMPORT error. Node \"" 
                << inline_def_name << "\" is not an Inline node "
                << getLocationString() << endl;
         } else {
-          Console(3) << "Warning: IMPORT error. Node named \"" 
+          Console(LogLevel::Warning) << "Warning: IMPORT error. Node named \"" 
                << inline_def_name << "\" does not exist."
                << getLocationString() << endl;
         }
@@ -640,7 +640,7 @@ void X3DSAX2Handlers::handleExportElement( const Attributes &attrs  ) {
     } else if( !as_name &&  name == "AS" ) {
       as_name = attrs.getValue( i );
     } else {
-      Console(3) << "Warning: Unknown attribute \"" << name  
+      Console(LogLevel::Warning) << "Warning: Unknown attribute \"" << name  
            << "\" in EXPORT element " << getLocationString() << endl;
     }
   }
@@ -662,7 +662,7 @@ void X3DSAX2Handlers::handleExportElement( const Attributes &attrs  ) {
       exported_nodes->addNode( toString( as_name ), 
                                n ); 
     } else {
-      Console(3) << "Warning: EXPORT error. Node named \"" 
+      Console(LogLevel::Warning) << "Warning: EXPORT error. Node named \"" 
            << local_def_name << "\" does not exist."
            << getLocationString() << endl;
     }
@@ -686,7 +686,7 @@ void X3DSAX2Handlers::handleConnectElement( const Attributes &attrs,
   node_stack.push( NodeElement( NULL ) );
 
   if( !defining_proto_connections ) {
-    Console(3) << "Warning: \"connect\" element can only be defined in a IS element "
+    Console(LogLevel::Warning) << "Warning: \"connect\" element can only be defined in a IS element "
          << getLocationString() << endl;
   } else if( proto_instance ) {
 
@@ -702,7 +702,7 @@ void X3DSAX2Handlers::handleConnectElement( const Attributes &attrs,
       } else if( !proto_field_name &&  name =="protoField" ) {
         proto_field_name = attrs.getValue( i );
       } else {
-        Console(3) << "Warning: Unknown attribute \"" << name  
+        Console(LogLevel::Warning) << "Warning: Unknown attribute \"" << name  
              << "\" in EXPORT element " << getLocationString() << endl;
       }
     }
@@ -739,13 +739,13 @@ void X3DSAX2Handlers::handleConnectElement( const Attributes &attrs,
         proto_instance->getField( toString( proto_field_name ) );
       
       if( !node_field ) {
-        Console(3) << "Warning: Could not find field \"" << node_field_name  
+        Console(LogLevel::Warning) << "Warning: Could not find field \"" << node_field_name  
              << "\" in parent node" << getLocationString() << endl;
         return;
       }
       
       if( !proto_field ) {
-        Console(3) << "Warning: Could not find field \"" << proto_field_name  
+        Console(LogLevel::Warning) << "Warning: Could not find field \"" << proto_field_name  
              << "\" in ProtoInterface of PROTO node" 
              << getLocationString() << endl;
         return;
@@ -753,32 +753,32 @@ void X3DSAX2Handlers::handleConnectElement( const Attributes &attrs,
       
       if( node_field->getAccessType() != Field::INPUT_OUTPUT &&
           node_field->getAccessType() != proto_field->getAccessType() ) {
-        Console(3) << "Warning: accessType of \"nodeField\"("
-                                                                         << node_field->getName() << ") and \"protoField\"("
-                                                                         << proto_field->getName() << ") does not match"
-                                                                         << getLocationString() << endl;
+        Console(LogLevel::Warning) << "Warning: accessType of \"nodeField\"("
+                                   << node_field->getName() << ") and \"protoField\"("
+                                   << proto_field->getName() << ") does not match"
+                                   << getLocationString() << endl;
         return;
       }
       
       if( node_field->getX3DType() != proto_field->getX3DType() ) {
-        Console(3) << "Warning: type of \"nodeField\" and \"protoField\" does not match"
+        Console(LogLevel::Warning) << "Warning: type of \"nodeField\" and \"protoField\" does not match"
              << getLocationString() << endl;
         return;
       }
 
-                        const Field::FieldSet &routes_out = node_field->getRoutesOut();
-                        for( Field::FieldSet::const_iterator i = routes_out.begin();
-                                         i != routes_out.end(); ++i ) {
-                                if( (*i)->getOwner() == proto_instance ) {
-                                        Console(3) << "Warning: \"nodeField\"("
-                                                                                 << node_field->getName()
-                                                                                 << ") is already associated with a field in the "
-                                                                                 << "ProtoInterface."
-                                                                                 << getLocationString() << endl;
-                                        return;
-                                }
-                        }
-  
+      const Field::FieldSet &routes_out = node_field->getRoutesOut();
+      for( Field::FieldSet::const_iterator i = routes_out.begin();
+           i != routes_out.end(); ++i ) {
+        if( (*i)->getOwner() == proto_instance ) {
+          Console(LogLevel::Warning) << "Warning: \"nodeField\"("
+                                     << node_field->getName()
+                                     << ") is already associated with a field in the "
+                                     << "ProtoInterface."
+                                     << getLocationString() << endl;
+          return;
+        }
+      }
+
       proto_instance->connectField( toString( proto_field_name ), node_field );
     }
   }
@@ -806,7 +806,7 @@ void X3DSAX2Handlers::protoStartElement( const XMLCh* const uri,
 
   if( defining_proto_body > 0 ) {
     if( defining_proto_connections ) {
-      Console(3) << "Warning: Only \"connect\" elements allowed in IS element "
+      Console(LogLevel::Warning) << "Warning: Only \"connect\" elements allowed in IS element "
                  << getLocationString() << endl;
       node_stack.push( NodeElement( NULL ) );
     } else {
@@ -835,18 +835,18 @@ void X3DSAX2Handlers::protoStartElement( const XMLCh* const uri,
     }
   } else if( localname_string == "ProtoDeclare" ) {
     if( proto_declaration.get() ) {
-      Console(3) << "Warning: ProtoDeclare element not allowed inside other ProtoDeclare element"
+      Console(LogLevel::Warning) << "Warning: ProtoDeclare element not allowed inside other ProtoDeclare element"
            << getLocationString() << endl;
       node_stack.push( NodeElement( NULL ) );
     } else {
       const XMLCh *name = attrs.getValue( gname );
       if( !name ) {
-        Console(3) << "Warning: ProtoDeclare element is missing required \"name\" attibute "
+        Console(LogLevel::Warning) << "Warning: ProtoDeclare element is missing required \"name\" attibute "
              << getLocationString() << endl;
         node_stack.push( NodeElement( NULL ) );
       } else {
         proto_declaration.reset( new ProtoDeclaration( toString( name ), "",
-                                                                                                                                 vector< string >(), proto_declarations ) );
+                                                       vector< string >(), proto_declarations ) );
       }
     }
   } else  if( localname_string == "ExternProtoDeclare" ) {
@@ -854,7 +854,7 @@ void X3DSAX2Handlers::protoStartElement( const XMLCh* const uri,
   } else if( localname_string == "ProtoInterface" ) {
     if( !proto_declaration.get() || defining_proto_interface || 
         defining_proto_body != 0 || defining_extern_proto ) {
-      Console(3) << "Warning: ProtoInterface element can only be a child element of ProtoDeclare element "
+      Console(LogLevel::Warning) << "Warning: ProtoInterface element can only be a child element of ProtoDeclare element "
            << getLocationString() << endl;
       node_stack.push( NodeElement( NULL ) );
     } else {
@@ -864,17 +864,17 @@ void X3DSAX2Handlers::protoStartElement( const XMLCh* const uri,
     if( !proto_declaration.get() || defining_proto_interface || 
         defining_proto_body != 0 || defining_extern_proto ||
         proto_declaration->getProtoBody() != "" ) {
-      Console(3) << "Warning: ProtoBody element can only be a child element of ProtoDeclare element "
+      Console(LogLevel::Warning) << "Warning: ProtoBody element can only be a child element of ProtoDeclare element "
            << getLocationString() << endl;
       node_stack.push( NodeElement( NULL ) );
     } else {
       defining_proto_body = 1;
     }
   } else if( proto_declaration.get() && !defining_proto_interface && 
-             !defining_proto_body == 0 && !defining_extern_proto ) {
+             !defining_proto_body && !defining_extern_proto ) {
     // Element in ProtoDeclare proto declare that is not ProtoInterface 
     // or ProtoBody.
-    Console(3) << "Warning: Only ProtoBody and ProtoInterface elements allowed in ProtoDeclare element "
+    Console(LogLevel::Warning) << "Warning: Only ProtoBody and ProtoInterface elements allowed in ProtoDeclare element "
          << getLocationString() << endl;
     node_stack.push( NodeElement( NULL ) );
   } else {
@@ -894,7 +894,7 @@ void X3DSAX2Handlers::protoStartElement( const XMLCh* const uri,
         fd->value = fd->value + s.str();
         }
       } else {
-        Console(3) << "Warning: Only \"field\" elements allowed in ProtoInstance element "
+        Console(LogLevel::Warning) << "Warning: Only \"field\" elements allowed in ProtoInstance element "
              << getLocationString() << endl;
         node_stack.push( NodeElement( NULL ) );
       }
@@ -975,12 +975,12 @@ X3DPrototypeInstance* X3DSAX2Handlers::handleProtoInstanceElement( const Attribu
       proto_declarations->getProtoDeclaration( s_name );
     if( proto ) return proto->newProtoInstance();
     else {
-      Console(3) << "Warning: Could not find PROTO declaration for \"" << s_name  
+      Console(LogLevel::Warning) << "Warning: Could not find PROTO declaration for \"" << s_name  
            << "\" instance (" << toString( locator->getSystemId() )
            << ", line " << locator->getLineNumber() << ")" << endl;
     }
   } else {
-    Console(3) << "Warning: ProtoInstance element is missing required \"name\" attibute ("
+    Console(LogLevel::Warning) << "Warning: ProtoInstance element is missing required \"name\" attibute ("
          << toString( locator->getSystemId() )
 
 
@@ -993,18 +993,18 @@ X3DPrototypeInstance* X3DSAX2Handlers::handleProtoInstanceElement( const Attribu
 
 void X3DSAX2Handlers::handleExternProtoDeclareElement( const Attributes &attrs ) {
   if( proto_declaration.get() ) {
-    Console(3) << "Warning: ExternProtoDeclare element not allowed inside other ProtoDeclare element"
+    Console(LogLevel::Warning) << "Warning: ExternProtoDeclare element not allowed inside other ProtoDeclare element"
          << getLocationString() << endl;
     node_stack.push( NodeElement( NULL ) );
   } else {
     const XMLCh *name = attrs.getValue( gname );
     const XMLCh *url = attrs.getValue( gurl );
     if( !name ) {
-      Console(3) << "Warning: ExternProtoDeclare element is missing required \"name\" attibute "
+      Console(LogLevel::Warning) << "Warning: ExternProtoDeclare element is missing required \"name\" attibute "
            << getLocationString() << endl;
       node_stack.push( NodeElement( NULL ) );
     } else if( !url ) {
-      Console(3) << "Warning: ExternProtoDeclare element is missing required \"url\" attibute "
+      Console(LogLevel::Warning) << "Warning: ExternProtoDeclare element is missing required \"url\" attibute "
            << getLocationString() << endl;
       node_stack.push( NodeElement( NULL ) );
     } else {
@@ -1047,7 +1047,7 @@ void X3DSAX2Handlers::handleExternProtoDeclareElement( const Attributes &attrs )
         break;
       }
       if( url_used == "" ) {
-        Console(3) << "Warning: None of the urls in ExternProtoDeclare element could "
+        Console(LogLevel::Warning) << "Warning: None of the urls in ExternProtoDeclare element could "
              << "be sucessfully loaded. "<< getLocationString()<< endl;
         node_stack.push( NodeElement( NULL ) );
         return;
@@ -1058,14 +1058,14 @@ void X3DSAX2Handlers::handleExternProtoDeclareElement( const Attributes &attrs )
         // get the first prototype declaration from the url that was read.
         pd = proto_vector.getFirstProtoDeclaration();
         if( !pd ) {
-          Console(3) << "Warning: Could not find any PROTO declaration "
+          Console(LogLevel::Warning) << "Warning: Could not find any PROTO declaration "
                << getLocationString()<< endl;
           node_stack.push( NodeElement( NULL ) );
         }
       } else {
         pd = proto_vector.getProtoDeclaration( proto_name );
         if( !pd ) {
-          Console(3) << "Warning: Could not find PROTO declaration \"" 
+          Console(LogLevel::Warning) << "Warning: Could not find PROTO declaration \"" 
                << proto_name
                << "\" " << getLocationString()<< endl;
           node_stack.push( NodeElement( NULL ) );
@@ -1092,7 +1092,7 @@ void X3DSAX2Handlers::handleFieldValueElement( const Attributes &attrs,
   X3DPrototypeInstance *pi = dynamic_cast< X3DPrototypeInstance * >( parent );
 
   if( !pi ) {
-    Console(3) << "Warning: fieldValue element can only appear inside ProtoInstance "
+    Console(LogLevel::Warning) << "Warning: fieldValue element can only appear inside ProtoInstance "
          << "element (" << toString( locator->getSystemId() )
          << ", line " << locator->getLineNumber() << ")" << endl;
     return;
@@ -1104,7 +1104,7 @@ void X3DSAX2Handlers::handleFieldValueElement( const Attributes &attrs,
     name_value_missing = false;
     Field *field = pi->getField( toString( name ) );
     if( !field ) {
-      Console(3) << "Warning: Field named \"" << name 
+      Console(LogLevel::Warning) << "Warning: Field named \"" << name 
            << "\" does not exist in ProtoInstance node (" 
            << toString( locator->getSystemId() )
            << ", line " << locator->getLineNumber() << ")" << endl;
@@ -1113,7 +1113,7 @@ void X3DSAX2Handlers::handleFieldValueElement( const Attributes &attrs,
     ParsableField *pfield = 
       dynamic_cast< ParsableField * >( field );
     if( !pfield ) {
-      Console(3) << "Warning: Cannot parse field \"" << name 
+      Console(LogLevel::Warning) << "Warning: Cannot parse field \"" << name 
            << "\". Field type must be a subclass of ParsableField "
            << "in order to be parsable (" << toString( locator->getSystemId() )
            << ", line " << locator->getLineNumber() << ")" << endl;
@@ -1147,7 +1147,7 @@ void X3DSAX2Handlers::handleFieldValueElement( const Attributes &attrs,
   } else if( name && !value ) {
     Field *field = pi->getField( toString( name ) );
     if( !field ) {
-      Console(3) << "Warning: Field named \"" << name 
+      Console(LogLevel::Warning) << "Warning: Field named \"" << name 
            << "\" does not exist in ProtoInstance node (" 
            << toString( locator->getSystemId() )
            << ", line " << locator->getLineNumber() << ")" << endl;
@@ -1163,7 +1163,7 @@ void X3DSAX2Handlers::handleFieldValueElement( const Attributes &attrs,
   }
 
   if( name_value_missing ) {
-    Console(3) << "Warning: Both \"name\" and \"value\" attributes required "
+    Console(LogLevel::Warning) << "Warning: Both \"name\" and \"value\" attributes required "
                << "in fieldValue element"  << getLocationString() << endl;
     return;
   }
@@ -1172,10 +1172,10 @@ void X3DSAX2Handlers::handleFieldValueElement( const Attributes &attrs,
 void X3DSAX2Handlers::startCDATA() {
   inside_cdata = true;
   cdata = "";
-        if( proto_declaration.get() && !defining_proto_body ) {
-                Console(3) << "Warning: CDATA elements not allowed inside ProtoInterface."
-                                                         << getLocationString() << endl;
-        }
+  if( proto_declaration.get() && !defining_proto_body ) {
+    Console(LogLevel::Warning) << "Warning: CDATA elements not allowed inside ProtoInterface."
+                               << getLocationString() << endl;
+  }
 }
  
 
@@ -1195,7 +1195,7 @@ void X3DSAX2Handlers::endCDATA() {
 //                             const XMLSize_t length ) {
   
    ///  if( length > 0 )
-  //Console(4) << "Comment: " << toString( chars ) << endl;
+  //Console(LogLevel::Error) << "Comment: " << toString( chars ) << endl;
 //}
 
 void X3DSAX2Handlers::characters(const XMLCh *const chars, 
@@ -1299,8 +1299,8 @@ void X3DSAX2Handlers::startElement(const XMLCh* const uri,
                                 // in the X3D specification that stops this, since this is the case then IS
                                 // statements are also allowed in some cases.
                                 if( !proto_instance ) {
-          Console(3) << "Warning: IS elements only allowed inside ProtoBody elements"
-                                                                                 << getLocationString() << endl;
+          Console(LogLevel::Warning) << "Warning: IS elements only allowed inside ProtoBody elements"
+                                     << getLocationString() << endl;
           node_stack.push( NodeElement( NULL ) );
         } else {
           defining_proto_connections = true;
@@ -1308,9 +1308,9 @@ void X3DSAX2Handlers::startElement(const XMLCh* const uri,
                         } else if( localname_string == "connect" ) {
         handleConnectElement( attrs, parent );
                         } else {
-        Console(3) << "Warning: Only fieldValue ( and in some cases IS ) elements "
-                                                                         << "allowed in ProtoInstance element "
-                                                                         << getLocationString() << endl;
+        Console(LogLevel::Warning) << "Warning: Only fieldValue ( and in some cases IS ) elements "
+                                   << "allowed in ProtoInstance element "
+                                   << getLocationString() << endl;
         node_stack.push( NodeElement( NULL ) );
       }
     } else {
@@ -1321,16 +1321,16 @@ void X3DSAX2Handlers::startElement(const XMLCh* const uri,
         protoStartElement( uri, localname, qname, attrs );
         return;
       } else if( localname_string == "ProtoInterface" ) {
-        Console(3) << "Warning: ProtoInterface elements only allowed inside ProtoDeclare elements"
+        Console(LogLevel::Warning) << "Warning: ProtoInterface elements only allowed inside ProtoDeclare elements"
              << getLocationString() << endl;
         node_stack.push( NodeElement( NULL ) );
       } else if( localname_string == "ProtoBody" ) {
-        Console(3) << "Warning: ProtoBody elements only allowed inside ProtoDeclare elements"
+        Console(LogLevel::Warning) << "Warning: ProtoBody elements only allowed inside ProtoDeclare elements"
              << getLocationString() << endl;
         node_stack.push( NodeElement( NULL ) );
       } else if( localname_string == "IS" )  {
         if( !proto_instance ) {
-          Console(3) << "Warning: IS elements only allowed inside ProtoBody elements"
+          Console(LogLevel::Warning) << "Warning: IS elements only allowed inside ProtoBody elements"
                << getLocationString() << endl;
           node_stack.push( NodeElement( NULL ) );
         } else {
@@ -1407,7 +1407,7 @@ void X3DSAX2Handlers::startElement(const XMLCh* const uri,
       } else {
         const XMLCh *use_name = attrs.getValue( gUSE );
         const XMLCh *copy_value = attrs.getValue( gCOPY );
-        bool proto_instance = false;
+        bool _proto_instance = false;
 
         if( use_name ) {
           // if we have a USE attribute, lookup the matching node and use that.
@@ -1451,13 +1451,13 @@ void X3DSAX2Handlers::startElement(const XMLCh* const uri,
             // since we don't want it to occur until all child nodes
             // have been created and set.
             if( new_node ) new_node->setManualInitialize( true );
-            proto_instance = true;
+            _proto_instance = true;
           } else {
             try {
               new_node = 
                 H3DNodeDatabase::createNode( localname_string );
               if( !new_node ) {
-                Console(3) << "Warning: Could not create \"" << localname 
+                Console(LogLevel::Warning) << "Warning: Could not create \"" << localname 
                      << "\" node. It does not exist in the H3DNodeDatabase " 
                      << getLocationString() << endl;
               } else {
@@ -1478,7 +1478,7 @@ void X3DSAX2Handlers::startElement(const XMLCh* const uri,
                 } 
               }
             } catch (  const H3D::Exception::H3DException &e ) {
-              Console(3) << "Warning: Could not create \"" << localname 
+              Console(LogLevel::Warning) << "Warning: Could not create \"" << localname 
                    << "\" node. Exception in constructor: "
                    << e << " " << getLocationString() << endl;
             }
@@ -1511,20 +1511,20 @@ void X3DSAX2Handlers::startElement(const XMLCh* const uri,
               if( dynamic_cast< PythonScript * >(parent) &&
                   container_field == "references" &&
                   node_stack.top().haveConnectElement() ) {
-                Console(3) << "Warning: When using a PythonScript in a Protobody "
+                Console(LogLevel::Warning) << "Warning: When using a PythonScript in a Protobody "
                            << "all nodes specified for the references field of PythonScript "
                            << "have to be declared before the connect elements. "
                            << getLocationString() << endl;
               }
 #endif
-            } else if( proto_instance ) {
+            } else if( _proto_instance ) {
             } else if( use_name ) {
               // node is defined by USE, so we ignore all fields that are not
               // USE, class, DEF, or containerField
             } else {
               Field *field = new_node->getField( name );
               if( !field ) {
-                Console(3) << "Warning: Couldn't find field named \"" << name 
+                Console(LogLevel::Warning) << "Warning: Couldn't find field named \"" << name 
                            << "\" in " << qname << " node " 
                            << getLocationString() << endl;
               } else {
@@ -1690,12 +1690,12 @@ void X3DSAX2Handlers::endElement (const XMLCh *const uri,
             if( mf ) {
               mf->push_back( new_node );
             }
-            else Console(3) << "Warning: 'value' element ignored. Only used if "
+            else Console(LogLevel::Warning) << "Warning: 'value' element ignored. Only used if "
                       << "field type is SFNode or MFNode" 
                       << getLocationString() << endl;
           }
         } else {
-          Console(3) << "Warning: 'value' element ignored. Only used if "
+          Console(LogLevel::Warning) << "Warning: 'value' element ignored. Only used if "
                << "accesstype is initializeOnly or inputOutput " 
                << getLocationString() << endl;
         }
@@ -1760,7 +1760,7 @@ void X3DSAX2Handlers::fatalError(const SAXParseException& e) {
 }
 
 void X3DSAX2Handlers::warning(const SAXParseException& e) {
-  Console(3) << "Warning: " << e.getMessage() << " (" << e.getSystemId()
+  Console(LogLevel::Warning) << "Warning: " << e.getMessage() << " (" << e.getSystemId()
              << ", line " << e.getLineNumber()
              << ", char " << e.getColumnNumber() << ")" << endl;
 }
