@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -30,6 +30,7 @@
 
 #include <H3D/X3DTexture2DNode.h>
 #include <H3DUtil/Image.h>
+#include <H3D/GlobalSettings.h>
 
 using namespace H3D;
 
@@ -41,10 +42,10 @@ H3DNodeDatabase X3DTexture2DNode::database(
         );
 
 namespace X3DTexture2DNodeInternals {
-  FIELDDB_ELEMENT( X3DTexture2DNode, repeatS, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( X3DTexture2DNode, repeatT, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( X3DTexture2DNode, scaleToPowerOfTwo, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( X3DTexture2DNode, textureProperties, INPUT_OUTPUT );
+  FIELDDB_ELEMENT( X3DTexture2DNode, repeatS, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( X3DTexture2DNode, repeatT, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( X3DTexture2DNode, scaleToPowerOfTwo, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( X3DTexture2DNode, textureProperties, INPUT_OUTPUT )
 }
 
 X3DTexture2DNode::X3DTexture2DNode( 
@@ -61,12 +62,13 @@ X3DTexture2DNode::X3DTexture2DNode(
   repeatT ( _repeatT  ),
   scaleToPowerOfTwo( _scaleToP2 ),
   textureProperties( _textureProperties ),
-  texture_id( 0 ),
-  texture_unit( GL_TEXTURE0_ARB ),
-  texture_target( 0 ),
-  imageNeedsUpdate ( new Field ){
+  imageUpdated ( new Field ),
+  updateTextureProperties( new UpdateTextureProperties ),
+  textureUpdated( new Field ) {
 
   type_name = "X3DTexture2DNode";
+  texture_unit = GL_TEXTURE0_ARB ;
+  texture_target = GL_TEXTURE_2D ;
 
   database.initFields( this );
   image->setName( "image" );
@@ -76,15 +78,23 @@ X3DTexture2DNode::X3DTexture2DNode(
   repeatT->setValue( true );
   scaleToPowerOfTwo->setValue( true );
 
+  textureUpdated->setOwner( this );
+  textureUpdated->setName( "textureUpdated" );
+
   image->route( displayList );
   repeatS->route( displayList );
   repeatT->route( displayList );
   scaleToPowerOfTwo->route( displayList );
   textureProperties->route( displayList );
+  textureUpdated->route( displayList );
 
-  imageNeedsUpdate->setName( "ImageNeedsUpdate" );
-  imageNeedsUpdate->setOwner( this );
-  image->route( imageNeedsUpdate );
+  imageUpdated->setName( "imageUpdated" );
+  imageUpdated->setOwner( this );
+  image->route( imageUpdated );
+
+  updateTextureProperties->setName( "updateTextureProperties" );
+  updateTextureProperties->setOwner(this);
+  textureProperties->route(updateTextureProperties);
 }
 
 void X3DTexture2DNode::SFImage::setValueFromString( const string& s ) {
@@ -123,29 +133,24 @@ string X3DTexture2DNode::SFImage::getValueAsString( const string& separator) {
 
 GLint X3DTexture2DNode::glInternalFormat( Image *i ) {
   TextureProperties *texture_properties = textureProperties->getValue();
-  if( GLEW_ARB_texture_compression && 
-      texture_properties &&
-      texture_properties->textureCompression->getValue() != "DEFAULT" ) {
-    switch( i->pixelType() ) {
-    case Image::LUMINANCE: 
-      return GL_COMPRESSED_LUMINANCE_ARB;
-    case Image::LUMINANCE_ALPHA: 
-      return GL_COMPRESSED_LUMINANCE_ALPHA_ARB;
-    case Image::RGB:
-    case Image::BGR:
-      return GL_COMPRESSED_RGB_ARB;
-    case Image::RGBA:
-    case Image::BGRA:
-      return GL_COMPRESSED_RGBA_ARB;
-    default:
-      return GL_RGBA;
+  GLint tex_format;
+  if( texture_properties ) {
+    if( !texture_properties->glInternalFormat( i, tex_format ) ) {
+      tex_format = X3DTextureNode::glInternalFormat( i );
     }
   } else {
-    return X3DTextureNode::glInternalFormat( i );
+    tex_format = X3DTextureNode::glInternalFormat( i );
   }
+
+  // Choose compression
+  std::string compression = "DEFAULT";
+  if( texture_properties ) {
+    compression = texture_properties->textureCompression->getValue();
+  }
+  return glCompressedInternalFormat( tex_format, compression );
 }
 
-void X3DTexture2DNode::glTexImage( Image *i, GLenum texture_target,
+void X3DTexture2DNode::glTexImage( Image *i, GLenum _texture_target,
                                    bool scale_to_power_of_two ) {
   // the image data to render
   void *image_data = i->getImageData();
@@ -161,18 +166,41 @@ void X3DTexture2DNode::glTexImage( Image *i, GLenum texture_target,
   glGetIntegerv( GL_UNPACK_ALIGNMENT, &byte_alignment );
   glPixelStorei( GL_UNPACK_ALIGNMENT, i->byteAlignment() );
 
-  if( scale_to_power_of_two && !GLEW_ARB_texture_non_power_of_two ) {
+  // Get global max texture dimension
+  H3DInt32 max_dimension= -1;
+  GlobalSettings*  gs = GlobalSettings::getActive();
+  if( gs ) {
+    GraphicsOptions* go = NULL;
+    gs->getOptionNode( go );
+    if ( go ) {
+      max_dimension= go->maxTextureDimension->getValue();
+    }
+  }
+
+  if( (scale_to_power_of_two && !GLEW_ARB_texture_non_power_of_two) || max_dimension > 0 ) {
     // check if any scaling is required and if so scale the image.
     bool needs_scaling = false;
+    bool needs_scale_to_power_of_two = (scale_to_power_of_two && !GLEW_ARB_texture_non_power_of_two);
+
     unsigned int new_width  = i->width();
     unsigned int new_height = i->height(); 
 
-    if( !isPowerOfTwo( new_width ) ) {
+    // Enforce global maximum texture dimension
+    if ( max_dimension > 0 && new_width > (unsigned int)max_dimension ) {
+      new_width= (unsigned int)max_dimension;
+      needs_scaling= true;
+    }
+    if ( max_dimension > 0 && new_height > (unsigned int)max_dimension ) {
+      new_height= (unsigned int)max_dimension;
+      needs_scaling= true;
+    }
+    
+    if( needs_scale_to_power_of_two && !isPowerOfTwo( new_width ) ) {
       new_width = nextPowerOfTwo( new_width );
       needs_scaling = true;
     } 
 
-    if( !isPowerOfTwo( new_height ) ) {
+    if( needs_scale_to_power_of_two && !isPowerOfTwo( new_height ) ) {
       new_height = nextPowerOfTwo( new_height );
       needs_scaling = true;
     } 
@@ -224,7 +252,7 @@ void X3DTexture2DNode::glTexImage( Image *i, GLenum texture_target,
   }
 
   if( texture_properties && texture_properties->generateMipMaps->getValue() ) {
-    gluBuild2DMipmaps(  texture_target, 
+    gluBuild2DMipmaps(  _texture_target, 
                         glInternalFormat( i ),
                         width,
                         height,
@@ -237,22 +265,33 @@ void X3DTexture2DNode::glTexImage( Image *i, GLenum texture_target,
     texture_properties->borderWidth->getValue() : 0;
 
     if( border_width < 0 || border_width > 1 ) {
-      Console(3) << "Warning: Invalid borderWidth \"" << border_width 
+      Console(LogLevel::Warning) << "Warning: Invalid borderWidth \"" << border_width 
                  << "\". Must be 0 or 1 (in " << getName()
                    << ")" << endl;
       border_width = 0;
     }
 
-    // install the image as a 2d texture/
-    glTexImage2D( texture_target, 
-                  0, // mipmap level
-                  glInternalFormat( i ),
-                  width,
-                  height,
-                  border_width, // border
-                  glPixelFormat( i ),
-                  glPixelComponentType( i ),
-                  image_data );
+    // install the image as a 2d texture
+    if( i->compressionType() == Image::NO_COMPRESSION ) {
+      glTexImage2D( _texture_target, 
+        0, // mipmap level
+        glInternalFormat( i ),
+        width,
+        height,
+        border_width, // border
+        glPixelFormat( i ),
+        glPixelComponentType( i ),
+        image_data );
+    } else {
+      glCompressedTexImage2D( _texture_target,
+        0, // mipmap level
+        glInternalFormat( i ),
+        width,
+        height,
+        border_width, // border
+        width*height*i->bitsPerPixel() / 8,
+        image_data );
+    }
   }
 
   glPixelStorei( GL_UNPACK_ALIGNMENT, byte_alignment );
@@ -266,20 +305,22 @@ void X3DTexture2DNode::glTexImage( Image *i, GLenum texture_target,
 }
 
 void X3DTexture2DNode::render()     {
+#ifndef USE_APPLICATION_MANAGED_ACTIVE_TEXTURE_UNIT
   glGetIntegerv( GL_ACTIVE_TEXTURE_ARB, &texture_unit );
-
-  GLenum target = getTextureTarget();
-
+#endif
+  GLenum texture_target_prev = texture_target;
+  updateTextureProperties->upToDate();
+  bool texture_target_changed = (texture_target_prev != texture_target);
   Image * i = static_cast< Image * >(image->getValue());
-  if( displayList->hasCausedEvent( image ) || texture_target != target ) {
-    
-    if( !image->imageChanged() || texture_id == 0 || texture_target != target ) {
+  if( !imageUpdated->isUpToDate()||texture_target_changed ){
+    if( !image->imageChanged() || texture_id == 0|| texture_target_changed) {
       // the image has changed so remove the old texture and install 
       // the new
       glDeleteTextures( 1, &texture_id );
       texture_id = 0;
-      texture_target = target;
       if( i ) {
+        texture_width = i->width();
+        texture_height = i->height();
         texture_id = renderImage( i, 
                                   texture_target, 
                                   scaleToPowerOfTwo->getValue() );
@@ -304,50 +345,52 @@ void X3DTexture2DNode::render()     {
   if( i ) {
     renderTextureProperties();
   }
-  imageNeedsUpdate->upToDate();
+  imageUpdated->upToDate();
 }
 
 void X3DTexture2DNode::renderTextureProperties() {
+  // If the texture is resident (see bindless textures), then we can no longer change the texture properties
+  // and attempting to do so will result in an error state
+  if ( getTextureHandle() != 0 ) return;
+
   TextureProperties *texture_properties = textureProperties->getValue();
   if( texture_properties ) {
     texture_properties->renderTextureProperties( texture_target );
   } else {
     // set up texture parameters 
-    if ( repeatS->getValue() )
+    if( !repeatS->getValue() ) {
+      glTexParameteri( texture_target, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE );
+    } else {
       glTexParameteri( texture_target, GL_TEXTURE_WRAP_S, GL_REPEAT );
-    else
-      glTexParameteri( texture_target, 
-                       GL_TEXTURE_WRAP_S, 
-                       GL_CLAMP_TO_EDGE );
-    if ( repeatT->getValue() )
+    }
+    if( !repeatT->getValue() ) {
+      glTexParameteri( texture_target, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE );
+    } else {
       glTexParameteri( texture_target, GL_TEXTURE_WRAP_T, GL_REPEAT );
-    else
-      glTexParameteri( texture_target, GL_TEXTURE_WRAP_T, 
-                       GL_CLAMP_TO_EDGE );
+    }
     glTexParameteri( texture_target, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
     glTexParameteri( texture_target, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
   }
 }
 
-void X3DTexture2DNode::renderSubImage( Image *image, GLenum texture_target, 
+void X3DTexture2DNode::renderSubImage( Image *_image, GLenum _texture_target, 
                                        int x_offset, int y_offset,
                                        int width, int height ) {
-  // todo: is there a way to do this without copying data?
-  unsigned char *image_data = (unsigned char *) image->getImageData();
+  unsigned char *image_data = (unsigned char *) _image->getImageData();
 
-  unsigned int bytes_per_pixel = image->bitsPerPixel() / 8;
+  unsigned int bytes_per_pixel = _image->bitsPerPixel() / 8;
 
   unsigned char *modified_data = 
     new unsigned char[ width * height * bytes_per_pixel ]; 
   for( unsigned int i = 0; i < (unsigned int)height; ++i ) {
     memcpy( modified_data + i * width * bytes_per_pixel, 
-            image_data + ( (i + y_offset )* image->width() + x_offset) * bytes_per_pixel,
+            image_data + ( (i + y_offset )* _image->width() + x_offset) * bytes_per_pixel,
             width * bytes_per_pixel );
   }
 
  GLint byte_alignment;
   glGetIntegerv( GL_UNPACK_ALIGNMENT, &byte_alignment );
-  glPixelStorei( GL_UNPACK_ALIGNMENT, image->byteAlignment() );
+  glPixelStorei( GL_UNPACK_ALIGNMENT, _image->byteAlignment() );
 
   TextureProperties *texture_properties = textureProperties->getValue();
   if( texture_properties ) {
@@ -365,11 +408,11 @@ void X3DTexture2DNode::renderSubImage( Image *image, GLenum texture_target,
     glPixelTransferf( GL_ALPHA_BIAS, bias.w );
   }
 
-  glTexSubImage2D( texture_target, 0, 
+  glTexSubImage2D( _texture_target, 0, 
                    x_offset, y_offset, 
                    width, height, 
-                   glPixelFormat( image ),
-                   glPixelComponentType( image ), 
+                   glPixelFormat( _image ),
+                   glPixelComponentType( _image ), 
                    modified_data );
 
   if( texture_properties ) glPopAttrib();
@@ -382,7 +425,9 @@ void X3DTexture2DNode::enableTexturing() {
   glEnable( texture_target );
   Image * i = static_cast< Image * >(image->getValue());
   if( i ) {
-    // update blend state when image exist and image needs update
+    // need to always update blending state no matter if image are updated or not
+    // because there is no opengl state tracking in H3D.  blend state can be 
+    // modified at any place 
     Image::PixelType pixel_type = i->pixelType();
     if( pixel_type == Image::LUMINANCE_ALPHA ||
       pixel_type == Image::RGBA || 
@@ -407,22 +452,16 @@ void X3DTexture2DNode::disableTexturing() {
 }
 
 
-GLenum X3DTexture2DNode::getTextureTarget() {
-  TextureProperties *texture_properties = textureProperties->getValue();
-  if( texture_properties ) {
-    const string &target_type = texture_properties->textureType->getValue();
-    if( target_type == "RECTANGLE" ) return GL_TEXTURE_RECTANGLE_ARB;
-    else if( target_type == "2DARRAY" ) {
-      Console(3) << "Warning: Invalid textureType \"2DARRAY\" in TextureProperties for "
-     << "X3DTexture2DNode. \"2DARRAY\" can only be used for 3D textures" << endl;
-    } else if( target_type != "NORMAL" ) {
-      Console(3) << "Warning: Invalid textureType: \"" << target_type << "\" in TextureProperties for "
-     << "X3DTexture2DNode. " << endl;
-    }
-  }
-  return GL_TEXTURE_2D;
+GLuint64 X3DTexture2DNode::getTextureHandle() {
+  // When the image is used as a bindless texture in a shader
+  // then it is possible that the image is never rendered. So we take
+  // this opportunity to ensure that the image is upToDate since otherwise
+  // changes to the image will not be handled.
+  image->upToDate();
+  return H3DSingleTextureNode::getTextureHandle();
 }
 
+
 std::pair<H3DInt32,H3DInt32> X3DTexture2DNode::getDefaultSaveDimensions () {
   Image* im= image->getValue();
   if ( im ) {
@@ -431,3 +470,83 @@ std::pair<H3DInt32,H3DInt32> X3DTexture2DNode::getDefaultSaveDimensions () {
     return X3DTextureNode::getDefaultSaveDimensions();
   }
 }
+
+GLenum X3DTexture2DNode::glPixelFormat( Image *i ) {
+  TextureProperties *texture_properties = textureProperties->getValue();
+  GLenum tex_prop_format;
+  if( texture_properties && texture_properties->glPixelFormat( i, tex_prop_format ) ) {
+    return tex_prop_format;
+  } else {
+    return X3DTextureNode::glPixelFormat( i );
+  }
+}
+
+Image* X3DTexture2DNode::renderToImage( H3DInt32 _width, H3DInt32 _height, bool output_float_texture /* = false */ ){
+  // for the moment glGetTexImage based solution does not works well for depth texture, as there is no 
+  // proper way for the moment to actually get the pixel format of a generatedTexture when it contains depth texture.
+  // so that solution is by passed for now.
+  return X3DTextureNode::renderToImage( _width, _height, output_float_texture );
+  GLuint t_id = getTextureId();
+  if( glIsTexture(t_id) ) {
+    int bpp;
+
+    // Note: Resizing of image is not supported
+    std::pair < H3DInt32, H3DInt32 > size= getDefaultSaveDimensions ();
+    if ( _width != size.first || _height != size.second ) {
+      _width= size.first;
+      _height= size.second;
+    }
+
+    // Create container for image data, then bind buffer and read from it.
+    Image* _image;
+    if( output_float_texture ) {
+      bpp = sizeof(float)*8*4;
+      _image= new PixelImage ( _width, _height, 1, bpp, Image::RGBA, Image::RATIONAL );
+    }else{
+      bpp = 32;
+      _image= new PixelImage ( _width, _height, 1, bpp, Image::BGRA, Image::UNSIGNED );
+    }
+    GLint active_texture_bind = 0;
+    glGetIntegerv( GL_TEXTURE_BINDING_2D, &active_texture_bind );
+    glBindTexture( texture_target, t_id );
+    glGetTexImage( texture_target, 0, glPixelFormat( _image ), glPixelComponentType(_image), _image->getImageData() );
+    glBindTexture( texture_target, active_texture_bind );
+    return _image;
+  }
+
+  Console(LogLevel::Error) << "ERROR: Texture ID: "<< t_id << " is not valid. Can not save texture!"<<endl;
+  return NULL;
+}
+
+void X3DTexture2DNode::UpdateTextureProperties::update(){
+  X3DTexture2DNode* t = static_cast<X3DTexture2DNode*>( getOwner() );
+  TextureProperties *texture_properties = t->textureProperties->getValue();
+  string target_type;
+  if( texture_properties ) {
+    target_type = texture_properties->textureType->getValue();
+    if( target_type == "RECTANGLE" )  t->setTextureTarget(GL_TEXTURE_RECTANGLE_ARB);
+    else if( target_type == "2DARRAY" ) {
+      Console(LogLevel::Warning) << "Warning: Invalid textureType \"2DARRAY\" in TextureProperties for "
+        << "X3DTexture2DNode. \"2DARRAY\" can only be used for 3D textures" << endl;
+    } else if( target_type != "NORMAL" ) {
+      Console(LogLevel::Warning) << "Warning: Invalid textureType: \"" << target_type << "\" in TextureProperties for "
+        << "X3DTexture2DNode. " << endl;
+    }
+  }
+}
+
+void X3DTexture2DNode::setTextureWidth( int _width ) {
+  int current_width = this->getTextureWidth();
+  if( current_width!=_width ) {
+    H3DSingleTextureNode::setTextureWidth( _width );
+    this->textureUpdated->touch();
+  }
+}
+
+void X3DTexture2DNode::setTextureHeight( int _height ) {
+  int current_hight = getTextureHeight();
+  if( current_hight!=_height ) {
+    H3DSingleTextureNode::setTextureHeight( _height );
+    this->textureUpdated->touch();
+  }
+}
\ No newline at end of file
