@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -30,6 +30,9 @@
 
 #include <H3D/X3DTexture3DNode.h>
 #include <H3DUtil/Image.h>
+#include <H3D/GlobalSettings.h>
+
+#include <H3DUtil/LoadImageFunctions.h>
 
 using namespace H3D;
 
@@ -41,11 +44,11 @@ H3DNodeDatabase X3DTexture3DNode::database(
         );
 
 namespace X3DTexture3DNodeInternals {
-  FIELDDB_ELEMENT( X3DTexture3DNode, repeatS, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( X3DTexture3DNode, repeatT, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( X3DTexture3DNode, repeatR, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( X3DTexture3DNode, scaleToPowerOfTwo, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( X3DTexture3DNode, textureProperties, INPUT_OUTPUT );
+  FIELDDB_ELEMENT( X3DTexture3DNode, repeatS, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( X3DTexture3DNode, repeatT, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( X3DTexture3DNode, repeatR, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( X3DTexture3DNode, scaleToPowerOfTwo, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( X3DTexture3DNode, textureProperties, INPUT_OUTPUT )
 }
 
 X3DTexture3DNode::X3DTexture3DNode( 
@@ -56,20 +59,26 @@ X3DTexture3DNode::X3DTexture3DNode(
                                    Inst< SFBool  > _repeatR,
                                    Inst< SFBool  > _scaleToP2,
                                    Inst< SFImage > _image,
-                                   Inst< SFTextureProperties > _textureProp ) :
-  H3DSingleTextureNode( _displayList, _metadata ),
+                                   Inst< SFTextureProperties > _textureProp,
+                                   Inst< UpdateSaveToURL > _saveToUrl,
+                                   Inst< SFBool > _saveSuccess,
+                                   Inst< SFInt32 > _saveHeight,
+                                   Inst< SFInt32 > _saveWidth ) :
+  H3DSingleTextureNode( _displayList, _metadata, _saveToUrl, _saveSuccess, _saveHeight, _saveWidth ),
   H3DImageObject( _image ),
   repeatS ( _repeatS  ),
   repeatT ( _repeatT  ),
   repeatR ( _repeatR  ),
   scaleToPowerOfTwo( _scaleToP2 ),
   textureProperties( _textureProp ),
-  texture_id( 0 ),
-  texture_unit( GL_TEXTURE0_ARB ),
-  texture_target( 0 ),
-  imageNeedsUpdate( new Field ){
+  imageUpdated( new Field ),
+  updateTextureProperties( new UpdateTextureProperties ),
+  textureUpdated( new Field ){
 
   type_name = "X3DTexture3DNode";
+  
+  texture_unit = GL_TEXTURE0_ARB ;
+  texture_target = GL_TEXTURE_3D ;
 
   database.initFields( this );
   image->setName( "image" );
@@ -80,16 +89,24 @@ X3DTexture3DNode::X3DTexture3DNode(
   repeatR->setValue( false );
   scaleToPowerOfTwo->setValue( true );
 
+  textureUpdated->setOwner( this );
+  textureUpdated->setName( "textureUpdated" );
+
   image->route( displayList );
   repeatS->route( displayList );
   repeatT->route( displayList );
   repeatR->route( displayList );
   scaleToPowerOfTwo->route( displayList );
   textureProperties->route( displayList );
+  textureUpdated->route( displayList );
 
-  imageNeedsUpdate->setName( "ImageNeedsUpdate" );
-  imageNeedsUpdate->setOwner( this );
-  image->route( imageNeedsUpdate );
+  imageUpdated->setName( "imageUpdated" );
+  imageUpdated->setOwner( this );
+  image->route( imageUpdated );
+
+  updateTextureProperties->setName( "updateTextureProperties" );
+  updateTextureProperties->setOwner(this);
+  textureProperties->route(updateTextureProperties);
 }
 
 string X3DTexture3DNode::SFImage::getValueAsString( const string& separator) {
@@ -126,29 +143,24 @@ string X3DTexture3DNode::SFImage::getValueAsString( const string& separator) {
 
 GLint X3DTexture3DNode::glInternalFormat( Image *i ) {
   TextureProperties *texture_properties = textureProperties->getValue();
-  if( GLEW_ARB_texture_compression && 
-      texture_properties &&
-      texture_properties->textureCompression->getValue() != "DEFAULT" ) {
-    switch( i->pixelType() ) {
-    case Image::LUMINANCE: 
-      return GL_COMPRESSED_LUMINANCE_ARB;
-    case Image::LUMINANCE_ALPHA: 
-      return GL_COMPRESSED_LUMINANCE_ALPHA_ARB;
-    case Image::RGB:
-    case Image::BGR:
-      return GL_COMPRESSED_RGB_ARB;
-    case Image::RGBA:
-    case Image::BGRA:
-      return GL_COMPRESSED_RGBA_ARB;
-    default:
-      return GL_RGBA;
+  GLint tex_format;
+  if( texture_properties ) {
+    if( !texture_properties->glInternalFormat( i, tex_format ) ) {
+      tex_format = H3DSingleTextureNode::glInternalFormat( i );
     }
   } else {
-    return H3DSingleTextureNode::glInternalFormat( i );
+    tex_format = H3DSingleTextureNode::glInternalFormat( i );
   }
+
+  // Choose compression
+  std::string compression = "DEFAULT";
+  if( texture_properties ) {
+    compression = texture_properties->textureCompression->getValue();
+  }
+  return glCompressedInternalFormat( tex_format, compression );
 }
 
-void X3DTexture3DNode::glTexImage( Image *i, GLenum texture_target,
+void X3DTexture3DNode::glTexImage( Image *i, GLenum _texture_target,
                                    bool scale_to_power_of_two ) {
   // the image data to render
   void *image_data = i->getImageData();
@@ -162,22 +174,48 @@ void X3DTexture3DNode::glTexImage( Image *i, GLenum texture_target,
   // at the end of the function or not.
   bool free_image_data = false;
 
-  if( scale_to_power_of_two && !GLEW_ARB_texture_non_power_of_two ) {
+  // Get global max texture dimension
+  H3DInt32 max_dimension= -1;
+  GlobalSettings*  gs = GlobalSettings::getActive();
+  if( gs ) {
+    GraphicsOptions* go = NULL;
+    gs->getOptionNode( go );
+    if ( go ) {
+      max_dimension= go->maxTextureDimension->getValue();
+    }
+  }
+
+  if( (scale_to_power_of_two && !GLEW_ARB_texture_non_power_of_two) || max_dimension > 0 ) {
     bool needs_scaling = false;
+    bool needs_scale_to_power_of_two = (scale_to_power_of_two && !GLEW_ARB_texture_non_power_of_two);
     unsigned int new_width  = i->width();
     unsigned int new_height = i->height(); 
     unsigned int new_depth = i->depth(); 
 
-    if( !isPowerOfTwo( new_width ) ) {
+    // Enforce global maximum texture dimension
+    if ( max_dimension > 0 && new_width > (unsigned int)max_dimension ) {
+      new_width= (unsigned int)max_dimension;
+      needs_scaling= true;
+    }
+    if ( max_dimension > 0 && new_height > (unsigned int)max_dimension ) {
+      new_height= (unsigned int)max_dimension;
+      needs_scaling= true;
+    }
+    if ( max_dimension > 0 && new_depth > (unsigned int)max_dimension ) {
+      new_depth= (unsigned int)max_dimension;
+      needs_scaling= true;
+    }
+
+    if( needs_scale_to_power_of_two&&!isPowerOfTwo( new_width ) ) {
       new_width = nextPowerOfTwo( new_width );
       needs_scaling = true;
     } 
-    if( !isPowerOfTwo( new_height ) ) {
+    if( needs_scale_to_power_of_two&&!isPowerOfTwo( new_height ) ) {
       new_height = nextPowerOfTwo( new_height );
       needs_scaling = true;
     } 
 
-    if( !isPowerOfTwo( new_depth ) ) {
+    if( needs_scale_to_power_of_two&&!isPowerOfTwo( new_depth ) ) {
       new_depth = nextPowerOfTwo( new_depth );
       needs_scaling = true;
     } 
@@ -214,7 +252,7 @@ void X3DTexture3DNode::glTexImage( Image *i, GLenum texture_target,
 
   if( texture_properties && texture_properties->generateMipMaps->getValue() ) {
 #if( GLU_VERSION_1_3 )
-    gluBuild3DMipmaps(  texture_target, 
+    gluBuild3DMipmaps(  _texture_target, 
                         glInternalFormat( i ),
                         width,
                         height,
@@ -229,23 +267,35 @@ void X3DTexture3DNode::glTexImage( Image *i, GLenum texture_target,
       texture_properties->borderWidth->getValue() : 0;
     
     if( border_width < 0 || border_width > 1 ) {
-      Console(3) << "Warning: Invalid borderWidth \"" << border_width 
+      Console(LogLevel::Warning) << "Warning: Invalid borderWidth \"" << border_width 
                  << "\". Must be 0 or 1 (in " << getName()
                  << ")" << endl;
       border_width = 0;
     }
     
     // install the image as a 3d texture/
-    glTexImage3D( texture_target, 
-                  0, // mipmap level
-                  glInternalFormat( i ),
-                  width,
-                  height,
-                  depth,
-                  border_width, // border
-                  glPixelFormat( i ),
-                  glPixelComponentType( i ),
-                  image_data );
+    if( i->compressionType() == Image::NO_COMPRESSION ) {
+      glTexImage3D( _texture_target, 
+                    0, // mipmap level
+                    glInternalFormat( i ),
+                    width,
+                    height,
+                    depth,
+                    border_width, // border
+                    glPixelFormat( i ),
+                    glPixelComponentType( i ),
+                    image_data );
+    } else {
+      glCompressedTexImage3D( _texture_target,
+        0, // mipmap level
+        glInternalFormat( i ),
+        width,
+        height,
+        depth,
+        border_width, // border
+        width*height*i->bitsPerPixel() / 8,
+        image_data );
+    }
   }
 
   glPixelStorei( GL_UNPACK_ALIGNMENT, byte_alignment );
@@ -258,20 +308,23 @@ void X3DTexture3DNode::glTexImage( Image *i, GLenum texture_target,
 }
 
 void X3DTexture3DNode::render()     {
+#ifndef USE_APPLICATION_MANAGED_ACTIVE_TEXTURE_UNIT
   glGetIntegerv( GL_ACTIVE_TEXTURE_ARB, &texture_unit );
-
-  GLenum target = getTextureTarget();
-
+#endif  
+  GLenum texture_target_prev = texture_target;
+  updateTextureProperties->upToDate();
+  bool texture_target_changed = (texture_target_prev != texture_target);
   Image * i = static_cast< Image * >(image->getValue());
-  if( displayList->hasCausedEvent( image ) || texture_target != target ) {
-
-    if( !image->imageChanged() || texture_id == 0 || texture_target != target ) {
+  if( !imageUpdated->isUpToDate() || texture_target_changed ){
+    if( !image->imageChanged() || texture_id == 0|| texture_target_changed) {
       // the image has changed so remove the old texture and install 
       // the new
       glDeleteTextures( 1, &texture_id );
       texture_id = 0;
-      texture_target = target;
       if( i ) {
+        texture_width = i->width();
+        texture_height = i->height();
+        texture_depth = i->depth();
         texture_id = renderImage( i, 
                                   texture_target, 
                                   scaleToPowerOfTwo->getValue() );
@@ -298,48 +351,50 @@ void X3DTexture3DNode::render()     {
   if( i ) {
     renderTextureProperties();
   }
-  imageNeedsUpdate->upToDate();
+  imageUpdated->upToDate();
 }
 
 void X3DTexture3DNode::renderTextureProperties() {
+  // If the texture is resident (see bindless textures), then we can no longer change the texture properties
+  // and attempting to do so will result in an error state
+  if ( getTextureHandle() != 0 ) return;
+
   TextureProperties *texture_properties = textureProperties->getValue();
   if( texture_properties ) {
     texture_properties->renderTextureProperties( texture_target );
   } else {
     // set up texture parameters 
-    if ( repeatS->getValue() )
+    if ( !repeatS->getValue() ) {
+      glTexParameteri( texture_target, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE );
+    } else {
       glTexParameteri( texture_target, GL_TEXTURE_WRAP_S, GL_REPEAT );
-    else
-      glTexParameteri( texture_target, 
-                       GL_TEXTURE_WRAP_S, 
-                       GL_CLAMP_TO_EDGE );
-    if ( repeatT->getValue() )
+    }
+    if ( !repeatT->getValue() ) {
+      glTexParameteri( texture_target, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE );
+    } else {
       glTexParameteri( texture_target, GL_TEXTURE_WRAP_T, GL_REPEAT );
-    else
-      glTexParameteri( texture_target, GL_TEXTURE_WRAP_T, 
-                       GL_CLAMP_TO_EDGE );
-    if ( repeatR->getValue() )
+    }
+    if( !repeatR->getValue() ) {
+      glTexParameteri( texture_target, GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE );
+    } else {
       glTexParameteri( texture_target, GL_TEXTURE_WRAP_R, GL_REPEAT );
-    else
-      glTexParameteri( texture_target, GL_TEXTURE_WRAP_R, 
-                       GL_CLAMP_TO_EDGE );
+    }
     glTexParameteri( texture_target, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
     glTexParameteri( texture_target, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
   }
 }
 
-void X3DTexture3DNode::renderSubImage( Image *image, 
-                                       GLenum texture_target, 
+void X3DTexture3DNode::renderSubImage( Image *_image, 
+                                       GLenum _texture_target, 
                                        int x_offset, 
                                        int y_offset, 
                                        int z_offset,
                                        int width, 
                                        int height,
                                        int depth ) {
-  // todo: is there a way to do this without copying data?
-  unsigned char *image_data = (unsigned char *) image->getImageData();
+  unsigned char *image_data = (unsigned char *) _image->getImageData();
 
-  unsigned int bytes_per_pixel = image->bitsPerPixel() / 8;
+  unsigned int bytes_per_pixel = _image->bitsPerPixel() / 8;
 
   unsigned char *modified_data = 
     new unsigned char[ width * height * depth * bytes_per_pixel ]; 
@@ -347,15 +402,15 @@ void X3DTexture3DNode::renderSubImage( Image *image,
     for( unsigned int y = 0; y < (unsigned int)height; ++y ) {
       memcpy( modified_data + (z * height + y ) * width * bytes_per_pixel, 
               image_data + 
-              ( ((z + z_offset) * image->height() + (y + y_offset ) ) * 
-                image->width() + x_offset) * bytes_per_pixel,
+              ( ((z + z_offset) * _image->height() + (y + y_offset ) ) * 
+                _image->width() + x_offset) * bytes_per_pixel,
               width * bytes_per_pixel );
     }
   }
 
   GLint byte_alignment;
   glGetIntegerv( GL_UNPACK_ALIGNMENT, &byte_alignment );
-  glPixelStorei( GL_UNPACK_ALIGNMENT, image->byteAlignment() );
+  glPixelStorei( GL_UNPACK_ALIGNMENT, _image->byteAlignment() );
 
   TextureProperties *texture_properties = textureProperties->getValue();
   if( texture_properties ) {
@@ -373,11 +428,11 @@ void X3DTexture3DNode::renderSubImage( Image *image,
     glPixelTransferf( GL_ALPHA_BIAS, bias.w );
   }
 
-  glTexSubImage3D( texture_target, 0, 
+  glTexSubImage3D( _texture_target, 0, 
                    x_offset, y_offset, z_offset, 
                    width, height, depth, 
-                   glPixelFormat( image ),
-                   glPixelComponentType( image ), 
+                   glPixelFormat( _image ),
+                   glPixelComponentType( _image ), 
                    modified_data );
 
   if( texture_properties ) glPopAttrib();
@@ -387,7 +442,7 @@ void X3DTexture3DNode::renderSubImage( Image *image,
 
 void X3DTexture3DNode::enableTexturing() {
  // texture 2d arrays cannot be enabled, only be used with shaders.
-  if( texture_target != GL_TEXTURE_2D_ARRAY_EXT ) 
+  if( texture_target != GL_TEXTURE_2D_ARRAY_EXT&&texture_target!=GL_TEXTURE_2D_MULTISAMPLE_ARRAY ) 
     glEnable( texture_target );
   Image * i = static_cast< Image * >(image->getValue());
   if( i ) {
@@ -419,21 +474,29 @@ void X3DTexture3DNode::disableTexturing() {
   }
 }
 
-
-GLenum X3DTexture3DNode::getTextureTarget() {
-  TextureProperties *texture_properties = textureProperties->getValue();
+void X3DTexture3DNode::UpdateTextureProperties::update(){
+  X3DTexture3DNode* t = static_cast<X3DTexture3DNode*>(getOwner());
+  TextureProperties *texture_properties = t->textureProperties->getValue();
   if( texture_properties ) {
-    const string &target_type = texture_properties->textureType->getValue();
-    if( target_type == "2D_ARRAY" ) return GL_TEXTURE_2D_ARRAY_EXT;
+    string target_type = texture_properties->textureType->getValue();
+    if( target_type == "2D_ARRAY" ) {t->setTextureTarget(GL_TEXTURE_2D_ARRAY_EXT);} 
     else if( target_type == "2D_RECTANGLE" ) {
-      Console(3) << "Warning: Invalid textureType \"2D_RECTANGLE\" in TextureProperties for "
-     << "X3DTexture2DNode. \"2D_RECTANGLE\" can only be used for 2D textures" << endl;
+      Console(LogLevel::Warning) << "Warning: Invalid textureType \"2D_RECTANGLE\" in TextureProperties for "
+        << "X3DTexture2DNode. \"2D_RECTANGLE\" can only be used for 2D textures" << endl;
     } else if( target_type != "NORMAL" ) {
-      Console(3) << "Warning: Invalid textureType: \"" << target_type << "\" in TextureProperties for "
-     << "X3DTexture3DNode. " << endl;
+      Console(LogLevel::Warning) << "Warning: Invalid textureType: \"" << target_type << "\" in TextureProperties for "
+        << "X3DTexture3DNode. " << endl;
     }
   }
-  return GL_TEXTURE_3D;
+}
+
+GLuint64 X3DTexture3DNode::getTextureHandle() {
+  // When the image is used as a bindless texture in a shader
+  // then it is possible that the image is never rendered. So we take
+  // this opportunity to ensure that the image is upToDate since otherwise
+  // changes to the image will not be handled.
+  image->upToDate();
+  return H3DSingleTextureNode::getTextureHandle();
 }
 
 std::pair<H3DInt32,H3DInt32> X3DTexture3DNode::getDefaultSaveDimensions () {
@@ -444,3 +507,53 @@ std::pair<H3DInt32,H3DInt32> X3DTexture3DNode::getDefaultSaveDimensions () {
     return X3DTextureNode::getDefaultSaveDimensions();
   }
 }
+
+GLenum X3DTexture3DNode::glPixelFormat( Image *i ) {
+  TextureProperties *texture_properties = textureProperties->getValue();
+  GLenum tex_prop_format;
+  if( texture_properties && texture_properties->glPixelFormat( i, tex_prop_format ) ) {
+    return tex_prop_format;
+  } else {
+    return H3DSingleTextureNode::glPixelFormat( i );
+  }
+}
+
+void X3DTexture3DNode::UpdateSaveToURL::onNewValue( const string &v ) {
+#ifdef HAVE_TEEM
+  X3DTexture3DNode * node = static_cast< X3DTexture3DNode * >(getOwner());
+  Image * image = node->image->getValue();
+  string file_ending = v.size() > 3 ? v.substr( v.size() - 4, 4 ) : "";
+  if( file_ending == ".exr" || file_ending == ".png" || !image ) {
+#endif
+    H3DSingleTextureNode::UpdateSaveToURL::onNewValue( v );
+#ifdef HAVE_TEEM
+  } else {
+    node->saveSuccess->setValue( H3DUtil::saveImageAsNrrdFile( v, image ) == 0, node->id );
+  }
+#endif
+}
+
+void X3DTexture3DNode::setTextureWidth( int _width ) {
+  int current_width = this->getTextureWidth();
+  if ( current_width!=_width ) {
+    H3DSingleTextureNode::setTextureWidth( _width );
+    this->textureUpdated->touch();
+  }
+}
+
+void X3DTexture3DNode::setTextureHeight( int _height ) {
+  int current_hight = getTextureHeight();
+  if ( current_hight!=_height ) {
+    H3DSingleTextureNode::setTextureHeight( _height );
+    this->textureUpdated->touch();
+  }
+}
+
+void X3DTexture3DNode::setTextureDepth( int _depth ) {
+  int current_depth = getTextureDepth();
+  if( current_depth!=_depth ) {
+    H3DSingleTextureNode::setTextureDepth( _depth );
+    this->textureUpdated->touch();
+  }
+}
+
