@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of H3D API.
 //
@@ -31,6 +31,7 @@
 #include <H3D/X3DViewpointNode.h>
 #include <H3D/ViewpointGroup.h>
 #include <H3D/H3DNavigation.h>
+#include <H3DUtil/H3DMath.h>
 
 using namespace H3D;
 
@@ -45,16 +46,16 @@ H3DNodeDatabase X3DViewpointNode::database(
                                     &X3DBindableNode::database );
 
 namespace X3DViewpointNodeInternals {
-  FIELDDB_ELEMENT( X3DViewpointNode, centerOfRotation, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( X3DViewpointNode, description, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( X3DViewpointNode, jump, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( X3DViewpointNode, orientation, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( X3DViewpointNode, position, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( X3DViewpointNode, retainUserOffsets, INPUT_OUTPUT );
-  FIELDDB_ELEMENT( X3DViewpointNode, accForwardMatrix, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( X3DViewpointNode, accInverseMatrix, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( X3DViewpointNode, totalPosition, OUTPUT_ONLY );
-  FIELDDB_ELEMENT( X3DViewpointNode, totalOrientation, OUTPUT_ONLY );
+  FIELDDB_ELEMENT( X3DViewpointNode, centerOfRotation, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( X3DViewpointNode, description, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( X3DViewpointNode, jump, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( X3DViewpointNode, orientation, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( X3DViewpointNode, position, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( X3DViewpointNode, retainUserOffsets, INPUT_OUTPUT )
+  FIELDDB_ELEMENT( X3DViewpointNode, accForwardMatrix, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( X3DViewpointNode, accInverseMatrix, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( X3DViewpointNode, totalPosition, OUTPUT_ONLY )
+  FIELDDB_ELEMENT( X3DViewpointNode, totalOrientation, OUTPUT_ONLY )
 
   inline bool intersectLinePlane( const Vec3f &from,
                                   const Vec3f &to,
@@ -348,13 +349,14 @@ void X3DViewpointNode::getProjectionDimensions( EyeMode eye_mode,
                                                 H3DFloat &bottom,
                                                 H3DFloat &right,
                                                 H3DFloat &left,
-                                                StereoInfo * stereo_info ) {
+                                                StereoInfo * stereo_info,
+                                                bool update_stereo_proj_shift ) {
   windowFromfieldOfView( width,
                          height,
                          clip_near,
                          top, bottom, right, left );
   H3DFloat frustum_shift = 0;
-  if( eye_mode == LEFT_EYE || eye_mode == RIGHT_EYE ) {
+  if( eye_mode != MONO ) {
     if( !stereo_info )
       stereo_info = StereoInfo::getActive();
     
@@ -370,21 +372,30 @@ void X3DViewpointNode::getProjectionDimensions( EyeMode eye_mode,
     if( eye_mode == RIGHT_EYE )
       frustum_shift = -frustum_shift;
   } 
-  left += frustum_shift;
-  right += frustum_shift;
+  if ( eye_mode == BOTH_EYE ) {
+    if( stereo_info&&update_stereo_proj_shift ) {
+      float new_matrix_proj_shift = 2*frustum_shift/(right-left);
+      float diff_threshold = 0.00001f;
+      if ( H3DAbs( stereo_info->matrixProjShift->getValue()-new_matrix_proj_shift )>diff_threshold ) {
+        stereo_info->updateProjShift( new_matrix_proj_shift );
+      }
+    }
+  } else {
+    left += frustum_shift;
+    right += frustum_shift;
+  }
 }
 
 void X3DViewpointNode::setupViewMatrix( EyeMode eye_mode,
-                                        StereoInfo * stereo_info  ) {
+                                        StereoInfo * stereo_info,
+                                        bool update_stereo_view_shift ) {
   const Vec3f &vp_position = totalPosition->getValue();
   const Rotation &vp_orientation = totalOrientation->getValue();
   const Matrix4f &vp_inv_m = accInverseMatrix->getValue();  
-  // OpenGL version of vp_inv_m
-  GLfloat vp_inv_transform[] = { 
-    vp_inv_m[0][0], vp_inv_m[1][0], vp_inv_m[2][0], 0,
-    vp_inv_m[0][1], vp_inv_m[1][1], vp_inv_m[2][1], 0,
-    vp_inv_m[0][2], vp_inv_m[1][2], vp_inv_m[2][2], 0,
-    vp_inv_m[0][3], vp_inv_m[1][3], vp_inv_m[2][3], 1 };
+
+  Vec3f axis = vp_orientation.axis;
+  axis.normalizeSafe();
+  Matrix4f result = Matrix4f( Rotation( vp_orientation.axis, -vp_orientation.angle ) ) * Matrix4f( -vp_position ) * vp_inv_m;
 
   if( eye_mode != MONO ) {
     // if stereo mode, viewpoint varies depending on what eye 
@@ -406,22 +417,18 @@ void X3DViewpointNode::setupViewMatrix( EyeMode eye_mode,
 
     // move to the eye position
     if( eye_mode == LEFT_EYE ) {
-      glTranslatef( -left_eye.x, -left_eye.y, -left_eye.z );
+      result = Matrix4f( -left_eye ) * result;
     } else if( eye_mode == RIGHT_EYE ) {
-      glTranslatef( left_eye.x, left_eye.y, left_eye.z );
+      result = Matrix4f( left_eye ) * result;
+    } else if( update_stereo_view_shift && (stereo_info) && ( eye_mode == BOTH_EYE ) ) {
+      if( stereo_info->matrixViewShift->getValue()!=half_interocular_distance ) {
+        stereo_info->updateViewShift( half_interocular_distance );
+      }
     }
   }
 
-  // set up view matrix based on X3DViewpointNode field values.
-  glRotatef( (H3DFloat) -(180/Constants::pi)*vp_orientation.angle, 
-             vp_orientation.axis.x, 
-             vp_orientation.axis.y,
-             vp_orientation.axis.z );
-  
-  glTranslatef( -vp_position.x,
-                -vp_position.y, 
-                -vp_position.z );
-  glMultMatrixf( vp_inv_transform );
+  // Transpose to convert to column major
+  glMultMatrixf( &result.transpose()[0][0] );
 
 }
 
@@ -438,6 +445,35 @@ Matrix4f X3DViewpointNode::getViewMatrix( EyeMode eye_mode,
                    mv[1], mv[5], mv[9], mv[13],
                    mv[2], mv[6], mv[10], mv[14],
                    mv[3], mv[7], mv[11], mv[15] );
+  glMatrixMode( GL_MODELVIEW );
+  glPopMatrix();
+  return m;
+}
+
+Matrix4f X3DViewpointNode::getProjectionMatrix( EyeMode eye_mode,
+                                          H3DFloat width, H3DFloat height,
+                                          H3DFloat clip_near, H3DFloat clip_far,
+                                          StereoInfo* stereo_info,
+                                          bool mirrored_in_y ) {
+  glMatrixMode(GL_PROJECTION);
+  glPushMatrix();
+  glLoadIdentity();
+
+  if( mirrored_in_y ) {
+    glScalef( 1, -1, 1 );
+    glFrontFace( GL_CW );
+  } else {
+    glFrontFace( GL_CCW );
+  }
+  setupProjection( eye_mode, width, height, clip_near, clip_far, stereo_info );
+  
+  GLfloat pm[16];
+  glGetFloatv( GL_PROJECTION_MATRIX, pm );
+  H3D::Matrix4f m( pm[0], pm[4], pm[8],  pm[12],
+                   pm[1], pm[5], pm[9],  pm[13],
+                   pm[2], pm[6], pm[10], pm[14],
+                   pm[3], pm[7], pm[11], pm[15] );
+  glMatrixMode(GL_PROJECTION);
   glPopMatrix();
   return m;
 }
